@@ -1,24 +1,23 @@
-/* A Bison parser, made by GNU Bison 2.3.  */
+
+/* A Bison parser, made by GNU Bison 2.4.1.  */
 
 /* Skeleton implementation for Bison's Yacc-like parsers in C
-
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
+   
+      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
    Free Software Foundation, Inc.
-
-   This program is free software; you can redistribute it and/or modify
+   
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
-
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+   
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-
+   
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* As a special exception, you may create a larger work that contains
    part or all of the Bison parser skeleton and distribute that work
@@ -29,7 +28,7 @@
    special exception, which will cause the skeleton and the resulting
    Bison output files to be licensed under the GNU General Public
    License without this special exception.
-
+   
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
@@ -47,7 +46,7 @@
 #define YYBISON 1
 
 /* Bison version.  */
-#define YYBISON_VERSION "2.3"
+#define YYBISON_VERSION "2.4.1"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -55,71 +54,28 @@
 /* Pure parsers.  */
 #define YYPURE 1
 
+/* Push parsers.  */
+#define YYPUSH 0
+
+/* Pull parsers.  */
+#define YYPULL 1
+
 /* Using locations.  */
 #define YYLSP_NEEDED 0
 
 /* Substitute the variable and function names.  */
-#define yyparse raptor_parsedate_parse
-#define yylex   raptor_parsedate_lex
-#define yyerror raptor_parsedate_error
-#define yylval  raptor_parsedate_lval
-#define yychar  raptor_parsedate_char
-#define yydebug raptor_parsedate_debug
-#define yynerrs raptor_parsedate_nerrs
-
-
-/* Tokens.  */
-#ifndef YYTOKENTYPE
-# define YYTOKENTYPE
-   /* Put the tokens into the symbol table, so that GDB and other debuggers
-      know about them.  */
-   enum yytokentype {
-     tAGO = 258,
-     tDAY = 259,
-     tDAY_UNIT = 260,
-     tDAYZONE = 261,
-     tDST = 262,
-     tHOUR_UNIT = 263,
-     tID = 264,
-     tTZONE = 265,
-     tWZONE = 266,
-     tZZONE = 267,
-     tMERIDIAN = 268,
-     tMINUTE_UNIT = 269,
-     tMONTH = 270,
-     tMONTH_UNIT = 271,
-     tSEC_UNIT = 272,
-     tSNUMBER = 273,
-     tUNUMBER = 274,
-     tYEAR_UNIT = 275,
-     tZONE = 276
-   };
-#endif
-/* Tokens.  */
-#define tAGO 258
-#define tDAY 259
-#define tDAY_UNIT 260
-#define tDAYZONE 261
-#define tDST 262
-#define tHOUR_UNIT 263
-#define tID 264
-#define tTZONE 265
-#define tWZONE 266
-#define tZZONE 267
-#define tMERIDIAN 268
-#define tMINUTE_UNIT 269
-#define tMONTH 270
-#define tMONTH_UNIT 271
-#define tSEC_UNIT 272
-#define tSNUMBER 273
-#define tUNUMBER 274
-#define tYEAR_UNIT 275
-#define tZONE 276
-
-
+#define yyparse         raptor_parsedate_parse
+#define yylex           raptor_parsedate_lex
+#define yyerror         raptor_parsedate_error
+#define yylval          raptor_parsedate_lval
+#define yychar          raptor_parsedate_char
+#define yydebug         raptor_parsedate_debug
+#define yynerrs         raptor_parsedate_nerrs
 
 
 /* Copy the first part of user declarations.  */
+
+/* Line 189 of yacc.c  */
 #line 1 "./parsedate.y"
 
 /*
@@ -285,6 +241,9 @@ static int LookupWord (YYSTYPE *lvalp, char *buff);
 
 
 
+/* Line 189 of yacc.c  */
+#line 246 "parsedate.c"
+
 /* Enabling traces.  */
 #ifndef YYDEBUG
 # define YYDEBUG 0
@@ -303,20 +262,70 @@ static int LookupWord (YYSTYPE *lvalp, char *buff);
 # define YYTOKEN_TABLE 0
 #endif
 
+
+/* Tokens.  */
+#ifndef YYTOKENTYPE
+# define YYTOKENTYPE
+   /* Put the tokens into the symbol table, so that GDB and other debuggers
+      know about them.  */
+   enum yytokentype {
+     tAGO = 258,
+     tDAY = 259,
+     tDAY_UNIT = 260,
+     tDAYZONE = 261,
+     tDST = 262,
+     tHOUR_UNIT = 263,
+     tID = 264,
+     tTZONE = 265,
+     tWZONE = 266,
+     tZZONE = 267,
+     tMERIDIAN = 268,
+     tMINUTE_UNIT = 269,
+     tMONTH = 270,
+     tMONTH_UNIT = 271,
+     tSEC_UNIT = 272,
+     tSNUMBER = 273,
+     tUNUMBER = 274,
+     tYEAR_UNIT = 275,
+     tZONE = 276
+   };
+#endif
+/* Tokens.  */
+#define tAGO 258
+#define tDAY 259
+#define tDAY_UNIT 260
+#define tDAYZONE 261
+#define tDST 262
+#define tHOUR_UNIT 263
+#define tID 264
+#define tTZONE 265
+#define tWZONE 266
+#define tZZONE 267
+#define tMERIDIAN 268
+#define tMINUTE_UNIT 269
+#define tMONTH 270
+#define tMONTH_UNIT 271
+#define tSEC_UNIT 272
+#define tSNUMBER 273
+#define tUNUMBER 274
+#define tYEAR_UNIT 275
+#define tZONE 276
+
+
+
+
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
-typedef int YYSTYPE;
+
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
-# define YYSTYPE_IS_TRIVIAL 1
 #endif
 
 
-
 /* Copy the second part of user declarations.  */
 
 
-/* Line 216 of yacc.c.  */
-#line 320 "parsedate.c"
+/* Line 264 of yacc.c  */
+#line 329 "parsedate.c"
 
 #ifdef short
 # undef short
@@ -391,14 +400,14 @@ typedef short int yytype_int16;
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 static int
-YYID (int i)
+YYID (int yyi)
 #else
 static int
-YYID (i)
-    int i;
+YYID (yyi)
+    int yyi;
 #endif
 {
-  return i;
+  return yyi;
 }
 #endif
 
@@ -479,9 +488,9 @@ void free (void *); /* INFRINGES ON USER NAME SPACE */
 /* A type that is properly aligned for any stack member.  */
 union yyalloc
 {
-  yytype_int16 yyss;
-  YYSTYPE yyvs;
-  };
+  yytype_int16 yyss_alloc;
+  YYSTYPE yyvs_alloc;
+};
 
 /* The size of the maximum gap between one aligned stack and the next.  */
 # define YYSTACK_GAP_MAXIMUM (sizeof (union yyalloc) - 1)
@@ -515,12 +524,12 @@ union yyalloc
    elements in the stack, and YYPTR gives the new location of the
    stack.  Advance YYPTR to a properly aligned location for the next
    stack.  */
-# define YYSTACK_RELOCATE(Stack)					\
+# define YYSTACK_RELOCATE(Stack_alloc, Stack)				\
     do									\
       {									\
 	YYSIZE_T yynewbytes;						\
-	YYCOPY (&yyptr->Stack, Stack, yysize);				\
-	Stack = &yyptr->Stack;						\
+	YYCOPY (&yyptr->Stack_alloc, Stack, yysize);			\
+	Stack = &yyptr->Stack_alloc;					\
 	yynewbytes = yystacksize * sizeof (*Stack) + YYSTACK_GAP_MAXIMUM; \
 	yyptr += yynewbytes / sizeof (*yyptr);				\
       }									\
@@ -978,17 +987,20 @@ yy_symbol_print (yyoutput, yytype, yyvaluep)
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 static void
-yy_stack_print (yytype_int16 *bottom, yytype_int16 *top)
+yy_stack_print (yytype_int16 *yybottom, yytype_int16 *yytop)
 #else
 static void
-yy_stack_print (bottom, top)
-    yytype_int16 *bottom;
-    yytype_int16 *top;
+yy_stack_print (yybottom, yytop)
+    yytype_int16 *yybottom;
+    yytype_int16 *yytop;
 #endif
 {
   YYFPRINTF (stderr, "Stack now");
-  for (; bottom <= top; ++bottom)
-    YYFPRINTF (stderr, " %d", *bottom);
+  for (; yybottom <= yytop; yybottom++)
+    {
+      int yybot = *yybottom;
+      YYFPRINTF (stderr, " %d", yybot);
+    }
   YYFPRINTF (stderr, "\n");
 }
 
@@ -1022,11 +1034,11 @@ yy_reduce_print (yyvsp, yyrule)
   /* The symbols being reduced.  */
   for (yyi = 0; yyi < yynrhs; yyi++)
     {
-      fprintf (stderr, "   $%d = ", yyi + 1);
+      YYFPRINTF (stderr, "   $%d = ", yyi + 1);
       yy_symbol_print (stderr, yyrhs[yyprhs[yyrule] + yyi],
 		       &(yyvsp[(yyi + 1) - (yynrhs)])
 		       		       );
-      fprintf (stderr, "\n");
+      YYFPRINTF (stderr, "\n");
     }
 }
 
@@ -1306,10 +1318,8 @@ yydestruct (yymsg, yytype, yyvaluep)
 	break;
     }
 }
-
 
 /* Prevent warnings from -Wmissing-prototypes.  */
-
 #ifdef YYPARSE_PARAM
 #if defined __STDC__ || defined __cplusplus
 int yyparse (void *YYPARSE_PARAM);
@@ -1328,10 +1338,9 @@ int yyparse ();
 
 
 
-
-/*----------.
-| yyparse.  |
-`----------*/
+/*-------------------------.
+| yyparse or yypush_parse.  |
+`-------------------------*/
 
 #ifdef YYPARSE_PARAM
 #if (defined __STDC__ || defined __C99__FUNC__ \
@@ -1355,22 +1364,46 @@ yyparse ()
 #endif
 #endif
 {
-  /* The look-ahead symbol.  */
+/* The lookahead symbol.  */
 int yychar;
 
-/* The semantic value of the look-ahead symbol.  */
+/* The semantic value of the lookahead symbol.  */
 YYSTYPE yylval;
 
-/* Number of syntax errors so far.  */
-int yynerrs;
+    /* Number of syntax errors so far.  */
+    int yynerrs;
+
+    int yystate;
+    /* Number of tokens to shift before error messages enabled.  */
+    int yyerrstatus;
+
+    /* The stacks and their tools:
+       `yyss': related to states.
+       `yyvs': related to semantic values.
+
+       Refer to the stacks thru separate pointers, to allow yyoverflow
+       to reallocate them elsewhere.  */
+
+    /* The state stack.  */
+    yytype_int16 yyssa[YYINITDEPTH];
+    yytype_int16 *yyss;
+    yytype_int16 *yyssp;
+
+    /* The semantic value stack.  */
+    YYSTYPE yyvsa[YYINITDEPTH];
+    YYSTYPE *yyvs;
+    YYSTYPE *yyvsp;
+
+    YYSIZE_T yystacksize;
 
-  int yystate;
   int yyn;
   int yyresult;
-  /* Number of tokens to shift before error messages enabled.  */
-  int yyerrstatus;
-  /* Look-ahead token as an internal (translated) token number.  */
-  int yytoken = 0;
+  /* Lookahead token as an internal (translated) token number.  */
+  int yytoken;
+  /* The variables used to return semantic value and location from the
+     action routines.  */
+  YYSTYPE yyval;
+
 #if YYERROR_VERBOSE
   /* Buffer for error messages, and its allocated size.  */
   char yymsgbuf[128];
@@ -1378,51 +1411,28 @@ int yynerrs;
   YYSIZE_T yymsg_alloc = sizeof yymsgbuf;
 #endif
 
-  /* Three stacks and their tools:
-     `yyss': related to states,
-     `yyvs': related to semantic values,
-     `yyls': related to locations.
-
-     Refer to the stacks thru separate pointers, to allow yyoverflow
-     to reallocate them elsewhere.  */
-
-  /* The state stack.  */
-  yytype_int16 yyssa[YYINITDEPTH];
-  yytype_int16 *yyss = yyssa;
-  yytype_int16 *yyssp;
-
-  /* The semantic value stack.  */
-  YYSTYPE yyvsa[YYINITDEPTH];
-  YYSTYPE *yyvs = yyvsa;
-  YYSTYPE *yyvsp;
-
-
-
 #define YYPOPSTACK(N)   (yyvsp -= (N), yyssp -= (N))
 
-  YYSIZE_T yystacksize = YYINITDEPTH;
-
-  /* The variables used to return semantic value and location from the
-     action routines.  */
-  YYSTYPE yyval;
-
-
   /* The number of symbols on the RHS of the reduced rule.
      Keep to zero when no symbol should be popped.  */
   int yylen = 0;
 
+  yytoken = 0;
+  yyss = yyssa;
+  yyvs = yyvsa;
+  yystacksize = YYINITDEPTH;
+
   YYDPRINTF ((stderr, "Starting parse\n"));
 
   yystate = 0;
   yyerrstatus = 0;
   yynerrs = 0;
-  yychar = YYEMPTY;		/* Cause a token to be read.  */
+  yychar = YYEMPTY; /* Cause a token to be read.  */
 
   /* Initialize stack pointers.
      Waste one element of value and location stack
      so that they stay on the same level as the state stack.
      The wasted elements are never initialized.  */
-
   yyssp = yyss;
   yyvsp = yyvs;
 
@@ -1452,7 +1462,6 @@ int yynerrs;
 	YYSTYPE *yyvs1 = yyvs;
 	yytype_int16 *yyss1 = yyss;
 
-
 	/* Each stack pointer address is followed by the size of the
 	   data in use in that stack, in bytes.  This used to be a
 	   conditional around just the two extra args, but that might
@@ -1460,7 +1469,6 @@ int yynerrs;
 	yyoverflow (YY_("memory exhausted"),
 		    &yyss1, yysize * sizeof (*yyssp),
 		    &yyvs1, yysize * sizeof (*yyvsp),
-
 		    &yystacksize);
 
 	yyss = yyss1;
@@ -1483,9 +1491,8 @@ int yynerrs;
 	  (union yyalloc *) YYSTACK_ALLOC (YYSTACK_BYTES (yystacksize));
 	if (! yyptr)
 	  goto yyexhaustedlab;
-	YYSTACK_RELOCATE (yyss);
-	YYSTACK_RELOCATE (yyvs);
-
+	YYSTACK_RELOCATE (yyss_alloc, yyss);
+	YYSTACK_RELOCATE (yyvs_alloc, yyvs);
 #  undef YYSTACK_RELOCATE
 	if (yyss1 != yyssa)
 	  YYSTACK_FREE (yyss1);
@@ -1496,7 +1503,6 @@ int yynerrs;
       yyssp = yyss + yysize - 1;
       yyvsp = yyvs + yysize - 1;
 
-
       YYDPRINTF ((stderr, "Stack size increased to %lu\n",
 		  (unsigned long int) yystacksize));
 
@@ -1506,6 +1512,9 @@ int yynerrs;
 
   YYDPRINTF ((stderr, "Entering state %d\n", yystate));
 
+  if (yystate == YYFINAL)
+    YYACCEPT;
+
   goto yybackup;
 
 /*-----------.
@@ -1514,16 +1523,16 @@ int yynerrs;
 yybackup:
 
   /* Do appropriate processing given the current state.  Read a
-     look-ahead token if we need one and don't already have one.  */
+     lookahead token if we need one and don't already have one.  */
 
-  /* First try to decide what to do without reference to look-ahead token.  */
+  /* First try to decide what to do without reference to lookahead token.  */
   yyn = yypact[yystate];
   if (yyn == YYPACT_NINF)
     goto yydefault;
 
-  /* Not known => get a look-ahead token if don't already have one.  */
+  /* Not known => get a lookahead token if don't already have one.  */
 
-  /* YYCHAR is either YYEMPTY or YYEOF or a valid look-ahead symbol.  */
+  /* YYCHAR is either YYEMPTY or YYEOF or a valid lookahead symbol.  */
   if (yychar == YYEMPTY)
     {
       YYDPRINTF ((stderr, "Reading a token: "));
@@ -1555,20 +1564,16 @@ yybackup:
       goto yyreduce;
     }
 
-  if (yyn == YYFINAL)
-    YYACCEPT;
-
   /* Count tokens shifted since error; after three, turn off error
      status.  */
   if (yyerrstatus)
     yyerrstatus--;
 
-  /* Shift the look-ahead token.  */
+  /* Shift the lookahead token.  */
   YY_SYMBOL_PRINT ("Shifting", yytoken, &yylval, &yylloc);
 
-  /* Discard the shifted token unless it is eof.  */
-  if (yychar != YYEOF)
-    yychar = YYEMPTY;
+  /* Discard the shifted token.  */
+  yychar = YYEMPTY;
 
   yystate = yyn;
   *++yyvsp = yylval;
@@ -1608,6 +1613,8 @@ yyreduce:
   switch (yyn)
     {
         case 4:
+
+/* Line 1455 of yacc.c  */
 #line 184 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyHaveTime++;
@@ -1615,6 +1622,8 @@ yyreduce:
     break;
 
   case 5:
+
+/* Line 1455 of yacc.c  */
 #line 187 "./parsedate.y"
     {
 	        ((struct date_yy *)parm)->yyHaveZone++;
@@ -1622,6 +1631,8 @@ yyreduce:
     break;
 
   case 6:
+
+/* Line 1455 of yacc.c  */
 #line 190 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyHaveDate++;
@@ -1629,6 +1640,8 @@ yyreduce:
     break;
 
   case 7:
+
+/* Line 1455 of yacc.c  */
 #line 193 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyHaveDay++;
@@ -1636,6 +1649,8 @@ yyreduce:
     break;
 
   case 8:
+
+/* Line 1455 of yacc.c  */
 #line 196 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyHaveRel++;
@@ -1643,6 +1658,8 @@ yyreduce:
     break;
 
   case 11:
+
+/* Line 1455 of yacc.c  */
 #line 203 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyHour = (yyvsp[(1) - (2)].Number);
@@ -1653,6 +1670,8 @@ yyreduce:
     break;
 
   case 13:
+
+/* Line 1455 of yacc.c  */
 #line 213 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyMeridian = MER24;
@@ -1660,6 +1679,8 @@ yyreduce:
     break;
 
   case 14:
+
+/* Line 1455 of yacc.c  */
 #line 216 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyMeridian = MER24;
@@ -1668,6 +1689,8 @@ yyreduce:
     break;
 
   case 15:
+
+/* Line 1455 of yacc.c  */
 #line 220 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyMeridian = MER24;
@@ -1675,6 +1698,8 @@ yyreduce:
     break;
 
   case 16:
+
+/* Line 1455 of yacc.c  */
 #line 223 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyMeridian = MER24;
@@ -1683,6 +1708,8 @@ yyreduce:
     break;
 
   case 17:
+
+/* Line 1455 of yacc.c  */
 #line 229 "./parsedate.y"
     {
 		((struct date_yy *)parm)->yyHaveZone++;
@@ -1690,6 +1717,8 @@ yyreduce:
     break;
 
   case 18:
+
+/* Line 1455 of yacc.c  */
 #line 232 "./parsedate.y"
     {
 		((struct date_yy *)parm)->yyHaveZone++;
@@ -1697,6 +1726,8 @@ yyreduce:
     break;
 
   case 19:
+
+/* Line 1455 of yacc.c  */
 #line 235 "./parsedate.y"
     {
 		((struct date_yy *)parm)->yyHaveZone++;
@@ -1704,12 +1735,16 @@ yyreduce:
     break;
 
   case 21:
+
+/* Line 1455 of yacc.c  */
 #line 241 "./parsedate.y"
     {
 	}
     break;
 
   case 23:
+
+/* Line 1455 of yacc.c  */
 #line 246 "./parsedate.y"
     {
 		/* format: [+-]hhmm */
@@ -1722,6 +1757,8 @@ yyreduce:
     break;
 
   case 24:
+
+/* Line 1455 of yacc.c  */
 #line 256 "./parsedate.y"
     {
 		/* format: [+-]hh:mm */
@@ -1730,6 +1767,8 @@ yyreduce:
     break;
 
   case 25:
+
+/* Line 1455 of yacc.c  */
 #line 262 "./parsedate.y"
     {
 		/* format: hh:mm:ss */
@@ -1738,6 +1777,8 @@ yyreduce:
     break;
 
   case 26:
+
+/* Line 1455 of yacc.c  */
 #line 268 "./parsedate.y"
     {
 		/* format: hh:mm */
@@ -1747,6 +1788,8 @@ yyreduce:
     break;
 
   case 27:
+
+/* Line 1455 of yacc.c  */
 #line 284 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyTimezone = (yyvsp[(1) - (1)].Number);
@@ -1754,6 +1797,8 @@ yyreduce:
     break;
 
   case 28:
+
+/* Line 1455 of yacc.c  */
 #line 287 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyTimezone = (yyvsp[(1) - (1)].Number);
@@ -1761,6 +1806,8 @@ yyreduce:
     break;
 
   case 29:
+
+/* Line 1455 of yacc.c  */
 #line 290 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyTimezone = (yyvsp[(1) - (1)].Number);
@@ -1768,6 +1815,8 @@ yyreduce:
     break;
 
   case 30:
+
+/* Line 1455 of yacc.c  */
 #line 293 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyTimezone = (yyvsp[(1) - (1)].Number);
@@ -1775,6 +1824,8 @@ yyreduce:
     break;
 
   case 31:
+
+/* Line 1455 of yacc.c  */
 #line 296 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyTimezone = (yyvsp[(1) - (1)].Number) - 60;
@@ -1782,6 +1833,8 @@ yyreduce:
     break;
 
   case 32:
+
+/* Line 1455 of yacc.c  */
 #line 299 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyTimezone = (yyvsp[(1) - (2)].Number) - 60;
@@ -1789,6 +1842,8 @@ yyreduce:
     break;
 
   case 33:
+
+/* Line 1455 of yacc.c  */
 #line 304 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyDayOrdinal = 1;
@@ -1797,6 +1852,8 @@ yyreduce:
     break;
 
   case 34:
+
+/* Line 1455 of yacc.c  */
 #line 308 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyDayOrdinal = 1;
@@ -1805,6 +1862,8 @@ yyreduce:
     break;
 
   case 35:
+
+/* Line 1455 of yacc.c  */
 #line 312 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyDayOrdinal = (yyvsp[(1) - (2)].Number);
@@ -1813,6 +1872,8 @@ yyreduce:
     break;
 
   case 36:
+
+/* Line 1455 of yacc.c  */
 #line 318 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyMonth = (yyvsp[(1) - (3)].Number);
@@ -1821,6 +1882,8 @@ yyreduce:
     break;
 
   case 37:
+
+/* Line 1455 of yacc.c  */
 #line 322 "./parsedate.y"
     {
 		((struct date_yy *)parm)->yyYear = (yyvsp[(8) - (8)].Number);
@@ -1836,6 +1899,8 @@ yyreduce:
     break;
 
   case 38:
+
+/* Line 1455 of yacc.c  */
 #line 333 "./parsedate.y"
     {
 	  /* Interpret as YYYY/MM/DD if $1 >= 1000, otherwise as MM/DD/YY.
@@ -1858,6 +1923,8 @@ yyreduce:
     break;
 
   case 40:
+
+/* Line 1455 of yacc.c  */
 #line 352 "./parsedate.y"
     {
 			((struct date_yy *)parm)->yyHaveTime++;
@@ -1865,6 +1932,8 @@ yyreduce:
     break;
 
   case 41:
+
+/* Line 1455 of yacc.c  */
 #line 355 "./parsedate.y"
     {
 	    /* e.g. 17-JUN-1992.  */
@@ -1875,6 +1944,8 @@ yyreduce:
     break;
 
   case 42:
+
+/* Line 1455 of yacc.c  */
 #line 361 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyMonth = (yyvsp[(1) - (3)].Number);
@@ -1884,6 +1955,8 @@ yyreduce:
     break;
 
   case 43:
+
+/* Line 1455 of yacc.c  */
 #line 366 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyMonth = (yyvsp[(1) - (2)].Number);
@@ -1896,6 +1969,8 @@ yyreduce:
     break;
 
   case 44:
+
+/* Line 1455 of yacc.c  */
 #line 374 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyMonth = (yyvsp[(1) - (4)].Number);
@@ -1905,6 +1980,8 @@ yyreduce:
     break;
 
   case 45:
+
+/* Line 1455 of yacc.c  */
 #line 379 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyMonth = (yyvsp[(2) - (2)].Number);
@@ -1917,6 +1994,8 @@ yyreduce:
     break;
 
   case 46:
+
+/* Line 1455 of yacc.c  */
 #line 387 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyMonth = (yyvsp[(2) - (3)].Number);
@@ -1926,6 +2005,8 @@ yyreduce:
     break;
 
   case 48:
+
+/* Line 1455 of yacc.c  */
 #line 395 "./parsedate.y"
     {
 		int i = (yyvsp[(1) - (3)].Number);
@@ -1947,6 +2028,8 @@ yyreduce:
     break;
 
   case 49:
+
+/* Line 1455 of yacc.c  */
 #line 414 "./parsedate.y"
     {
 	    /* ISO 8601 format.  yyyy-mm-dd.  */
@@ -1957,6 +2040,8 @@ yyreduce:
     break;
 
   case 50:
+
+/* Line 1455 of yacc.c  */
 #line 420 "./parsedate.y"
     {
 		/* ISO 8601 format   yyyy-mm */
@@ -1967,6 +2052,8 @@ yyreduce:
     break;
 
   case 51:
+
+/* Line 1455 of yacc.c  */
 #line 426 "./parsedate.y"
     {
 		const int om = (1 + 9) % 12; /* offset month */
@@ -1981,6 +2068,8 @@ yyreduce:
     break;
 
   case 52:
+
+/* Line 1455 of yacc.c  */
 #line 438 "./parsedate.y"
     {
 		((struct date_yy *)parm)->yyDay = ((yyvsp[(2) - (2)].Number) / 10) * 7 + ((yyvsp[(2) - (2)].Number) % 10) - 8;
@@ -1988,6 +2077,8 @@ yyreduce:
     break;
 
   case 53:
+
+/* Line 1455 of yacc.c  */
 #line 441 "./parsedate.y"
     {
 		((struct date_yy *)parm)->yyDay = (yyvsp[(2) - (3)].Number) * 7 - (yyvsp[(3) - (3)].Number) - 8;
@@ -1995,6 +2086,8 @@ yyreduce:
     break;
 
   case 55:
+
+/* Line 1455 of yacc.c  */
 #line 448 "./parsedate.y"
     {
 		int i = (yyvsp[(1) - (3)].Number);
@@ -2022,6 +2115,8 @@ yyreduce:
     break;
 
   case 56:
+
+/* Line 1455 of yacc.c  */
 #line 473 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyRelSeconds =
@@ -2040,6 +2135,8 @@ yyreduce:
     break;
 
   case 58:
+
+/* Line 1455 of yacc.c  */
 #line 490 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyRelYear += (yyvsp[(1) - (2)].Number) * (yyvsp[(2) - (2)].Number);
@@ -2047,6 +2144,8 @@ yyreduce:
     break;
 
   case 59:
+
+/* Line 1455 of yacc.c  */
 #line 493 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyRelYear += (yyvsp[(1) - (2)].Number) * (yyvsp[(2) - (2)].Number);
@@ -2054,6 +2153,8 @@ yyreduce:
     break;
 
   case 60:
+
+/* Line 1455 of yacc.c  */
 #line 496 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyRelYear += (yyvsp[(1) - (1)].Number);
@@ -2061,6 +2162,8 @@ yyreduce:
     break;
 
   case 61:
+
+/* Line 1455 of yacc.c  */
 #line 499 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyRelMonth += (yyvsp[(1) - (2)].Number) * (yyvsp[(2) - (2)].Number);
@@ -2068,6 +2171,8 @@ yyreduce:
     break;
 
   case 62:
+
+/* Line 1455 of yacc.c  */
 #line 502 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyRelMonth += (yyvsp[(1) - (2)].Number) * (yyvsp[(2) - (2)].Number);
@@ -2075,6 +2180,8 @@ yyreduce:
     break;
 
   case 63:
+
+/* Line 1455 of yacc.c  */
 #line 505 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyRelMonth += (yyvsp[(1) - (1)].Number);
@@ -2082,6 +2189,8 @@ yyreduce:
     break;
 
   case 64:
+
+/* Line 1455 of yacc.c  */
 #line 508 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyRelDay += (yyvsp[(1) - (2)].Number) * (yyvsp[(2) - (2)].Number);
@@ -2089,6 +2198,8 @@ yyreduce:
     break;
 
   case 65:
+
+/* Line 1455 of yacc.c  */
 #line 511 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyRelDay += (yyvsp[(1) - (2)].Number) * (yyvsp[(2) - (2)].Number);
@@ -2096,6 +2207,8 @@ yyreduce:
     break;
 
   case 66:
+
+/* Line 1455 of yacc.c  */
 #line 514 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyRelDay += (yyvsp[(1) - (1)].Number);
@@ -2103,6 +2216,8 @@ yyreduce:
     break;
 
   case 67:
+
+/* Line 1455 of yacc.c  */
 #line 517 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyRelHour += (yyvsp[(1) - (2)].Number) * (yyvsp[(2) - (2)].Number);
@@ -2110,6 +2225,8 @@ yyreduce:
     break;
 
   case 68:
+
+/* Line 1455 of yacc.c  */
 #line 520 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyRelHour += (yyvsp[(1) - (2)].Number) * (yyvsp[(2) - (2)].Number);
@@ -2117,6 +2234,8 @@ yyreduce:
     break;
 
   case 69:
+
+/* Line 1455 of yacc.c  */
 #line 523 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyRelHour += (yyvsp[(1) - (1)].Number);
@@ -2124,6 +2243,8 @@ yyreduce:
     break;
 
   case 70:
+
+/* Line 1455 of yacc.c  */
 #line 526 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyRelMinutes += (yyvsp[(1) - (2)].Number) * (yyvsp[(2) - (2)].Number);
@@ -2131,6 +2252,8 @@ yyreduce:
     break;
 
   case 71:
+
+/* Line 1455 of yacc.c  */
 #line 529 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyRelMinutes += (yyvsp[(1) - (2)].Number) * (yyvsp[(2) - (2)].Number);
@@ -2138,6 +2261,8 @@ yyreduce:
     break;
 
   case 72:
+
+/* Line 1455 of yacc.c  */
 #line 532 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyRelMinutes += (yyvsp[(1) - (1)].Number);
@@ -2145,6 +2270,8 @@ yyreduce:
     break;
 
   case 73:
+
+/* Line 1455 of yacc.c  */
 #line 535 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyRelSeconds += (yyvsp[(1) - (2)].Number) * (yyvsp[(2) - (2)].Number);
@@ -2152,6 +2279,8 @@ yyreduce:
     break;
 
   case 74:
+
+/* Line 1455 of yacc.c  */
 #line 538 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyRelSeconds += (yyvsp[(1) - (2)].Number) * (yyvsp[(2) - (2)].Number);
@@ -2159,6 +2288,8 @@ yyreduce:
     break;
 
   case 75:
+
+/* Line 1455 of yacc.c  */
 #line 541 "./parsedate.y"
     {
 	    ((struct date_yy *)parm)->yyRelSeconds += (yyvsp[(1) - (1)].Number);
@@ -2166,6 +2297,8 @@ yyreduce:
     break;
 
   case 76:
+
+/* Line 1455 of yacc.c  */
 #line 547 "./parsedate.y"
     {
 	    if (((struct date_yy *)parm)->yyHaveTime && 
@@ -2202,6 +2335,8 @@ yyreduce:
     break;
 
   case 77:
+
+/* Line 1455 of yacc.c  */
 #line 582 "./parsedate.y"
     {
 			 ((struct date_yy *)parm)->yyMeridian = (yyvsp[(1) - (1)].Meridian);
@@ -2209,8 +2344,9 @@ yyreduce:
     break;
 
 
-/* Line 1267 of yacc.c.  */
-#line 2214 "parsedate.c"
+
+/* Line 1455 of yacc.c  */
+#line 2350 "parsedate.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -2221,7 +2357,6 @@ yyreduce:
 
   *++yyvsp = yyval;
 
-
   /* Now `shift' the result of the reduction.  Determine what state
      that goes to, based on the state we popped back to and the rule
      number reduced by.  */
@@ -2286,7 +2421,7 @@ yyerrlab:
 
   if (yyerrstatus == 3)
     {
-      /* If just tried and failed to reuse look-ahead token after an
+      /* If just tried and failed to reuse lookahead token after an
 	 error, discard it.  */
 
       if (yychar <= YYEOF)
@@ -2303,7 +2438,7 @@ yyerrlab:
 	}
     }
 
-  /* Else will try to reuse look-ahead token after shifting the error
+  /* Else will try to reuse lookahead token after shifting the error
      token.  */
   goto yyerrlab1;
 
@@ -2360,9 +2495,6 @@ yyerrlab1:
       YY_STACK_PRINT (yyss, yyssp);
     }
 
-  if (yyn == YYFINAL)
-    YYACCEPT;
-
   *++yyvsp = yylval;
 
 
@@ -2387,7 +2519,7 @@ yyabortlab:
   yyresult = 1;
   goto yyreturn;
 
-#ifndef yyoverflow
+#if !defined(yyoverflow) || YYERROR_VERBOSE
 /*-------------------------------------------------.
 | yyexhaustedlab -- memory exhaustion comes here.  |
 `-------------------------------------------------*/
@@ -2398,7 +2530,7 @@ yyexhaustedlab:
 #endif
 
 yyreturn:
-  if (yychar != YYEOF && yychar != YYEMPTY)
+  if (yychar != YYEMPTY)
      yydestruct ("Cleanup: discarding lookahead",
 		 yytoken, &yylval);
   /* Do not reclaim the symbols of the rule which action triggered
@@ -2424,6 +2556,8 @@ yyreturn:
 }
 
 
+
+/* Line 1675 of yacc.c  */
 #line 587 "./parsedate.y"
 
 
@@ -2648,7 +2782,11 @@ ToHour(int Hours, MERIDIAN Meridian)
 	Hours = 0;
       return Hours + 12;
     default:
-      abort ();
+#ifdef RAPTOR_DEBUG
+      fprintf(stderr, "%s:%d:%s: UNKNOWN Meridian %d - add a new case", 
+              __FILE__, __LINE__, __func__, (int)Meridian);
+#endif
+      return -1;
     }
   /* NOTREACHED */
 }
