@@ -2,7 +2,7 @@
  *
  * raptor_libxml.c - Raptor libxml functions
  *
- * Copyright (C) 2000-2007, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2000-2009, David Beckett http://www.dajobe.org/
  * Copyright (C) 2000-2004, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -55,16 +55,15 @@ static void raptor_libxml_warning(void* user_data, const char *msg, ...) RAPTOR_
 static void raptor_libxml_error_common(void* user_data, const char *msg, va_list args,  const char *prefix, int is_fatal) RAPTOR_PRINTF_FORMAT(2, 0);
 static void raptor_libxml_error(void *context, const char *msg, ...) RAPTOR_PRINTF_FORMAT(2, 3);
 static void raptor_libxml_fatal_error(void *context, const char *msg, ...) RAPTOR_PRINTF_FORMAT(2, 3);
-static void raptor_libxml_generic_error(void* user_data, const char *msg, ...) RAPTOR_PRINTF_FORMAT(2, 3);
 
 
 
-static const char* xml_warning_prefix="XML parser warning - ";
-static const char* xml_error_prefix="XML parser error - ";
-static const char* xml_generic_error_prefix="XML error - ";
-static const char* xml_fatal_error_prefix="XML parser fatal error - ";
-static const char* xml_validation_error_prefix="XML parser validation error - ";
-static const char* xml_validation_warning_prefix="XML parser validation warning - ";
+static const char* const xml_warning_prefix="XML parser warning - ";
+static const char* const xml_error_prefix="XML parser error - ";
+static const char* const xml_generic_error_prefix="XML error - ";
+static const char* const xml_fatal_error_prefix="XML parser fatal error - ";
+static const char* const xml_validation_error_prefix="XML parser validation error - ";
+static const char* const xml_validation_warning_prefix="XML parser validation warning - ";
 
 
 #ifdef HAVE_XMLSAX2INTERNALSUBSET
@@ -271,9 +270,10 @@ raptor_libxml_warning(void* user_data, const char *msg, ...)
       nmsg[length-2]='\0';
   }
   
-  raptor_log_error_varargs(RAPTOR_LOG_LEVEL_ERROR,
-                           sax2->error_handlers->warning_handler, 
-                           sax2->error_handlers->warning_user_data,
+  raptor_log_error_varargs(sax2->world,
+                           RAPTOR_LOG_LEVEL_WARNING,
+                           sax2->error_handlers->handlers[RAPTOR_LOG_LEVEL_WARNING].handler,
+                           sax2->error_handlers->handlers[RAPTOR_LOG_LEVEL_WARNING].user_data,
                            sax2->locator, 
                            nmsg ? nmsg : msg, 
                            args);
@@ -316,16 +316,18 @@ raptor_libxml_error_common(void* user_data, const char *msg, va_list args,
   }
 
   if(is_fatal)
-    raptor_log_error_varargs(RAPTOR_LOG_LEVEL_FATAL,
-                             sax2->error_handlers->fatal_error_handler, 
-                             sax2->error_handlers->fatal_error_user_data,
+    raptor_log_error_varargs(sax2->world,
+                             RAPTOR_LOG_LEVEL_FATAL,
+                             sax2->error_handlers->handlers[RAPTOR_LOG_LEVEL_FATAL].handler, 
+                             sax2->error_handlers->handlers[RAPTOR_LOG_LEVEL_FATAL].user_data,
                              sax2->locator, 
                              nmsg ? nmsg : msg, 
                              args);
   else
-    raptor_log_error_varargs(RAPTOR_LOG_LEVEL_ERROR,
-                             sax2->error_handlers->error_handler, 
-                             sax2->error_handlers->error_user_data,
+    raptor_log_error_varargs(sax2->world,
+                             RAPTOR_LOG_LEVEL_ERROR,
+                             sax2->error_handlers->handlers[RAPTOR_LOG_LEVEL_ERROR].handler,
+                             sax2->error_handlers->handlers[RAPTOR_LOG_LEVEL_ERROR].user_data,
                              sax2->locator, 
                              nmsg ? nmsg : msg, 
                              args);
@@ -347,13 +349,40 @@ raptor_libxml_error(void* user_data, const char *msg, ...)
 
 
 
-static void
+void
 raptor_libxml_generic_error(void* user_data, const char *msg, ...) 
 {
+  raptor_error_handlers* error_handlers=(raptor_error_handlers*)user_data;
   va_list args;
+  const char* prefix=xml_generic_error_prefix;
+  int prefix_length=strlen(prefix);
+  int length;
+  char *nmsg;
 
   va_start(args, msg);
-  raptor_libxml_error_common(user_data, msg, args, xml_generic_error_prefix, 0);
+
+  /* no SAX2 and locator from error_handlers */
+
+  length=prefix_length+strlen(msg)+1;
+  nmsg=(char*)RAPTOR_MALLOC(cstring, length);
+  if(nmsg) {
+    strcpy(nmsg, prefix);
+    strcpy(nmsg+prefix_length, msg);
+    if(nmsg[length-1]=='\n')
+      nmsg[length-1]='\0';
+  }
+
+  raptor_log_error_varargs(error_handlers->world,
+                           RAPTOR_LOG_LEVEL_ERROR,
+                           error_handlers->handlers[RAPTOR_LOG_LEVEL_ERROR].handler,
+                           error_handlers->handlers[RAPTOR_LOG_LEVEL_ERROR].user_data,
+                           error_handlers->locator,
+                           nmsg ? nmsg : msg, 
+                           args);
+  
+  if(nmsg)
+    RAPTOR_FREE(cstring,nmsg);
+
   va_end(args);
 }
 
@@ -403,9 +432,10 @@ raptor_libxml_validation_warning(void* user_data, const char *msg, ...)
       nmsg[length-2]='\0';
   }
 
-  raptor_log_error_varargs(RAPTOR_LOG_LEVEL_WARNING,
-                           sax2->error_handlers->warning_handler, 
-                           sax2->error_handlers->warning_user_data,
+  raptor_log_error_varargs(sax2->world,
+                           RAPTOR_LOG_LEVEL_WARNING,
+                           sax2->error_handlers->handlers[RAPTOR_LOG_LEVEL_WARNING].handler,
+                           sax2->error_handlers->handlers[RAPTOR_LOG_LEVEL_WARNING].user_data,
                            sax2->locator, 
                            nmsg ? nmsg : msg, 
                            args);
@@ -446,6 +476,7 @@ raptor_libxml_init(raptor_sax2* sax2, raptor_uri *base_uri)
   sax->warning=(warningSAXFunc)raptor_libxml_warning;
   sax->error=(errorSAXFunc)raptor_libxml_error;
   sax->fatalError=(fatalErrorSAXFunc)raptor_libxml_fatal_error;
+  sax->serror=(xmlStructuredErrorFunc)raptor_libxml_xmlStructuredErrorFunc;
 
 #ifdef RAPTOR_LIBXML_XMLSAXHANDLER_EXTERNALSUBSET
   sax->externalSubset = raptor_libxml_externalSubset;
@@ -462,6 +493,7 @@ raptor_libxml_init_sax_error_handlers(xmlSAXHandler *sax) {
   sax->warning=(warningSAXFunc)raptor_libxml_warning;
   sax->error=(errorSAXFunc)raptor_libxml_error;
   sax->fatalError=(fatalErrorSAXFunc)raptor_libxml_fatal_error;
+  sax->serror=(xmlStructuredErrorFunc)raptor_libxml_xmlStructuredErrorFunc;
 
 #ifdef RAPTOR_LIBXML_XMLSAXHANDLER_INITIALIZED
   sax->initialized = 1;
@@ -469,17 +501,15 @@ raptor_libxml_init_sax_error_handlers(xmlSAXHandler *sax) {
 }
 
 
-void
-raptor_libxml_init_generic_error_handlers(raptor_sax2 *sax2)
-{
-  xmlGenericError = (xmlGenericErrorFunc)raptor_libxml_generic_error;
-  xmlGenericErrorContext = sax2;
-}
-
-
 void
 raptor_libxml_free(xmlParserCtxtPtr xc) {
   libxml2_endDocument(xc);
+
+  if(xc->myDoc) {
+    xmlFreeDoc(xc->myDoc);
+    xc->myDoc=NULL;
+  }
+
   xmlFreeParserCtxt(xc);
 }
 
@@ -510,7 +540,7 @@ raptor_libxml_free(xmlParserCtxtPtr xc) {
  * When the minimum libxml2 version goes up, the #ifdefs for
  * older versions can be removed.
  */
-static const char* raptor_libxml_domain_labels[XML_LAST_DL+2]= {
+static const char* const raptor_libxml_domain_labels[XML_LAST_DL+2]= {
   NULL,                /* XML_FROM_NONE */
   "parser",            /* XML_FROM_PARSER */
   "tree",              /* XML_FROM_TREE */
@@ -561,9 +591,28 @@ static const char* raptor_libxml_domain_labels[XML_LAST_DL+2]= {
 void 
 raptor_libxml_xmlStructuredErrorFunc(void *user_data, xmlErrorPtr err)
 {
-  raptor_error_handlers* error_handlers=(raptor_error_handlers*)user_data;
+  raptor_error_handlers* error_handlers = NULL;
   raptor_stringbuffer* sb;
   char *nmsg;
+  raptor_message_handler handler=NULL;
+  void* handler_data=NULL;
+  raptor_log_level level=RAPTOR_LOG_LEVEL_ERROR;
+
+  /* user_data OR err->ctxt->userData may point to a raptor_sax2 */
+  if(user_data) {
+    raptor_sax2* sax2 = (raptor_sax2*)user_data;
+    if(sax2->magic == RAPTOR_LIBXML_MAGIC)
+      error_handlers = sax2->error_handlers;
+  }
+  
+  if(err && err->ctxt) {
+    xmlParserCtxtPtr ctxt = (xmlParserCtxtPtr)err->ctxt;
+    if(ctxt->userData) {
+      raptor_sax2* sax2 = (raptor_sax2*)ctxt->userData;
+      if(sax2->magic == RAPTOR_LIBXML_MAGIC)
+        error_handlers = sax2->error_handlers;
+    }
+  }
   
   if(err == NULL || err->code == XML_ERR_OK || err->level == XML_ERR_NONE)
     return;
@@ -576,10 +625,11 @@ raptor_libxml_xmlStructuredErrorFunc(void *user_data, xmlErrorPtr err)
   if(err->level == XML_ERR_FATAL)
     err->level= XML_ERR_ERROR;
   
-  
+
   sb=raptor_new_stringbuffer();
-  raptor_stringbuffer_append_counted_string(sb, (const unsigned char*)"XML ",
-                                            4, 1);
+  if(err->domain != XML_FROM_HTML)
+    raptor_stringbuffer_append_counted_string(sb, (const unsigned char*)"XML ",
+                                              4, 1);
   
   if(err->domain != XML_FROM_NONE && err->domain < XML_LAST_DL) {
     const unsigned char* label;
@@ -607,30 +657,69 @@ raptor_libxml_xmlStructuredErrorFunc(void *user_data, xmlErrorPtr err)
     
     raptor_stringbuffer_append_counted_string(sb, msg, len, 1);
   }
+
+#if LIBXML_VERSION >= 20618
+  /* 2005-02-13 - v2.6.18 */
+
+  /* str1 has the detailed HTTP error */
+  if(err->domain == XML_FROM_HTTP && err->str1) {
+    unsigned char* msg;
+    size_t len;
+    msg=(unsigned char*)err->str1;
+    len= strlen((const char*)msg);
+    if(len && msg[len-1] == '\n')
+      msg[--len]='\0';
+    
+    raptor_stringbuffer_append_counted_string(sb, (const unsigned char*)" - ",
+                                              3, 1);
+    raptor_stringbuffer_append_counted_string(sb, msg, len, 1);
+  }
+#endif
   
   /* When err->domain == XML_FROM_XPATH then err->int1 is
    * the offset into err->str1, the line with the error
    */
+  if(err->domain == XML_FROM_XPATH && err->str1) {
+    raptor_stringbuffer_append_counted_string(sb, (const unsigned char*)" in ",
+                                              4, 1);
+    raptor_stringbuffer_append_string(sb, (const unsigned char*)err->str1, 1);
+  }
+
+  if(error_handlers) {
+    if(error_handlers->magic != RAPTOR_ERROR_HANDLER_MAGIC) {
+#ifdef RAPTOR_DEBUG
+      if(1) /* FIXME */
+        RAPTOR_DEBUG2("Received bogus error_handlers pointer %p\n",
+                      error_handlers);
+        else
+          RAPTOR_FATAL2("Received bogus error_handlers pointer %p\n",
+                        error_handlers);
+#endif
+      error_handlers=NULL;
+    }
+  }
   
   nmsg=(char*)raptor_stringbuffer_as_string(sb);
   if(err->level == XML_ERR_FATAL)
-    raptor_log_error(RAPTOR_LOG_LEVEL_FATAL, 
-                     error_handlers->fatal_error_handler,
-                     error_handlers->fatal_error_user_data,
-                     error_handlers->locator,
-                     nmsg);
+    level=RAPTOR_LOG_LEVEL_FATAL;
   else if(err->level == XML_ERR_ERROR)
-    raptor_log_error(RAPTOR_LOG_LEVEL_ERROR, 
-                     error_handlers->error_handler,
-                     error_handlers->error_user_data,
+    level=RAPTOR_LOG_LEVEL_ERROR;
+  else
+    level=RAPTOR_LOG_LEVEL_WARNING;
+
+  if(error_handlers && level <= error_handlers->last_log_level) {
+    handler=error_handlers->handlers[level].handler;
+    handler_data=error_handlers->handlers[level].user_data;
+  }
+
+  if(error_handlers)
+    raptor_log_error(error_handlers->world,
+                     level, handler, handler_data,
                      error_handlers->locator,
                      nmsg);
   else
-    raptor_log_error(RAPTOR_LOG_LEVEL_WARNING, 
-                     error_handlers->warning_handler,
-                     error_handlers->warning_user_data,
-                     error_handlers->locator,
-                     nmsg);
+    fputs(nmsg, stderr);
+
   
   raptor_free_stringbuffer(sb);
 }
