@@ -2,7 +2,7 @@
  *
  * raptor_qname.c - Raptor XML qname class
  *
- * Copyright (C) 2002-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2002-2008, David Beckett http://www.dajobe.org/
  * Copyright (C) 2002-2004, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -119,7 +119,7 @@ raptor_new_qname(raptor_namespace_stack *nstack,
   qname=(raptor_qname*)RAPTOR_CALLOC(raptor_qname, 1, sizeof(raptor_qname));
   if(!qname)
     return NULL;
-
+  qname->world=nstack->world;
 
   if(value) {
     int value_length=strlen((char*)value);
@@ -160,7 +160,7 @@ raptor_new_qname(raptor_namespace_stack *nstack,
       if(ns) {
         qname->nspace=ns;
 #if RAPTOR_DEBUG > 1
-        RAPTOR_DEBUG2("Found default namespace %s\n", ns->uri);
+        RAPTOR_DEBUG2("Found default namespace %s\n", raptor_uri_as_string_v2(nstack->world, ns->uri));
 #endif
       } else {
 #if RAPTOR_DEBUG > 1
@@ -195,7 +195,7 @@ raptor_new_qname(raptor_namespace_stack *nstack,
         error_handler((raptor_parser*)error_data, "The namespace prefix in \"%s\" was not declared.", name);
     } else {
 #if RAPTOR_DEBUG > 1
-      RAPTOR_DEBUG3("Found namespace prefix %s URI %s\n", ns->prefix, ns->uri);
+      RAPTOR_DEBUG3("Found namespace prefix %s URI %s\n", ns->prefix, raptor_uri_as_string_v2(nstack->world, ns->uri));
 #endif
       qname->nspace=ns;
     }
@@ -209,7 +209,7 @@ raptor_new_qname(raptor_namespace_stack *nstack,
   if(qname->nspace && local_name_length) {
     raptor_uri *uri=raptor_namespace_get_uri(qname->nspace);
     if(uri)
-      uri=raptor_new_uri_from_uri_local_name(uri, new_name);
+      uri=raptor_new_uri_from_uri_local_name_v2(qname->world, uri, new_name);
 
     qname->uri=uri;
   }
@@ -219,9 +219,40 @@ raptor_new_qname(raptor_namespace_stack *nstack,
 }
 
 
+#ifndef RAPTOR_DISABLE_V1
 /**
  * raptor_new_qname_from_namespace_local_name:
- * @ns: namespace of qname
+ * @ns: namespace of qname (or NULL)
+ * @local_name: element or attribute name
+ * @value: attribute value (else is an element)
+ *
+ * Constructor - create a new XML qname.
+ * 
+ * Create a new qname from the namespace and local element/attribute name,
+ * with optional (attribute) value.
+ * 
+ * raptor_init() MUST have been called before calling this function.
+ * Use raptor_new_qname_from_namespace_local_name_v2() if using raptor_world APIs.
+ *
+ * Return value: a new #raptor_qname object or NULL on failure
+ **/
+raptor_qname*
+raptor_new_qname_from_namespace_local_name(raptor_namespace *ns, 
+                                           const unsigned char *local_name,
+                                           const unsigned char *value)
+{
+  return raptor_new_qname_from_namespace_local_name_v2(raptor_world_instance(),
+                                                       ns,
+                                                       local_name,
+                                                       value);
+}
+#endif
+
+
+/**
+ * raptor_new_qname_from_namespace_local_name_v2:
+ * @world: raptor_world object
+ * @ns: namespace of qname (or NULL)
  * @local_name: element or attribute name
  * @value: attribute value (else is an element)
  *
@@ -233,20 +264,22 @@ raptor_new_qname(raptor_namespace_stack *nstack,
  * Return value: a new #raptor_qname object or NULL on failure
  **/
 raptor_qname*
-raptor_new_qname_from_namespace_local_name(raptor_namespace *ns, 
-                                           const unsigned char *local_name,
-                                           const unsigned char *value)
+raptor_new_qname_from_namespace_local_name_v2(raptor_world* world,
+                                              raptor_namespace *ns, 
+                                              const unsigned char *local_name,
+                                              const unsigned char *value)
 {
   raptor_qname* qname;
   unsigned char* new_name;
   int local_name_length=strlen((char*)local_name);
 
-  if(!ns || !local_name)
+  if(!local_name)
     return NULL;
 
   qname=(raptor_qname*)RAPTOR_CALLOC(raptor_qname, 1, sizeof(raptor_qname));
   if(!qname)
     return NULL;
+  qname->world=world;
 
   if(value) {
     int value_length=strlen((char*)value);
@@ -272,9 +305,11 @@ raptor_new_qname_from_namespace_local_name(raptor_namespace *ns,
 
   qname->nspace=ns;
 
-  qname->uri=raptor_namespace_get_uri(qname->nspace);
-  if(qname->uri)
-    qname->uri=raptor_new_uri_from_uri_local_name(qname->uri, new_name);
+  if(qname->nspace) {
+    qname->uri=raptor_namespace_get_uri(qname->nspace);
+    if(qname->uri)
+      qname->uri=raptor_new_uri_from_uri_local_name_v2(qname->world, qname->uri, new_name);
+  }
   
   return qname;
 }
@@ -296,6 +331,7 @@ raptor_qname_copy(raptor_qname *qname) {
   new_qname=(raptor_qname*)RAPTOR_CALLOC(raptor_qname, 1, sizeof(raptor_qname));
   if(!new_qname)
     return NULL;
+  new_qname->world=qname->world;
 
   if(qname->value) {
     int value_length=qname->value_length;
@@ -323,7 +359,7 @@ raptor_qname_copy(raptor_qname *qname) {
 
   new_qname->uri=raptor_namespace_get_uri(new_qname->nspace);
   if(new_qname->uri)
-    new_qname->uri=raptor_new_uri_from_uri_local_name(new_qname->uri, new_name);
+    new_qname->uri=raptor_new_uri_from_uri_local_name_v2(qname->world, new_qname->uri, new_name);
   
   return new_qname;
 }
@@ -354,11 +390,13 @@ raptor_qname_print(FILE *stream, raptor_qname* name)
 void
 raptor_free_qname(raptor_qname* name) 
 {
+  RAPTOR_ASSERT_OBJECT_POINTER_RETURN(name, raptor_qname);
+
   if(name->local_name)
     RAPTOR_FREE(cstring, (void*)name->local_name);
 
-  if(name->uri)
-    raptor_free_uri(name->uri);
+  if(name->uri && name->nspace)
+    raptor_free_uri_v2(name->world, name->uri);
 
   if(name->value)
     RAPTOR_FREE(cstring, (void*)name->value);
@@ -475,9 +513,9 @@ raptor_qname_string_to_uri(raptor_namespace_stack *nstack,
    */
   if(ns && (uri=raptor_namespace_get_uri(ns))) {
     if(local_name_length)
-      uri=raptor_new_uri_from_uri_local_name(uri, local_name);
+      uri=raptor_new_uri_from_uri_local_name_v2(nstack->world, uri, local_name);
     else
-      uri=raptor_uri_copy(uri);
+      uri=raptor_uri_copy_v2(nstack->world, uri);
   }
 
   return uri;
@@ -508,6 +546,46 @@ raptor_iostream_write_qname(raptor_iostream* iostr, raptor_qname *qname)
 }
 
 
+/**
+ * raptor_qname_to_counted_name:
+ * @qname: QName to write
+ * @length_p: pointer to variable to store length of name (or NULL)
+ * 
+ * Get the string form of a QName name
+ * 
+ * Return value: string or NULL on failure
+ **/
+unsigned char*
+raptor_qname_to_counted_name(raptor_qname *qname, size_t* length_p)
+{
+  size_t len=qname->local_name_length;
+  unsigned char* s;
+  unsigned char *p;
+
+  if(qname->nspace && qname->nspace->prefix_length > 0)
+    len+= 1 + qname->nspace->prefix_length;
+
+  if(length_p)
+    *length_p=len;
+  
+  s=(unsigned char*)RAPTOR_MALLOC(cstring, len+1);
+  if(!s)
+    return NULL;
+
+  p=s;
+  if(qname->nspace && qname->nspace->prefix_length > 0) {
+    strncpy((char*)p, (const char*)qname->nspace->prefix,
+            qname->nspace->prefix_length);
+    p+= qname->nspace->prefix_length;
+    *p++ = ':';
+  }
+  
+  strncpy((char*)p, (const char*)qname->local_name, qname->local_name_length+1);
+
+  return s;
+}
+
+
 /**
  * raptor_qname_get_namespace:
  * @name: #raptor_qname object
@@ -521,3 +599,50 @@ raptor_qname_get_namespace(raptor_qname* name)
 {
   return name->nspace;
 }
+
+
+/**
+ * raptor_qname_get_local_name:
+ * @name: #raptor_qname object
+ * 
+ * Get the #raptor_local_name of an XML QName.
+ * 
+ * Return value: the local_name
+ **/
+const unsigned char*
+raptor_qname_get_local_name(raptor_qname* name)
+{
+  return name->local_name;
+}
+
+
+/**
+ * raptor_qname_get_value:
+ * @name: #raptor_qname object
+ * 
+ * Get the #raptor_value of an XML QName.
+ * 
+ * Return value: the value
+ **/
+const unsigned char*
+raptor_qname_get_value(raptor_qname* name)
+{
+  return name->value;
+}
+
+/**
+ * raptor_qname_get_counted_value:
+ * @name: #raptor_qname object
+ * @length_p: pointer to variable to store length of name (or NULL)
+ * 
+ * Get the #raptor_value of an XML QName.
+ * 
+ * Return value: the value
+ **/
+const unsigned char*
+raptor_qname_get_counted_value(raptor_qname* name, size_t* length_p)
+{
+  if(length_p)
+    *length_p=name->value_length;
+  return name->value;
+}
