@@ -53,8 +53,11 @@
 
 
 int
-raptor_sax2_init(void)
+raptor_sax2_init(raptor_world* world)
 {
+  if(world->sax2_initialized++)
+    return 0;
+
 #ifdef RAPTOR_XML_LIBXML
   xmlInitParser();
 #endif
@@ -63,12 +66,17 @@ raptor_sax2_init(void)
 
 
 void
-raptor_sax2_finish(void)
+raptor_sax2_finish(raptor_world* world)
 {
+  if(--world->sax2_initialized)
+    return;
+
 #ifdef RAPTOR_XML_LIBXML
+  /* Should call this after all uses of libxml are done.
+   * In particular after xmlSetStructuredErrorFunc() otherwise
+   * it has reportedly caused an access violation on windows.
+   */
   xmlCleanupParser();
-  xmlSetStructuredErrorFunc(NULL, NULL);
-  xmlSetGenericErrorFunc(NULL, NULL);
 #endif
 }
 
@@ -94,6 +102,8 @@ raptor_new_sax2(void* user_data, raptor_error_handlers* error_handlers)
   sax2->magic=RAPTOR_LIBXML_MAGIC;
 #endif
 
+  sax2->world=error_handlers->world;
+
   sax2->user_data=user_data;
 
   sax2->locator=error_handlers->locator;
@@ -101,12 +111,21 @@ raptor_new_sax2(void* user_data, raptor_error_handlers* error_handlers)
   sax2->error_handlers=error_handlers;
 
 #ifdef RAPTOR_XML_LIBXML
-  /* sets xmlGenericErrorContext and xmlStructuredError */
-  xmlSetStructuredErrorFunc(&sax2->error_handlers, 
-                            raptor_libxml_xmlStructuredErrorFunc);
-  /* sets xmlGenericErrorContext and xmlGenericError */
-  xmlSetGenericErrorFunc(&sax2->error_handlers, 
-                         (xmlGenericErrorFunc)raptor_libxml_generic_error);
+  if(sax2->world->libxml_flags & RAPTOR_LIBXML_FLAGS_STRUCTURED_ERROR_SAVE) {
+    sax2->saved_structured_error_context = xmlGenericErrorContext;
+    sax2->saved_structured_error_handler = xmlStructuredError;
+    /* sets xmlGenericErrorContext and xmlStructuredError */
+    xmlSetStructuredErrorFunc(&sax2->error_handlers, 
+                              (xmlStructuredErrorFunc)raptor_libxml_xmlStructuredErrorFunc);
+  }
+  
+  if(sax2->world->libxml_flags & RAPTOR_LIBXML_FLAGS_GENERIC_ERROR_SAVE) {
+    sax2->saved_generic_error_context = xmlGenericErrorContext;
+    sax2->saved_generic_error_handler = xmlGenericError;
+    /* sets xmlGenericErrorContext and xmlGenericError */
+    xmlSetGenericErrorFunc(&sax2->error_handlers, 
+                           (xmlGenericErrorFunc)raptor_libxml_generic_error);
+  }
 #endif
   
   return sax2;
@@ -120,9 +139,12 @@ raptor_new_sax2(void* user_data, raptor_error_handlers* error_handlers)
  * Destructor - destroy a SAX2 object
  */
 void
-raptor_free_sax2(raptor_sax2 *sax2) {
+raptor_free_sax2(raptor_sax2 *sax2)
+{
   raptor_xml_element *xml_element;
 
+  RAPTOR_ASSERT_OBJECT_POINTER_RETURN(sax2, raptor_sax2);
+
 #ifdef RAPTOR_XML_EXPAT
   if(sax2->xp) {
     XML_ParserFree(sax2->xp);
@@ -135,6 +157,14 @@ raptor_free_sax2(raptor_sax2 *sax2) {
     raptor_libxml_free(sax2->xc);
     sax2->xc=NULL;
   }
+
+  if(sax2->world->libxml_flags & RAPTOR_LIBXML_FLAGS_STRUCTURED_ERROR_SAVE)
+    xmlSetStructuredErrorFunc(sax2->saved_structured_error_context,
+                              sax2->saved_structured_error_handler);
+
+  if(sax2->world->libxml_flags & RAPTOR_LIBXML_FLAGS_GENERIC_ERROR_SAVE)
+    xmlSetGenericErrorFunc(sax2->saved_generic_error_context,
+                           sax2->saved_generic_error_handler);
 #endif
 
   while( (xml_element=raptor_xml_element_pop(sax2)) )
@@ -143,7 +173,7 @@ raptor_free_sax2(raptor_sax2 *sax2) {
   raptor_namespaces_clear(&sax2->namespaces);
 
   if(sax2->base_uri)
-    raptor_free_uri(sax2->base_uri);
+    raptor_free_uri_v2(sax2->world, sax2->base_uri);
 
   RAPTOR_FREE(raptor_sax2, sax2);
 }
@@ -409,7 +439,8 @@ raptor_sax2_simple_error(void* user_data, const char *message, ...)
     raptor_log_level level=RAPTOR_LOG_LEVEL_ERROR;
     raptor_message_handler_closure* cl;
     cl=&sax2->error_handlers->handlers[level];
-    raptor_log_error_varargs(level, cl->handler, cl->user_data,
+    raptor_log_error_varargs(sax2->world,
+                             level, cl->handler, cl->user_data,
                              sax2->locator,
                              message, arguments);
   }
@@ -429,17 +460,14 @@ raptor_sax2_simple_error(void* user_data, const char *message, ...)
 void
 raptor_sax2_parse_start(raptor_sax2* sax2, raptor_uri *base_uri)
 {
-  const raptor_uri_handler *uri_handler;
-  void *uri_context;
-
   sax2->depth=0;
   sax2->root_element=NULL;
   sax2->current_element=NULL;
 
   if(sax2->base_uri)
-    raptor_free_uri(sax2->base_uri);
+    raptor_free_uri_v2(sax2->world, sax2->base_uri);
   if(base_uri)
-    sax2->base_uri=raptor_uri_copy(base_uri);
+    sax2->base_uri=raptor_uri_copy_v2(sax2->world, base_uri);
   else
     sax2->base_uri=NULL;
 
@@ -455,7 +483,7 @@ raptor_sax2_parse_start(raptor_sax2* sax2, raptor_uri *base_uri)
 #ifdef RAPTOR_XML_LIBXML
   raptor_libxml_init(sax2, base_uri);
 
-  xmlSetStructuredErrorFunc(sax2->error_handlers, 
+  xmlSetStructuredErrorFunc(&sax2->error_handlers, 
                             raptor_libxml_xmlStructuredErrorFunc);
 
 #if LIBXML_VERSION < 20425
@@ -470,12 +498,18 @@ raptor_sax2_parse_start(raptor_sax2* sax2, raptor_uri *base_uri)
 
   raptor_namespaces_clear(&sax2->namespaces);
 
-  raptor_uri_get_handler(&uri_handler, &uri_context);
-  raptor_namespaces_init(&sax2->namespaces,
-                         uri_handler, uri_context,
-                         (raptor_simple_message_handler)raptor_sax2_simple_error, sax2, 
-                         1);
-
+  if(raptor_namespaces_init_v2(sax2->world,
+                               &sax2->namespaces,
+                               (raptor_simple_message_handler)raptor_sax2_simple_error, sax2, 
+                               1)) {
+    /* log a fatal error and set sax2 to failed state
+       since the function signature does not currently support returning an error */
+    raptor_log_error_to_handlers(sax2->world,
+                                 sax2->error_handlers,
+                                 RAPTOR_LOG_LEVEL_FATAL, sax2->locator,
+                                 "raptor_namespaces_init_v2() failed");
+    sax2->failed = 1;
+  }
 }
 
 
@@ -511,7 +545,8 @@ raptor_sax2_parse_chunk(raptor_sax2* sax2, const unsigned char *buffer,
     if(!len) {
       /* no data given at all - emit a similar message to expat */
       raptor_sax2_update_document_locator(sax2, sax2->locator);
-      raptor_log_error_to_handlers(sax2->error_handlers,
+      raptor_log_error_to_handlers(sax2->world,
+                                   sax2->error_handlers,
                                    RAPTOR_LOG_LEVEL_ERROR, sax2->locator,
                                    "XML Parsing failed - no element found");
       return 1;
@@ -582,12 +617,12 @@ raptor_sax2_parse_chunk(raptor_sax2* sax2, const unsigned char *buffer,
 #if LIBXML_VERSION < 20425
   if(sax2->first_read && is_end) {
     /* parse all but the last character */
-    rc=xmlParseChunk(xc, (char*)buffer, len-1, 0);
-    if(rc)
+    rc = xmlParseChunk(xc, (char*)buffer, len-1, 0);
+    if(rc && rc != XML_WAR_UNDECLARED_ENTITY)
       goto handle_error;
     /* last character */
-    rc=xmlParseChunk(xc, (char*)buffer + (len-1), 1, 0);
-    if(rc)
+    rc = xmlParseChunk(xc, (char*)buffer + (len-1), 1, 0);
+    if(rc && rc != XML_WAR_UNDECLARED_ENTITY)
       goto handle_error;
     /* end */
     xmlParseChunk(xc, (char*)buffer, 0, 1);
@@ -599,8 +634,8 @@ raptor_sax2_parse_chunk(raptor_sax2* sax2, const unsigned char *buffer,
   sax2->first_read=0;
 #endif
     
-  rc=xmlParseChunk(xc, (char*)buffer, len, is_end);
-  if(rc) /* libxml: non 0 is failure */
+  rc = xmlParseChunk(xc, (char*)buffer, len, is_end);
+  if(rc && rc != XML_WAR_UNDECLARED_ENTITY) /* libxml: non 0 is failure */
     goto handle_error;
   if(is_end)
     return 0;
@@ -627,7 +662,7 @@ raptor_sax2_parse_chunk(raptor_sax2* sax2, const unsigned char *buffer,
 #endif
 #endif /* EXPAT */
 
-#if RAPTOR_XML_EXPAT
+#ifdef RAPTOR_XML_EXPAT
   if(1) {
     const char *error_prefix="XML Parsing failed - "; /* 21 chars */
     #define ERROR_PREFIX_LEN 21
@@ -642,12 +677,14 @@ raptor_sax2_parse_chunk(raptor_sax2* sax2, const unsigned char *buffer,
       strncpy(error_buffer, error_prefix, ERROR_PREFIX_LEN);
       strncpy(error_buffer+ERROR_PREFIX_LEN, error_message, error_length+1);
 
-      raptor_log_error_to_handlers(sax2->error_handlers,
+      raptor_log_error_to_handlers(sax2->world,
+                                   sax2->error_handlers,
                                    RAPTOR_LOG_LEVEL_ERROR,
                                    sax2->locator, error_buffer);
       RAPTOR_FREE(cstring, error_buffer);
     } else
-      raptor_log_error_to_handlers(sax2->error_handlers,
+      raptor_log_error_to_handlers(sax2->world,
+                                   sax2->error_handlers,
                                    RAPTOR_LOG_LEVEL_ERROR,
                                    sax2->locator, "XML Parsing failed");
   }
@@ -716,6 +753,9 @@ raptor_sax2_set_feature(raptor_sax2 *sax2, raptor_feature feature, int value)
     /* JSON serializer features */
     case RAPTOR_FEATURE_JSON_CALLBACK:
     case RAPTOR_FEATURE_JSON_EXTRA_DATA:
+    case RAPTOR_FEATURE_RSS_TRIPLES:
+    case RAPTOR_FEATURE_ATOM_ENTRY_URI:
+    case RAPTOR_FEATURE_PREFIX_ELEMENTS:
     
     /* Turtle serializer feature */
     case RAPTOR_FEATURE_WRITE_BASE_URI:
@@ -762,6 +802,9 @@ raptor_sax2_start_element(void* user_data, const unsigned char *name,
   unsigned char *xml_language=NULL;
   raptor_uri *xml_base=NULL;
 
+  if(sax2->failed)
+    return;
+
 #ifdef RAPTOR_XML_EXPAT
 #ifdef EXPAT_UTF8_BOM_CRASH
   sax2->tokens_count++;
@@ -779,7 +822,8 @@ raptor_sax2_start_element(void* user_data, const unsigned char *name,
       unsigned char *dst = xmlStrdup(value);
 
       if(!dst) {
-        raptor_log_error_to_handlers(sax2->error_handlers, 
+        raptor_log_error_to_handlers(sax2->world,
+                                     sax2->error_handlers, 
                                      RAPTOR_LOG_LEVEL_FATAL,
                                      sax2->locator, "Out of memory");
         return;
@@ -858,7 +902,8 @@ raptor_sax2_start_element(void* user_data, const unsigned char *name,
       } else if(!strcmp((char*)atts[i], "xml:lang")) {
         xml_language=(unsigned char*)RAPTOR_MALLOC(cstring, strlen((char*)atts[i+1])+1);
         if(!xml_language) {
-          raptor_log_error_to_handlers(sax2->error_handlers, 
+          raptor_log_error_to_handlers(sax2->world,
+                                       sax2->error_handlers, 
                                        RAPTOR_LOG_LEVEL_FATAL,
                                        sax2->locator, "Out of memory");
           goto fail;
@@ -882,9 +927,9 @@ raptor_sax2_start_element(void* user_data, const unsigned char *name,
         raptor_uri* base_uri;
         raptor_uri* xuri;
         base_uri=raptor_sax2_inscope_base_uri(sax2);
-        xuri=raptor_new_uri_relative_to_base(base_uri, atts[i+1]);
-        xml_base=raptor_new_uri_for_xmlbase(xuri);
-        raptor_free_uri(xuri);
+        xuri=raptor_new_uri_relative_to_base_v2(sax2->world, base_uri, atts[i+1]);
+        xml_base=raptor_new_uri_for_xmlbase_v2(sax2->world, xuri);
+        raptor_free_uri_v2(sax2->world, xuri);
       }
 
       /* delete all xml attributes whether processed above or not */
@@ -918,7 +963,8 @@ raptor_sax2_start_element(void* user_data, const unsigned char *name,
                                               ns_attributes_count, 
                                               sizeof(raptor_qname*));
     if(!named_attrs) {
-      raptor_log_error_to_handlers(sax2->error_handlers, 
+      raptor_log_error_to_handlers(sax2->world,
+                                   sax2->error_handlers, 
                                    RAPTOR_LOG_LEVEL_FATAL,
                                    sax2->locator, "Out of memory");
       goto fail;
@@ -970,7 +1016,7 @@ raptor_sax2_start_element(void* user_data, const unsigned char *name,
   if(xml_atts_copy)
     RAPTOR_FREE(cstringpointer, xml_atts_copy);
   if(xml_base)
-    raptor_free_uri(xml_base);
+    raptor_free_uri_v2(sax2->world, xml_base);
   if(xml_language)
     RAPTOR_FREE(cstring, xml_language);
   if(xml_element)
@@ -985,6 +1031,9 @@ raptor_sax2_end_element(void* user_data, const unsigned char *name)
   raptor_sax2* sax2=(raptor_sax2*)user_data;
   raptor_xml_element* xml_element;
 
+  if(sax2->failed)
+    return;
+
 #ifdef RAPTOR_XML_EXPAT
 #ifdef EXPAT_UTF8_BOM_CRASH
   sax2->tokens_count++;
@@ -1020,7 +1069,7 @@ void
 raptor_sax2_characters(void* user_data, const unsigned char *s, int len)
 {
   raptor_sax2* sax2=(raptor_sax2*)user_data;
-  if(sax2->characters_handler)
+  if(!sax2->failed && sax2->characters_handler)
     sax2->characters_handler(sax2->user_data, sax2->current_element, s, len);
 }
 
@@ -1036,7 +1085,7 @@ raptor_sax2_cdata(void* user_data, const unsigned char *s, int len)
 #endif
 #endif
 
-  if(sax2->cdata_handler)
+  if(!sax2->failed && sax2->cdata_handler)
     sax2->cdata_handler(sax2->user_data, sax2->current_element, s, len);
 }
 
@@ -1046,7 +1095,7 @@ void
 raptor_sax2_comment(void* user_data, const unsigned char *s)
 {
   raptor_sax2* sax2=(raptor_sax2*)user_data;
-  if(sax2->comment_handler)
+  if(!sax2->failed && sax2->comment_handler)
     sax2->comment_handler(sax2->user_data, sax2->current_element, s);
 }
 
@@ -1061,7 +1110,7 @@ raptor_sax2_unparsed_entity_decl(void* user_data,
                                  const unsigned char* notationName)
 {
   raptor_sax2* sax2=(raptor_sax2*)user_data;
-  if(sax2->unparsed_entity_decl_handler)
+  if(!sax2->failed && sax2->unparsed_entity_decl_handler)
     sax2->unparsed_entity_decl_handler(sax2->user_data,
                                        entityName, base, systemId, 
                                        publicId, notationName);
@@ -1077,6 +1126,10 @@ raptor_sax2_external_entity_ref(void* user_data,
                                 const unsigned char* publicId)
 {
   raptor_sax2* sax2=(raptor_sax2*)user_data;
+
+  if(sax2->failed)
+    return 0;
+
   if(sax2->external_entity_ref_handler)
     return sax2->external_entity_ref_handler(sax2->user_data,
                                              context, base, systemId, publicId);
