@@ -2,7 +2,7 @@
  *
  * raptor_serialize.c - Serializers
  *
- * Copyright (C) 2004-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2004-2008, David Beckett http://www.dajobe.org/
  * Copyright (C) 2004-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -48,13 +48,7 @@
 
 
 /* prototypes for helper functions */
-static raptor_serializer_factory* raptor_get_serializer_factory(const char *name);
-
-
-/* statics */
-
-/* list of serializer factories */
-static raptor_sequence* serializers=NULL;
+static raptor_serializer_factory* raptor_get_serializer_factory(raptor_world* world, const char *name);
 
 
 /* helper methods */
@@ -62,6 +56,8 @@ static raptor_sequence* serializers=NULL;
 static void
 raptor_free_serializer_factory(raptor_serializer_factory* factory)
 {
+  RAPTOR_ASSERT_OBJECT_POINTER_RETURN(factory, raptor_serializer_factory);
+
   if(factory->finish_factory)
     factory->finish_factory(factory);
   
@@ -82,44 +78,50 @@ raptor_free_serializer_factory(raptor_serializer_factory* factory)
 
 /* class methods */
 
-void
-raptor_serializers_init(void)
+int
+raptor_serializers_init(raptor_world* world)
 {
-  serializers=raptor_new_sequence((raptor_sequence_free_handler *)raptor_free_serializer_factory, NULL);
-  if(!serializers) {
-    raptor_finish();
-    RAPTOR_FATAL1("Out of memory\n");
-  }
+  int rc=0;
+
+  world->serializers=raptor_new_sequence((raptor_sequence_free_handler *)raptor_free_serializer_factory, NULL);
+  if(!world->serializers)
+    return 1;
 
   /* raptor_init_serializer_simple(); */
 
 #ifdef RAPTOR_SERIALIZER_NTRIPLES
-  raptor_init_serializer_ntriples();
+  rc+= raptor_init_serializer_ntriples(world) != 0;
 #endif
 
 #ifdef RAPTOR_SERIALIZER_TURTLE
-  raptor_init_serializer_turtle();
+  rc+= raptor_init_serializer_turtle(world) != 0;
 #endif
 
 #ifdef RAPTOR_SERIALIZER_RDFXML_ABBREV
-  raptor_init_serializer_rdfxmla();
+  rc+= raptor_init_serializer_rdfxmla(world) != 0;
 #endif
 
 #ifdef RAPTOR_SERIALIZER_RDFXML
-  raptor_init_serializer_rdfxml();
+  rc+= raptor_init_serializer_rdfxml(world) != 0;
 #endif
 
 #ifdef RAPTOR_SERIALIZER_RSS_1_0
-  raptor_init_serializer_rss10();
+  rc+= raptor_init_serializer_rss10(world) != 0;
 #endif
 
 #ifdef RAPTOR_SERIALIZER_ATOM
-  raptor_init_serializer_atom();
+  rc+= raptor_init_serializer_atom(world) != 0;
 #endif
 
 #ifdef RAPTOR_SERIALIZER_DOT
-  raptor_init_serializer_dot();
+  rc+= raptor_init_serializer_dot(world) != 0;
 #endif
+
+#ifdef RAPTOR_SERIALIZER_JSON
+  rc+= raptor_init_serializer_json(world) != 0;
+#endif
+
+  return rc;
 }
 
 
@@ -127,17 +129,18 @@ raptor_serializers_init(void)
  * raptor_serializers_finish - delete all the registered serializers
  */
 void
-raptor_serializers_finish(void)
+raptor_serializers_finish(raptor_world* world)
 {
-  if(serializers) {
-    raptor_free_sequence(serializers);
-    serializers=NULL;
+  if(world->serializers) {
+    raptor_free_sequence(world->serializers);
+    world->serializers=NULL;
   }
 }
 
 
 /*
  * raptor_serializer_register_factory - Register a syntax that can be generated by a serializer factory
+ * @world: raptor_world object
  * @name: the short syntax name
  * @label: readable label for syntax
  * @mime_type: MIME type of the syntax generated by the serializer (or NULL)
@@ -146,13 +149,16 @@ raptor_serializers_finish(void)
  * 
  * INTERNAL
  *
+ * Return value: non-0 on failure
  **/
-void
-raptor_serializer_register_factory(const char *name, const char *label,
+RAPTOR_EXTERN_C
+int
+raptor_serializer_register_factory(raptor_world* world,
+                                   const char *name, const char *label,
                                    const char *mime_type,
                                    const char *alias,
                                    const unsigned char *uri_string,
-                                   void (*factory) (raptor_serializer_factory*)) 
+                                   int (*factory) (raptor_serializer_factory*)) 
 {
   raptor_serializer_factory *serializer;
   char *name_copy, *label_copy, *mime_type_copy, *alias_copy;
@@ -164,15 +170,15 @@ raptor_serializer_register_factory(const char *name, const char *label,
                 name, label, (alias ? alias : "none"));
   RAPTOR_DEBUG3("MIME type %s, URI %s\n", 
                 (mime_type ? mime_type : "none"),
-                (uri_string ? uri_string : "none"));
+                (uri_string ? (const char *)uri_string : "none"));
 #endif
   
   for(i=0;
-      (serializer=(raptor_serializer_factory*)raptor_sequence_get_at(serializers, i));
+      (serializer=(raptor_serializer_factory*)raptor_sequence_get_at(world->serializers, i));
       i++) {
     if(!strcmp(serializer->name, name)) {
       RAPTOR_FATAL2("serializer %s already registered\n", name);
-      return;
+      return 1;
     }
   }
   
@@ -180,24 +186,26 @@ raptor_serializer_register_factory(const char *name, const char *label,
   serializer=(raptor_serializer_factory*)RAPTOR_CALLOC(raptor_serializer_factory, 1,
                                                sizeof(raptor_serializer_factory));
   if(!serializer)
-    goto fail_noserializer;
+    return 1;
+
+  serializer->world=world;
 
   name_copy=(char*)RAPTOR_CALLOC(cstring, strlen(name)+1, 1);
   if(!name_copy)
-    goto fail;
+    goto tidy;
   strcpy(name_copy, name);
   serializer->name=name_copy;
         
   label_copy=(char*)RAPTOR_CALLOC(cstring, strlen(label)+1, 1);
   if(!label_copy)
-    goto fail;
+    goto tidy;
   strcpy(label_copy, label);
   serializer->label=label_copy;
 
   if(mime_type) {
     mime_type_copy=(char*)RAPTOR_CALLOC(cstring, strlen(mime_type)+1, 1);
     if(!mime_type_copy)
-      goto fail;
+      goto tidy;
     strcpy(mime_type_copy, mime_type);
     serializer->mime_type=mime_type_copy;
   }
@@ -205,7 +213,7 @@ raptor_serializer_register_factory(const char *name, const char *label,
   if(uri_string) {
     uri_string_copy=(unsigned char*)RAPTOR_CALLOC(cstring, strlen((const char*)uri_string)+1, 1);
     if(!uri_string_copy)
-      goto fail;
+      goto tidy;
     strcpy((char*)uri_string_copy, (const char*)uri_string);
     serializer->uri_string=uri_string_copy;
   }
@@ -213,34 +221,34 @@ raptor_serializer_register_factory(const char *name, const char *label,
   if(alias) {
     alias_copy=(char*)RAPTOR_CALLOC(cstring, strlen(alias)+1, 1);
     if(!alias_copy)
-      goto fail;
+      goto tidy;
     strcpy(alias_copy, alias);
     serializer->alias=alias_copy;
   }
 
-  if(raptor_sequence_push(serializers, serializer))
-    goto fail_noserializer; /* on error, serializer is already freed by the sequence */
+  if(raptor_sequence_push(world->serializers, serializer))
+    return 1; /* on error, serializer is already freed by the sequence */
 
   /* Call the serializer registration function on the new object */
-  (*factory)(serializer);
+  if (factory(serializer))
+    return 1; /* serializer is owned and freed by the serializers sequence */
   
 #if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
   RAPTOR_DEBUG3("%s has context size %d\n", name, serializer->context_length);
 #endif
 
-  return;
+  return 0;
 
   /* Clean up on failure */
-  fail:
+  tidy:
   raptor_free_serializer_factory(serializer);
-  fail_noserializer:
-  raptor_finish();
-  RAPTOR_FATAL1("Out of memory\n");
+  return 1;
 }
 
 
 /**
  * raptor_get_serializer_factory:
+ * @world: raptor_world object
  * @name: the factory name or NULL for the default factory
  *
  * Get a serializer factory by name.
@@ -248,13 +256,13 @@ raptor_serializer_register_factory(const char *name, const char *label,
  * Return value: the factory object or NULL if there is no such factory
  **/
 static raptor_serializer_factory*
-raptor_get_serializer_factory(const char *name) 
+raptor_get_serializer_factory(raptor_world* world, const char *name) 
 {
   raptor_serializer_factory *factory;
 
   /* return 1st serializer if no particular one wanted - why? */
   if(!name) {
-    factory=(raptor_serializer_factory *)raptor_sequence_get_at(serializers, 0);
+    factory=(raptor_serializer_factory *)raptor_sequence_get_at(world->serializers, 0);
     if(!factory) {
       RAPTOR_DEBUG1("No (default) serializers registered\n");
       return NULL;
@@ -263,7 +271,7 @@ raptor_get_serializer_factory(const char *name)
     int i;
     
     for(i=0;
-        (factory=(raptor_serializer_factory*)raptor_sequence_get_at(serializers, i));
+        (factory=(raptor_serializer_factory*)raptor_sequence_get_at(world->serializers, i));
         i++) {
       if(!strcmp(factory->name, name) ||
          (factory->alias && !strcmp(factory->alias, name)))
@@ -282,6 +290,7 @@ raptor_get_serializer_factory(const char *name)
 }
 
 
+#ifndef RAPTOR_DISABLE_V1
 /**
  * raptor_serializers_enumerate:
  * @counter: index into the list of syntaxes
@@ -291,18 +300,50 @@ raptor_get_serializer_factory(const char *name)
  * @uri_string: pointer to store syntax URI string (or NULL)
  *
  * Get information on syntax serializers.
+ *
+ * raptor_init() MUST have been called before calling this function.
+ * Use raptor_serializers_enumerate_v2() if using raptor_world APIs.
+ * 
+ * Return value: non 0 on failure of if counter is out of range
+ **/
+int
+raptor_serializers_enumerate(const unsigned int counter,
+                             const char **name, const char **label,
+                             const char **mime_type,
+                             const unsigned char **uri_string)
+{
+  return raptor_serializers_enumerate_v2(raptor_world_instance(),
+                                         counter,
+                                         name, label,
+                                         mime_type,
+                                         uri_string);
+}
+#endif
+
+
+/**
+ * raptor_serializers_enumerate_v2:
+ * @world: raptor_world object
+ * @counter: index into the list of syntaxes
+ * @name: pointer to store the name of the syntax (or NULL)
+ * @label: pointer to store syntax readable label (or NULL)
+ * @mime_type: pointer to store syntax MIME Type (or NULL)
+ * @uri_string: pointer to store syntax URI string (or NULL)
+ *
+ * Get information on syntax serializers.
  * 
  * Return value: non 0 on failure of if counter is out of range
  **/
 int
-raptor_serializers_enumerate(const unsigned int counter,
-                             const char **name, const char **label,
-                             const char **mime_type,
-                             const unsigned char **uri_string)
+raptor_serializers_enumerate_v2(raptor_world* world,
+                                const unsigned int counter,
+                                const char **name, const char **label,
+                                const char **mime_type,
+                                const unsigned char **uri_string)
 {
   raptor_serializer_factory *factory;
 
-  factory=(raptor_serializer_factory*)raptor_sequence_get_at(serializers,
+  factory=(raptor_serializer_factory*)raptor_sequence_get_at(world->serializers,
                                                              counter);
 
   if(!factory)
@@ -321,36 +362,79 @@ raptor_serializers_enumerate(const unsigned int counter,
 }
 
 
+#ifndef RAPTOR_DISABLE_V1
 /**
  * raptor_serializer_syntax_name_check:
  * @name: the syntax name
  *
  * Check name of a serializer.
  *
+ * raptor_init() MUST have been called before calling this function.
+ * Use raptor_serializer_syntax_name_check_v2() if using raptor_world APIs.
+ *
+ * Return value: non 0 if name is a known syntax name
+ */
+int
+raptor_serializer_syntax_name_check(const char *name)
+{
+  return raptor_serializer_syntax_name_check_v2(raptor_world_instance(),
+                                                name);
+}
+#endif
+
+
+/**
+ * raptor_serializer_syntax_name_check_v2:
+ * @world: raptor_world object
+ * @name: the syntax name
+ *
+ * Check name of a serializer.
+ *
  * Return value: non 0 if name is a known syntax name
  */
 int
-raptor_serializer_syntax_name_check(const char *name)
+raptor_serializer_syntax_name_check_v2(raptor_world* world, const char *name)
 {
-  return (raptor_get_serializer_factory(name) != NULL);
+  return (raptor_get_serializer_factory(world, name) != NULL);
 }
 
 
+#ifndef RAPTOR_DISABLE_V1
 /**
  * raptor_new_serializer:
  * @name: the serializer name
  *
- * Constructor - create a new raptor_serializer object
+ * Constructor - create a new raptor_serializer object.
+ *
+ * raptor_init() MUST have been called before calling this function.
+ * Use raptor_new_serializer_v2() if using raptor_world APIs.
  *
  * Return value: a new #raptor_serializer object or NULL on failure
  */
 raptor_serializer*
 raptor_new_serializer(const char *name)
+{
+  return raptor_new_serializer_v2(raptor_world_instance(), name);
+}
+#endif
+
+
+/**
+ * raptor_new_serializer_v2:
+ * @world: raptor_world object
+ * @name: the serializer name
+ *
+ * Constructor - create a new raptor_serializer object.
+ *
+ * Return value: a new #raptor_serializer object or NULL on failure
+ */
+raptor_serializer*
+raptor_new_serializer_v2(raptor_world* world, const char *name)
 {
   raptor_serializer_factory* factory;
   raptor_serializer* rdf_serializer;
 
-  factory=raptor_get_serializer_factory(name);
+  factory=raptor_get_serializer_factory(world, name);
   if(!factory)
     return NULL;
 
@@ -358,6 +442,8 @@ raptor_new_serializer(const char *name)
                                            sizeof(raptor_serializer));
   if(!rdf_serializer)
     return NULL;
+
+  rdf_serializer->world=world;
   
   rdf_serializer->context=(char*)RAPTOR_CALLOC(raptor_serializer_context, 1,
                                                factory->context_length);
@@ -369,6 +455,10 @@ raptor_new_serializer(const char *name)
   rdf_serializer->factory=factory;
 
   /* Default features */
+  
+  /* Emit @base directive or equivalent */
+  rdf_serializer->feature_write_base_uri=1;
+  
   /* Emit relative URIs where possible */
   rdf_serializer->feature_relative_uris=1;
 
@@ -385,6 +475,21 @@ raptor_new_serializer(const char *name)
   /* Write XML declaration */
   rdf_serializer->feature_write_xml_declaration=1;
 
+  /* JSON callback function name */
+  rdf_serializer->feature_json_callback= NULL;
+
+  /* JSON extra data */
+  rdf_serializer->feature_json_extra_data= NULL;
+
+  /* RSS triples */
+  rdf_serializer->feature_rss_triples= NULL;
+
+  /* Atom entry URI */
+  rdf_serializer->feature_atom_entry_uri= NULL;
+
+  /* Prefix elements with a namespace */
+  rdf_serializer->feature_prefix_elements = 0;
+
   if(factory->init(rdf_serializer, name)) {
     raptor_free_serializer(rdf_serializer);
     return NULL;
@@ -394,6 +499,46 @@ raptor_new_serializer(const char *name)
 }
 
 
+/**
+ * raptor_serialize_start_to_iostream:
+ * @rdf_serializer:  the #raptor_serializer
+ * @uri: base URI or NULL if no base URI is required
+ * @iostream: #raptor_iostream to write serialization to
+ * 
+ * Start serialization to an iostream with given base URI
+ *
+ * The passed in @iostream does not becomes owned by the serializer
+ * and can be used by the caller after serializing is done.  It
+ * must be destroyed by the caller.  Compare to
+ * raptor_serialize_start() which will be deprecated in future.
+ *
+ * Return value: non-0 on failure.
+ **/
+int
+raptor_serialize_start_to_iostream(raptor_serializer *rdf_serializer,
+                                   raptor_uri *uri, raptor_iostream *iostream) 
+{
+  if(rdf_serializer->base_uri)
+    raptor_free_uri_v2(rdf_serializer->world, rdf_serializer->base_uri);
+
+  if(!iostream)
+    return 1;
+  
+  if(uri)
+    uri=raptor_uri_copy_v2(rdf_serializer->world, uri);
+  
+  rdf_serializer->base_uri=uri;
+  rdf_serializer->locator.uri=uri;
+  rdf_serializer->locator.line=rdf_serializer->locator.column = 0;
+
+  rdf_serializer->iostream=iostream;
+
+  if(rdf_serializer->factory->serialize_start)
+    return rdf_serializer->factory->serialize_start(rdf_serializer);
+  return 0;
+}
+
+
 /**
  * raptor_serialize_start:
  * @rdf_serializer:  the #raptor_serializer
@@ -402,30 +547,23 @@ raptor_new_serializer(const char *name)
  * 
  * Start serialization with given base URI
  *
+ * The passed in @iostream becomes owned by the serializer and will
+ * be destroyed when the serializing is complete.  Compare to
+ * raptor_serialize_start_to_iostream().   This function
+ * will be deprecated for raptor_serialize_start_to_iostream() in future.
+ *
  * Return value: non-0 on failure.
  **/
 int
 raptor_serialize_start(raptor_serializer *rdf_serializer, raptor_uri *uri,
                        raptor_iostream *iostream) 
 {
-  if(rdf_serializer->base_uri)
-    raptor_free_uri(rdf_serializer->base_uri);
+  int rc;
+  rc=raptor_serialize_start_to_iostream(rdf_serializer, uri, iostream);
+  if(!rc)
+    rdf_serializer->free_iostream_on_end=1;
 
-  if(!iostream)
-    return 1;
-  
-  if(uri)
-    uri=raptor_uri_copy(uri);
-  
-  rdf_serializer->base_uri=uri;
-  rdf_serializer->locator.uri=uri;
-  rdf_serializer->locator.line=rdf_serializer->locator.column = 0;
-
-  rdf_serializer->iostream=iostream;
-
-  if(rdf_serializer->factory->serialize_start)
-    return rdf_serializer->factory->serialize_start(rdf_serializer);
-  return 0;
+  return rc;
 }
 
 
@@ -447,9 +585,9 @@ raptor_serialize_start_to_filename(raptor_serializer *rdf_serializer,
     return 1;
 
   if(rdf_serializer->base_uri)
-    raptor_free_uri(rdf_serializer->base_uri);
+    raptor_free_uri_v2(rdf_serializer->world, rdf_serializer->base_uri);
 
-  rdf_serializer->base_uri=raptor_new_uri(uri_string);
+  rdf_serializer->base_uri=raptor_new_uri_v2(rdf_serializer->world, uri_string);
   rdf_serializer->locator.uri=rdf_serializer->base_uri;
   rdf_serializer->locator.line=rdf_serializer->locator.column = 0;
 
@@ -459,6 +597,8 @@ raptor_serialize_start_to_filename(raptor_serializer *rdf_serializer,
   if(!rdf_serializer->iostream)
     return 1;
 
+  rdf_serializer->free_iostream_on_end=1;
+
   if(rdf_serializer->factory->serialize_start)
     return rdf_serializer->factory->serialize_start(rdf_serializer);
   return 0;
@@ -483,10 +623,10 @@ raptor_serialize_start_to_string(raptor_serializer *rdf_serializer,
                                  void **string_p, size_t *length_p) 
 {
   if(rdf_serializer->base_uri)
-    raptor_free_uri(rdf_serializer->base_uri);
+    raptor_free_uri_v2(rdf_serializer->world, rdf_serializer->base_uri);
 
   if(uri)
-    rdf_serializer->base_uri=raptor_uri_copy(uri);
+    rdf_serializer->base_uri=raptor_uri_copy_v2(rdf_serializer->world, uri);
   else
     rdf_serializer->base_uri=NULL;
   rdf_serializer->locator.uri=rdf_serializer->base_uri;
@@ -498,6 +638,8 @@ raptor_serialize_start_to_string(raptor_serializer *rdf_serializer,
   if(!rdf_serializer->iostream)
     return 1;
 
+  rdf_serializer->free_iostream_on_end=1;
+
   if(rdf_serializer->factory->serialize_start)
     return rdf_serializer->factory->serialize_start(rdf_serializer);
   return 0;
@@ -521,10 +663,10 @@ raptor_serialize_start_to_file_handle(raptor_serializer *rdf_serializer,
                                       raptor_uri *uri, FILE *fh) 
 {
   if(rdf_serializer->base_uri)
-    raptor_free_uri(rdf_serializer->base_uri);
+    raptor_free_uri_v2(rdf_serializer->world, rdf_serializer->base_uri);
 
   if(uri)
-    rdf_serializer->base_uri=raptor_uri_copy(uri);
+    rdf_serializer->base_uri=raptor_uri_copy_v2(rdf_serializer->world, uri);
   else
     rdf_serializer->base_uri=NULL;
   rdf_serializer->locator.uri=rdf_serializer->base_uri;
@@ -534,6 +676,8 @@ raptor_serialize_start_to_file_handle(raptor_serializer *rdf_serializer,
   if(!rdf_serializer->iostream)
     return 1;
 
+  rdf_serializer->free_iostream_on_end=1;
+
   if(rdf_serializer->factory->serialize_start)
     return rdf_serializer->factory->serialize_start(rdf_serializer);
   return 0;
@@ -554,6 +698,9 @@ int
 raptor_serialize_set_namespace(raptor_serializer* rdf_serializer,
                                raptor_uri *uri, const unsigned char *prefix) 
 {
+  if(prefix && !*prefix)
+    prefix=NULL;
+  
   if(rdf_serializer->factory->declare_namespace)
     return rdf_serializer->factory->declare_namespace(rdf_serializer, 
                                                       uri, prefix);
@@ -628,7 +775,8 @@ raptor_serialize_end(raptor_serializer *rdf_serializer)
     rc=0;
 
   if(rdf_serializer->iostream) {
-    raptor_free_iostream(rdf_serializer->iostream);
+    if(rdf_serializer->free_iostream_on_end)
+      raptor_free_iostream(rdf_serializer->iostream);
     rdf_serializer->iostream=NULL;
   }
   return rc;
@@ -646,6 +794,8 @@ raptor_serialize_end(raptor_serializer *rdf_serializer)
 void
 raptor_free_serializer(raptor_serializer* rdf_serializer) 
 {
+  RAPTOR_ASSERT_OBJECT_POINTER_RETURN(rdf_serializer, raptor_serializer);
+
   if(rdf_serializer->factory)
     rdf_serializer->factory->terminate(rdf_serializer);
 
@@ -653,10 +803,40 @@ raptor_free_serializer(raptor_serializer* rdf_serializer)
     RAPTOR_FREE(raptor_serializer_context, rdf_serializer->context);
 
   if(rdf_serializer->base_uri)
-    raptor_free_uri(rdf_serializer->base_uri);
+    raptor_free_uri_v2(rdf_serializer->world, rdf_serializer->base_uri);
 
   if(rdf_serializer->feature_start_uri)
-    raptor_free_uri(rdf_serializer->feature_start_uri);
+    raptor_free_uri_v2(rdf_serializer->world, rdf_serializer->feature_start_uri);
+
+  if(rdf_serializer->feature_resource_border)
+    RAPTOR_FREE(cstring, rdf_serializer->feature_resource_border);
+  
+  if(rdf_serializer->feature_literal_border)
+    RAPTOR_FREE(cstring, rdf_serializer->feature_literal_border);
+  
+  if(rdf_serializer->feature_bnode_border)
+    RAPTOR_FREE(cstring, rdf_serializer->feature_bnode_border);
+  
+  if(rdf_serializer->feature_resource_fill)
+    RAPTOR_FREE(cstring, rdf_serializer->feature_resource_fill);
+  
+  if(rdf_serializer->feature_literal_fill)
+    RAPTOR_FREE(cstring, rdf_serializer->feature_literal_fill);
+  
+  if(rdf_serializer->feature_bnode_fill)
+    RAPTOR_FREE(cstring, rdf_serializer->feature_bnode_fill);
+  
+  if(rdf_serializer->feature_json_callback)
+    RAPTOR_FREE(cstring, rdf_serializer->feature_json_callback);
+
+  if(rdf_serializer->feature_json_extra_data)
+    RAPTOR_FREE(cstring, rdf_serializer->feature_json_extra_data);
+
+  if(rdf_serializer->feature_rss_triples)
+    RAPTOR_FREE(cstring, rdf_serializer->feature_rss_triples);
+
+  if(rdf_serializer->feature_atom_entry_uri)
+    RAPTOR_FREE(cstring, rdf_serializer->feature_atom_entry_uri);
 
   RAPTOR_FREE(raptor_serializer, rdf_serializer);
 }
@@ -677,7 +857,7 @@ raptor_serializer_get_iostream(raptor_serializer *serializer)
 }
 
 
-
+#ifndef RAPTOR_DISABLE_V1
 /**
  * raptor_serializer_features_enumerate:
  * @feature: feature enumeration (0+)
@@ -690,6 +870,9 @@ raptor_serializer_get_iostream(raptor_serializer *serializer)
  * If uri is not NULL, a pointer toa new raptor_uri is returned
  * that must be freed by the caller with raptor_free_uri().
  *
+ * raptor_init() MUST have been called before calling this function.
+ * Use raptor_serializer_features_enumerate_v2() if using raptor_world APIs.
+ *
  * Return value: 0 on success, <0 on failure, >0 if feature is unknown
  **/
 int
@@ -697,7 +880,34 @@ raptor_serializer_features_enumerate(const raptor_feature feature,
                                      const char **name, 
                                      raptor_uri **uri, const char **label)
 {
-  return raptor_features_enumerate_common(feature, name, uri, label, 2);
+  return raptor_serializer_features_enumerate_v2(raptor_world_instance(),
+                                                 feature, name, uri, label);
+}
+#endif
+
+
+/**
+ * raptor_serializer_features_enumerate_v2:
+ * @world: raptor_world object
+ * @feature: feature enumeration (0+)
+ * @name: pointer to store feature short name (or NULL)
+ * @uri: pointer to store feature URI (or NULL)
+ * @label: pointer to feature label (or NULL)
+ *
+ * Get list of serializer features.
+ * 
+ * If uri is not NULL, a pointer toa new raptor_uri is returned
+ * that must be freed by the caller with raptor_free_uri_v2().
+ *
+ * Return value: 0 on success, <0 on failure, >0 if feature is unknown
+ **/
+int
+raptor_serializer_features_enumerate_v2(raptor_world* world,
+                                        const raptor_feature feature,
+                                        const char **name, 
+                                        raptor_uri **uri, const char **label)
+{
+  return raptor_features_enumerate_common(world, feature, name, uri, label, 2);
 }
 
 
@@ -721,6 +931,10 @@ raptor_serializer_set_feature(raptor_serializer *serializer,
     return -1;
   
   switch(feature) {
+    case RAPTOR_FEATURE_WRITE_BASE_URI:
+      serializer->feature_write_base_uri=value;
+      break;
+
     case RAPTOR_FEATURE_RELATIVE_URIS:
       serializer->feature_relative_uris=value;
       break;
@@ -738,6 +952,10 @@ raptor_serializer_set_feature(raptor_serializer *serializer,
       serializer->feature_write_xml_declaration = value;
       break;
 
+    case RAPTOR_FEATURE_PREFIX_ELEMENTS:
+      serializer->feature_prefix_elements = value;
+      break;
+
     /* parser features */
     case RAPTOR_FEATURE_SCANNING:
     case RAPTOR_FEATURE_ASSUME_IS_RDF:
@@ -769,6 +987,14 @@ raptor_serializer_set_feature(raptor_serializer *serializer,
     case RAPTOR_FEATURE_RESOURCE_FILL:
     case RAPTOR_FEATURE_LITERAL_FILL:
     case RAPTOR_FEATURE_BNODE_FILL:
+    case RAPTOR_FEATURE_JSON_CALLBACK:
+    case RAPTOR_FEATURE_JSON_EXTRA_DATA:
+    case RAPTOR_FEATURE_RSS_TRIPLES:
+    case RAPTOR_FEATURE_ATOM_ENTRY_URI:
+
+    /* WWW features */
+    case RAPTOR_FEATURE_WWW_HTTP_CACHE_CONTROL:
+    case RAPTOR_FEATURE_WWW_HTTP_USER_AGENT:
 
     default:
       return -1;
@@ -825,12 +1051,14 @@ raptor_serializer_set_feature_string(raptor_serializer *serializer,
   switch(feature) {
     case RAPTOR_FEATURE_START_URI:
       if(value)
-        serializer->feature_start_uri=raptor_new_uri(value);
+        serializer->feature_start_uri=raptor_new_uri_v2(serializer->world, value);
       else
         return -1;
       break;
 
+    case RAPTOR_FEATURE_WRITE_BASE_URI:
     case RAPTOR_FEATURE_RELATIVE_URIS:
+    case RAPTOR_FEATURE_PREFIX_ELEMENTS:
       /* actually handled above because value_is_string is false */
       return -1;
       break;
@@ -887,6 +1115,31 @@ raptor_serializer_set_feature_string(raptor_serializer *serializer,
         (unsigned char **)&(serializer->feature_bnode_fill), value);
       break;
 
+    /* JSON serializer features */
+    case RAPTOR_FEATURE_JSON_CALLBACK:
+      return raptor_serializer_copy_string(
+        (unsigned char **)&(serializer->feature_json_callback), value);
+      break;
+
+    case RAPTOR_FEATURE_JSON_EXTRA_DATA:
+      return raptor_serializer_copy_string(
+        (unsigned char **)&(serializer->feature_json_extra_data), value);
+      break;
+
+    case RAPTOR_FEATURE_RSS_TRIPLES:
+      return raptor_serializer_copy_string(
+        (unsigned char **)&(serializer->feature_rss_triples), value);
+      break;
+
+    case RAPTOR_FEATURE_ATOM_ENTRY_URI:
+      return raptor_serializer_copy_string(
+        (unsigned char **)&(serializer->feature_atom_entry_uri), value);
+      break;
+
+    /* WWW features */
+    case RAPTOR_FEATURE_WWW_HTTP_CACHE_CONTROL:
+    case RAPTOR_FEATURE_WWW_HTTP_USER_AGENT:
+
     default:
       return -1;
       break;
@@ -916,6 +1169,10 @@ raptor_serializer_get_feature(raptor_serializer *serializer,
   int result= -1;
   
   switch(feature) {
+    case RAPTOR_FEATURE_WRITE_BASE_URI:
+      result=(serializer->feature_write_base_uri != 0);
+      break;
+
     case RAPTOR_FEATURE_RELATIVE_URIS:
       result=(serializer->feature_relative_uris != 0);
       break;
@@ -928,9 +1185,17 @@ raptor_serializer_get_feature(raptor_serializer *serializer,
     case RAPTOR_FEATURE_RESOURCE_FILL:
     case RAPTOR_FEATURE_LITERAL_FILL:
     case RAPTOR_FEATURE_BNODE_FILL:
+    case RAPTOR_FEATURE_JSON_CALLBACK:
+    case RAPTOR_FEATURE_JSON_EXTRA_DATA:
+    case RAPTOR_FEATURE_RSS_TRIPLES:
+    case RAPTOR_FEATURE_ATOM_ENTRY_URI:
       result= -1;
       break;
 
+    case RAPTOR_FEATURE_PREFIX_ELEMENTS:
+      result = serializer->feature_prefix_elements;
+      break;
+      
     case RAPTOR_FEATURE_WRITER_XML_VERSION:
       result=serializer->xml_version;
       break;
@@ -963,6 +1228,10 @@ raptor_serializer_get_feature(raptor_serializer *serializer,
     case RAPTOR_FEATURE_WRITER_AUTO_EMPTY:
     case RAPTOR_FEATURE_WRITER_INDENT_WIDTH:
 
+    /* WWW features */
+    case RAPTOR_FEATURE_WWW_HTTP_CACHE_CONTROL:
+    case RAPTOR_FEATURE_WWW_HTTP_USER_AGENT:
+
     default:
       break;
   }
@@ -993,9 +1262,10 @@ raptor_serializer_get_feature_string(raptor_serializer *serializer,
   switch(feature) {
     case RAPTOR_FEATURE_START_URI:
       if(serializer->feature_start_uri)
-        return raptor_uri_to_string(serializer->feature_start_uri);
+        return raptor_uri_to_string_v2(serializer->world, serializer->feature_start_uri);
       break;
 
+    case RAPTOR_FEATURE_WRITE_BASE_URI:
     case RAPTOR_FEATURE_RELATIVE_URIS:
       /* actually handled above because value_is_string is false */
       return NULL;
@@ -1020,6 +1290,21 @@ raptor_serializer_get_feature_string(raptor_serializer *serializer,
     case RAPTOR_FEATURE_BNODE_FILL:
       return (unsigned char *)(serializer->feature_bnode_fill);
       break;
+    case RAPTOR_FEATURE_JSON_CALLBACK:
+      return (unsigned char *)(serializer->feature_json_callback);
+      break;
+    case RAPTOR_FEATURE_JSON_EXTRA_DATA:
+      return (unsigned char *)(serializer->feature_json_extra_data);
+      break;
+    case RAPTOR_FEATURE_RSS_TRIPLES:
+      return (unsigned char *)(serializer->feature_rss_triples);
+      break;
+    case RAPTOR_FEATURE_ATOM_ENTRY_URI:
+      return (unsigned char *)(serializer->feature_atom_entry_uri);
+      break;
+    case RAPTOR_FEATURE_PREFIX_ELEMENTS:
+      return NULL;
+      break;
         
     /* parser features */
     case RAPTOR_FEATURE_SCANNING:
@@ -1047,6 +1332,10 @@ raptor_serializer_get_feature_string(raptor_serializer *serializer,
     case RAPTOR_FEATURE_WRITER_XML_VERSION:
     case RAPTOR_FEATURE_WRITER_XML_DECLARATION:
 
+    /* WWW features */
+    case RAPTOR_FEATURE_WWW_HTTP_CACHE_CONTROL:
+    case RAPTOR_FEATURE_WWW_HTTP_USER_AGENT:
+
     default:
       return NULL;
       break;
@@ -1115,7 +1404,7 @@ raptor_serializer_error_varargs(raptor_serializer* serializer,
     return;
   }
 
-  raptor_print_locator(stderr, &serializer->locator);
+  raptor_print_locator_v2(serializer->world, stderr, &serializer->locator);
   fprintf(stderr, " raptor error - ");
   vfprintf(stderr, message, arguments);
   fputc('\n', stderr);
@@ -1162,7 +1451,7 @@ raptor_serializer_warning_varargs(raptor_serializer* serializer, const char *mes
     return;
   }
 
-  raptor_print_locator(stderr, &serializer->locator);
+  raptor_print_locator_v2(serializer->world, stderr, &serializer->locator);
   fprintf(stderr, " raptor warning - ");
   vfprintf(stderr, message, arguments);
   fputc('\n', stderr);
@@ -1224,3 +1513,18 @@ raptor_serializer_get_locator(raptor_serializer *rdf_serializer)
 {
   return &rdf_serializer->locator;
 }
+
+
+/**
+ * raptor_serializer_get_world:
+ * @rdf_serializer: raptor serializer
+ * 
+ * Get the #raptor_world object associated with a serializer.
+ *
+ * Return value: raptor_world* pointer
+ **/
+raptor_world *
+raptor_serializer_get_world(raptor_serializer* rdf_serializer)
+{
+  return rdf_serializer->world;
+}
