@@ -2,7 +2,7 @@
  *
  * raptor_serialize_rdfxml.c - RDF/XML serializer
  *
- * Copyright (C) 2004-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2004-2008, David Beckett http://www.dajobe.org/
  * Copyright (C) 2004-2005, University of Bristol, UK http://www.bristol.ac.uk/
  *
  * This package is Free Software and part of Redland http://librdf.org/
@@ -88,14 +88,11 @@ static int
 raptor_rdfxml_serialize_init(raptor_serializer* serializer, const char *name)
 {
   raptor_rdfxml_serializer_context* context=(raptor_rdfxml_serializer_context*)serializer->context;
-  const raptor_uri_handler *uri_handler;
-  void *uri_context;
 
-  raptor_uri_get_handler(&uri_handler, &uri_context);
-  context->nstack=raptor_new_namespaces(uri_handler, uri_context,
-                                        (raptor_simple_message_handler)raptor_serializer_simple_error,
-                                        serializer,
-                                        1);
+  context->nstack=raptor_new_namespaces_v2(serializer->world,
+                                           (raptor_simple_message_handler)raptor_serializer_simple_error,
+                                           serializer,
+                                           1);
   if(!context->nstack)
     return 1;
   context->xml_nspace=raptor_new_namespace(context->nstack,
@@ -110,7 +107,7 @@ raptor_rdfxml_serialize_init(raptor_serializer* serializer, const char *name)
 
   context->namespaces=raptor_new_sequence(NULL, NULL);
 
-  context->rdf_xml_literal_uri=raptor_new_uri(raptor_xml_literal_datatype_uri_string);
+  context->rdf_xml_literal_uri=raptor_new_uri_v2(serializer->world, raptor_xml_literal_datatype_uri_string);
 
   if(!context->xml_nspace || !context->rdf_nspace || !context->namespaces ||
      !context->rdf_xml_literal_uri) {
@@ -155,7 +152,7 @@ raptor_rdfxml_serialize_terminate(raptor_serializer* serializer)
   }
 
   if(context->rdf_xml_literal_uri) {
-    raptor_free_uri(context->rdf_xml_literal_uri);
+    raptor_free_uri_v2(serializer->world, context->rdf_xml_literal_uri);
     context->rdf_xml_literal_uri=NULL;
   }
 
@@ -205,7 +202,7 @@ raptor_rdfxml_serialize_declare_namespace_from_namespace(raptor_serializer* seri
       return 1;
 
     if(ns->uri && nspace->uri &&
-       raptor_uri_equals(ns->uri, nspace->uri))
+       raptor_uri_equals_v2(serializer->world, ns->uri, nspace->uri))
       return 1;
   }
 
@@ -247,22 +244,18 @@ raptor_rdfxml_serialize_start(raptor_serializer* serializer)
 {
   raptor_rdfxml_serializer_context* context=(raptor_rdfxml_serializer_context*)serializer->context;
   raptor_xml_writer* xml_writer;
-  const raptor_uri_handler *uri_handler;
-  void *uri_context;
-
-  raptor_uri_get_handler(&uri_handler, &uri_context);
 
   if(context->xml_writer) {
     raptor_free_xml_writer(context->xml_writer);
     context->xml_writer=NULL;
   }
 
-  xml_writer=raptor_new_xml_writer(context->nstack,
-                                   uri_handler, uri_context,
-                                   serializer->iostream,
-                                   (raptor_simple_message_handler)raptor_serializer_simple_error,
-                                   serializer,
-                                   1);
+  xml_writer=raptor_new_xml_writer_v2(serializer->world,
+                                      context->nstack,
+                                      serializer->iostream,
+                                      (raptor_simple_message_handler)raptor_serializer_simple_error,
+                                      serializer,
+                                      1);
   if(!xml_writer)
     return 1;
   raptor_xml_writer_set_feature(xml_writer, RAPTOR_FEATURE_WRITER_XML_VERSION,
@@ -298,7 +291,7 @@ raptor_rdfxml_ensure_writen_header(raptor_serializer* serializer,
 
   base_uri=serializer->base_uri;
   if(base_uri)
-    base_uri=raptor_uri_copy(base_uri);
+    base_uri=raptor_uri_copy_v2(serializer->world, base_uri);
 
   context->rdf_RDF_element=raptor_new_xml_element_from_namespace_local_name(context->rdf_nspace, (const unsigned char*)"RDF", NULL, base_uri);
   if(!context->rdf_RDF_element)
@@ -313,15 +306,15 @@ raptor_rdfxml_ensure_writen_header(raptor_serializer* serializer,
       goto tidy;
   }
 
-  if(base_uri) {
+  if(base_uri && serializer->feature_write_base_uri) {
     const unsigned char* base_uri_string;
 
     attrs=(raptor_qname **)RAPTOR_CALLOC(qnamearray, 1, sizeof(raptor_qname*));
     if(!attrs)
       goto tidy;
 
-    base_uri_string=raptor_uri_as_string(base_uri);
-    attrs[attrs_count]=raptor_new_qname_from_namespace_local_name(context->xml_nspace, (const unsigned char*)"base",  base_uri_string);
+    base_uri_string=raptor_uri_as_string_v2(serializer->world, base_uri);
+    attrs[attrs_count]=raptor_new_qname_from_namespace_local_name_v2(serializer->world, context->xml_nspace, (const unsigned char*)"base",  base_uri_string);
     if(!attrs[attrs_count]) {
       RAPTOR_FREE(qnamearray, attrs);
       goto tidy;
@@ -343,7 +336,7 @@ raptor_rdfxml_ensure_writen_header(raptor_serializer* serializer,
 
   tidy:
   if(base_uri)
-    raptor_free_uri(base_uri);
+    raptor_free_uri_v2(serializer->world, base_uri);
 
   return rc;
 }
@@ -391,8 +384,8 @@ raptor_rdfxml_serialize_statement(raptor_serializer* serializer,
     unsigned char c;
 
     /* Do not use raptor_uri_as_counted_string() - we want a modifiable copy */
-    uri_string=raptor_uri_to_counted_string((raptor_uri*)statement->predicate,
-                                            &uri_len);
+    uri_string=raptor_uri_to_counted_string_v2(serializer->world, (raptor_uri*)statement->predicate,
+                                               &uri_len);
     if(!uri_string)
       goto oom;
 
@@ -414,7 +407,7 @@ raptor_rdfxml_serialize_statement(raptor_serializer* serializer,
     }
 
     c=*name; *name='\0';
-    predicate_ns_uri=raptor_new_uri(uri_string);
+    predicate_ns_uri=raptor_new_uri_v2(serializer->world, uri_string);
     if(!predicate_ns_uri)
       goto oom;
     *name=c;
@@ -426,12 +419,12 @@ raptor_rdfxml_serialize_statement(raptor_serializer* serializer,
                                                  nsprefix,
                                                  predicate_ns_uri, 0);
       if(!predicate_ns) {
-        raptor_free_uri(predicate_ns_uri);
+        raptor_free_uri_v2(serializer->world, predicate_ns_uri);
         goto oom;
       }
       free_predicate_ns=1;
     }
-    raptor_free_uri(predicate_ns_uri);
+    raptor_free_uri_v2(serializer->world, predicate_ns_uri);
   } else {
     raptor_serializer_error(serializer, "Cannot serialize a triple with subject node type %d\n", statement->predicate_type);
     goto tidy;
@@ -439,7 +432,7 @@ raptor_rdfxml_serialize_statement(raptor_serializer* serializer,
 
   /* base uri */
   if(serializer->base_uri)
-    base_uri=raptor_uri_copy(serializer->base_uri);
+    base_uri=raptor_uri_copy_v2(serializer->world, serializer->base_uri);
 
 
   rdf_Description_element=raptor_new_xml_element_from_namespace_local_name(context->rdf_nspace,
@@ -456,7 +449,7 @@ raptor_rdfxml_serialize_statement(raptor_serializer* serializer,
   /* subject */
   switch(statement->subject_type) {
     case RAPTOR_IDENTIFIER_TYPE_ANONYMOUS:
-      attrs[attrs_count]=raptor_new_qname_from_namespace_local_name(context->rdf_nspace, (const unsigned char*)"nodeID",  (unsigned char*)statement->subject);
+      attrs[attrs_count]=raptor_new_qname_from_namespace_local_name_v2(serializer->world, context->rdf_nspace, (const unsigned char*)"nodeID",  (unsigned char*)statement->subject);
       if(!attrs[attrs_count])
         goto oom;
       attrs_count++;
@@ -473,17 +466,18 @@ raptor_rdfxml_serialize_statement(raptor_serializer* serializer,
                 raptor_rdf_namespace_uri, *((int*)statement->subject));
       } else {
         if(serializer->feature_relative_uris) {
-          subject_uri_string=raptor_uri_to_relative_uri_string(serializer->base_uri,
-                                                              (raptor_uri*)statement->subject);
+          subject_uri_string=raptor_uri_to_relative_uri_string_v2(serializer->world,
+                                                                  serializer->base_uri,
+                                                                  (raptor_uri*)statement->subject);
           if(!subject_uri_string)
             goto oom;
         } else {
-          subject_uri_string=raptor_uri_as_string((raptor_uri*)statement->subject);
+          subject_uri_string=raptor_uri_as_string_v2(serializer->world, (raptor_uri*)statement->subject);
           allocated=0;
         }
       }
 
-      attrs[attrs_count]=raptor_new_qname_from_namespace_local_name(context->rdf_nspace, (const unsigned char*)"about",  subject_uri_string);
+      attrs[attrs_count]=raptor_new_qname_from_namespace_local_name_v2(serializer->world, context->rdf_nspace, (const unsigned char*)"about",  subject_uri_string);
       if(!attrs[attrs_count]) {
         if(allocated)
           RAPTOR_FREE(cstring, subject_uri_string);
@@ -532,7 +526,7 @@ raptor_rdfxml_serialize_statement(raptor_serializer* serializer,
   switch(object_type) {
     case RAPTOR_IDENTIFIER_TYPE_LITERAL:
       if(statement->object_literal_datatype &&
-         raptor_uri_equals(statement->object_literal_datatype,
+         raptor_uri_equals_v2(serializer->world, statement->object_literal_datatype,
                            context->rdf_xml_literal_uri))
         object_type = RAPTOR_IDENTIFIER_TYPE_XML_LITERAL;
 
@@ -552,7 +546,7 @@ raptor_rdfxml_serialize_statement(raptor_serializer* serializer,
       len=strlen((const char*)statement->object);
 
       if(object_type == RAPTOR_IDENTIFIER_TYPE_XML_LITERAL) {
-        attrs[attrs_count]=raptor_new_qname_from_namespace_local_name(context->rdf_nspace, (const unsigned char*)"parseType", (const unsigned char*)"Literal");
+        attrs[attrs_count]=raptor_new_qname_from_namespace_local_name_v2(serializer->world, context->rdf_nspace, (const unsigned char*)"parseType", (const unsigned char*)"Literal");
         if(!attrs[attrs_count])
           goto oom;
         attrs_count++;
@@ -571,7 +565,7 @@ raptor_rdfxml_serialize_statement(raptor_serializer* serializer,
                                         len);
       } else {
         if(statement->object_literal_datatype) {
-          attrs[attrs_count]=raptor_new_qname_from_namespace_local_name(context->rdf_nspace, (const unsigned char*)"datatype", (unsigned char*)raptor_uri_as_string((raptor_uri*)statement->object_literal_datatype));
+          attrs[attrs_count]=raptor_new_qname_from_namespace_local_name_v2(serializer->world, context->rdf_nspace, (const unsigned char*)"datatype", (unsigned char*)raptor_uri_as_string_v2(serializer->world, (raptor_uri*)statement->object_literal_datatype));
           if(!attrs[attrs_count])
             goto oom;
           attrs_count++;
@@ -598,7 +592,7 @@ raptor_rdfxml_serialize_statement(raptor_serializer* serializer,
       break;
 
     case RAPTOR_IDENTIFIER_TYPE_ANONYMOUS:
-      attrs[attrs_count]=raptor_new_qname_from_namespace_local_name(context->rdf_nspace, (const unsigned char*)"nodeID", (unsigned char*)statement->object);
+      attrs[attrs_count]=raptor_new_qname_from_namespace_local_name_v2(serializer->world, context->rdf_nspace, (const unsigned char*)"nodeID", (unsigned char*)statement->object);
       if(!attrs[attrs_count])
         goto oom;
       attrs_count++;
@@ -624,17 +618,18 @@ raptor_rdfxml_serialize_statement(raptor_serializer* serializer,
       } else {
         /* must be URI */
         if(serializer->feature_relative_uris) {
-          object_uri_string=raptor_uri_to_relative_uri_string(serializer->base_uri,
-                                                              (raptor_uri*)statement->object);
+          object_uri_string=raptor_uri_to_relative_uri_string_v2(serializer->world,
+                                                                 serializer->base_uri,
+                                                                 (raptor_uri*)statement->object);
           if(!object_uri_string)
             goto oom;
         } else {
-          object_uri_string=raptor_uri_to_string((raptor_uri*)statement->object);
+          object_uri_string=raptor_uri_to_string_v2(serializer->world, (raptor_uri*)statement->object);
           allocated=0;
         }
       }
 
-      attrs[attrs_count]=raptor_new_qname_from_namespace_local_name(context->rdf_nspace, (const unsigned char*)"resource", object_uri_string);
+      attrs[attrs_count]=raptor_new_qname_from_namespace_local_name_v2(serializer->world, context->rdf_nspace, (const unsigned char*)"resource", object_uri_string);
       if(!attrs[attrs_count]) {
         if(allocated)
           RAPTOR_FREE(cstring, object_uri_string);
@@ -687,7 +682,7 @@ raptor_rdfxml_serialize_statement(raptor_serializer* serializer,
   }
 
   if(base_uri)
-    raptor_free_uri(base_uri);
+    raptor_free_uri_v2(serializer->world, base_uri);
 
   if(free_predicate_ns)
     raptor_free_namespace(predicate_ns);
@@ -718,6 +713,8 @@ raptor_rdfxml_serialize_end(raptor_serializer* serializer)
       raptor_xml_writer_end_element(xml_writer, context->rdf_RDF_element);
       raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"\n", 1);
     }
+
+    raptor_xml_writer_flush(xml_writer);
   }
 
   if(context->rdf_RDF_element) {
@@ -756,8 +753,9 @@ raptor_rdfxml_serializer_register_factory(raptor_serializer_factory *factory)
 
 
 int
-raptor_init_serializer_rdfxml(void) {
-  return raptor_serializer_register_factory("rdfxml", "RDF/XML",
+raptor_init_serializer_rdfxml(raptor_world* world) {
+  return raptor_serializer_register_factory(world,
+                                            "rdfxml", "RDF/XML",
                                             "application/rdf+xml",
                                             NULL,
                                             (const unsigned char*)"http://www.w3.org/TR/rdf-syntax-grammar",
