@@ -1,8 +1,8 @@
 /* -*- Mode: c; c-basic-offset: 2 -*-
  *
- * raptor_serialize_rdfxml.c - Simple serializer
+ * raptor_serialize_simple.c - Simple serializer
  *
- * Copyright (C) 2004-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2004-2006, David Beckett http://www.dajobe.org/
  * Copyright (C) 2004-2005, University of Bristol, UK http://www.bristol.ac.uk/
  *
  * This package is Free Software and part of Redland http://librdf.org/
@@ -91,7 +91,7 @@ raptor_simple_serialize_statement(raptor_serializer* serializer,
     if(!statement->subject)
       RAPTOR_FATAL1("Statement has NULL subject URI\n");
 #endif
-    raptor_iostream_write_uri(iostr, (raptor_uri*)statement->subject);
+    raptor_iostream_write_uri_v2(serializer->world, iostr, (raptor_uri*)statement->subject);
   }
 
   raptor_iostream_write_counted_string(iostr, ", ", 2);
@@ -105,7 +105,7 @@ raptor_simple_serialize_statement(raptor_serializer* serializer,
     if(!statement->predicate)
       RAPTOR_FATAL1("Statement has NULL predicate URI\n");
 #endif
-    raptor_iostream_write_uri(iostr, (raptor_uri*)statement->predicate);
+    raptor_iostream_write_uri_v2(serializer->world, iostr, (raptor_uri*)statement->predicate);
   }
 
   raptor_iostream_write_counted_string(iostr, ", ", 2);
@@ -118,7 +118,7 @@ raptor_simple_serialize_statement(raptor_serializer* serializer,
       raptor_iostream_write_byte(iostr, '>');
     } else if(statement->object_literal_datatype) {
       raptor_iostream_write_byte(iostr, '<');
-      raptor_iostream_write_uri(iostr, (raptor_uri*)statement->object_literal_datatype);
+      raptor_iostream_write_uri_v2(serializer->world, iostr, (raptor_uri*)statement->object_literal_datatype);
       raptor_iostream_write_byte(iostr, '>');
     }
     raptor_iostream_write_byte(iostr, '"');
@@ -135,7 +135,7 @@ raptor_simple_serialize_statement(raptor_serializer* serializer,
     if(!statement->object)
       RAPTOR_FATAL1("Statement has NULL object URI\n");
 #endif
-    raptor_iostream_write_uri(iostr, (raptor_uri*)statement->object);
+    raptor_iostream_write_uri_v2(serializer->world, iostr, (raptor_uri*)statement->object);
   }
 
   raptor_iostream_write_counted_string(iostr, "]\n", 2);
@@ -171,8 +171,9 @@ raptor_simple_serializer_register_factory(raptor_serializer_factory *factory)
 
 
 int
-raptor_init_serializer_simple (void) {
-  return raptor_serializer_register_factory("simple",  "A simple format", 
+raptor_init_serializer_simple(raptor_world* world) {
+  return raptor_serializer_register_factory(world,
+                                            "simple", "A simple format", 
                                             NULL,
                                             NULL,
                                             NULL,
