@@ -2,7 +2,7 @@
  *
  * raptor_set.c - Sets for checking IDs
  *
- * Copyright (C) 2003-2007, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2003-2008, David Beckett http://www.dajobe.org/
  * Copyright (C) 2003-2004, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -59,6 +59,8 @@
 
 struct raptor_base_id_set_s
 {
+  raptor_world* world;
+
   /* The base URI of this set of IDs */
   raptor_uri *uri;
   
@@ -74,6 +76,8 @@ typedef struct raptor_base_id_set_s raptor_base_id_set;
 
 struct raptor_id_set_s
 {
+  raptor_world* world;
+
   /* start of trees, 1 per base URI */
   struct raptor_base_id_set_s* first;
 
@@ -88,19 +92,22 @@ struct raptor_id_set_s
 
 /**
  * raptor_new_id_set:
+ * @world: raptor_world object
  *
- * Constructor - create a new ID set.
+ * INTERNAL - Constructor - create a new ID set.
  * 
  * Return value: non 0 on failure
  **/
 raptor_id_set*
-raptor_new_id_set(void) 
+raptor_new_id_set(raptor_world* world)
 {
   raptor_id_set* set=(raptor_id_set*)RAPTOR_CALLOC(raptor_id_set, 1, 
                                                    sizeof(raptor_id_set));
   if(!set)
     return NULL;
-  
+
+  set->world=world;
+
   return set;
 }
 
@@ -109,7 +116,7 @@ raptor_new_id_set(void)
  * raptor_free_base_id_set:
  * @set: #raptor_base_id_set
  *
- * Destructor - destroy an raptor_base_id_set.
+ * INTERNAL - Destructor - Free a Base ID Set.
  *
  **/
 static void
@@ -118,7 +125,7 @@ raptor_free_base_id_set(raptor_base_id_set *base)
   if(base->tree)
     raptor_free_avltree(base->tree);
   if(base->uri)
-    raptor_free_uri(base->uri);
+    raptor_free_uri_v2(base->world, base->uri);
   RAPTOR_FREE(raptor_base_id_set, base);
 }
 
@@ -127,13 +134,17 @@ raptor_free_base_id_set(raptor_base_id_set *base)
  * raptor_free_id_set:
  * @set: #raptor_id_set
  *
- * Destroy a set.
+ * INTERNAL - Destructor - Free ID Set.
  *
  **/
 void
 raptor_free_id_set(raptor_id_set *set) 
 {
-  raptor_base_id_set *base=set->first;
+  raptor_base_id_set *base;
+
+  RAPTOR_ASSERT_OBJECT_POINTER_RETURN(set, raptor_id_set);
+
+  base=set->first;
   while(base) {
     raptor_base_id_set *next=base->next;
     raptor_free_base_id_set(base);
@@ -151,7 +162,7 @@ raptor_free_id_set(raptor_id_set *set)
  * @id: identifier name
  * @id_len: length of identifier
  *
- * Add an item to the set.
+ * INTERNAL - Add an item to the set.
  * 
  * Return value: <0 on failure, 0 on success, 1 if already present
  **/
@@ -167,7 +178,7 @@ raptor_id_set_add(raptor_id_set* set, raptor_uri *base_uri,
 
   base=set->first;
   while(base) {
-    if(raptor_uri_equals(base->uri, base_uri))
+    if(raptor_uri_equals_v2(set->world, base->uri, base_uri))
       break;
     base=base->next;
   }
@@ -179,9 +190,12 @@ raptor_id_set_add(raptor_id_set* set, raptor_uri *base_uri,
     if(!base)
       return -1;
 
-    base->uri=raptor_uri_copy(base_uri);
+    base->world=set->world;
 
-    base->tree=raptor_new_avltree((raptor_avltree_compare_function)strcmp,
+    base->uri=raptor_uri_copy_v2(set->world, base_uri);
+
+    base->tree=raptor_new_avltree(set->world,
+                                  (raptor_data_compare_function)strcmp,
                                   free, 0);
   
     /* Add to the start of the list */
@@ -205,7 +219,7 @@ raptor_id_set_add(raptor_id_set* set, raptor_uri *base_uri,
     }
   }
   
-  item=raptor_avltree_search(base->tree, (raptor_avltree_t)id);
+  item=(char*)raptor_avltree_search(base->tree, id);
 
   /* if already there, error */
   if(item) {
@@ -219,7 +233,7 @@ raptor_id_set_add(raptor_id_set* set, raptor_uri *base_uri,
   set->hits++;
 #endif
   
-  item=RAPTOR_MALLOC(cstring, id_len+1);
+  item=(char*)RAPTOR_MALLOC(cstring, id_len+1);
   if(!item)
     return 1;
 
@@ -248,21 +262,24 @@ int main(int argc, char *argv[]);
 int
 main(int argc, char *argv[]) 
 {
+  raptor_world *world;
   const char *program=raptor_basename(argv[0]);
   const char *items[8] = { "ron", "amy", "jen", "bij", "jib", "daj", "jim", NULL };
   raptor_id_set *set;
   raptor_uri *base_uri;
   int i=0;
   
-  raptor_init();
-  
-  base_uri=raptor_new_uri((const unsigned char*)"http://example.org/base#");
+  world = raptor_new_world();
+  if(!world || raptor_world_open(world))
+    exit(1);
+    
+  base_uri=raptor_new_uri_v2(world, (const unsigned char*)"http://example.org/base#");
 
 #if RAPTOR_DEBUG > 1
   fprintf(stderr, "%s: Creating set\n", program);
 #endif
 
-  set=raptor_new_id_set();
+  set=raptor_new_id_set(world);
   if(!set) {
     fprintf(stderr, "%s: Failed to create set\n", program);
     exit(1);
@@ -309,9 +326,9 @@ if(rc) {
 #endif
   raptor_free_id_set(set);
 
-  raptor_free_uri(base_uri);
+  raptor_free_uri_v2(world, base_uri);
   
-  raptor_finish();
+  raptor_free_world(world);
   
   /* keep gcc -Wall happy */
   return(0);
