@@ -2,7 +2,7 @@
  *
  * raptor_stringbuffer.c - Stringbuffer class for growing strings
  *
- * Copyright (C) 2003-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2003-2008, David Beckett http://www.dajobe.org/
  * Copyright (C) 2003-2004, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -84,16 +84,12 @@ static int raptor_stringbuffer_append_string_common(raptor_stringbuffer* stringb
  *
  * Create a new stringbuffer.
  * 
- * Return value: non 0 on failure
+ * Return value: pointer to a raptor_stringbuffer object or NULL on failure
  **/
 raptor_stringbuffer*
 raptor_new_stringbuffer(void) 
 {
-  raptor_stringbuffer* sb=(raptor_stringbuffer*)RAPTOR_CALLOC(raptor_stringbuffer, 1, sizeof(raptor_stringbuffer));
-  if(!sb)
-    return NULL;
-
-  return sb;
+  return (raptor_stringbuffer*)RAPTOR_CALLOC(raptor_stringbuffer, 1, sizeof(raptor_stringbuffer));
 }
 
 
@@ -107,6 +103,8 @@ raptor_new_stringbuffer(void)
 void
 raptor_free_stringbuffer(raptor_stringbuffer *stringbuffer) 
 {
+  RAPTOR_ASSERT_OBJECT_POINTER_RETURN(stringbuffer, raptor_stringbuffer);
+
   if(stringbuffer->head) {
     raptor_stringbuffer_node *node=stringbuffer->head;
   
@@ -159,8 +157,11 @@ raptor_stringbuffer_append_string_common(raptor_stringbuffer* stringbuffer,
     return 0;
   
   node=(raptor_stringbuffer_node*)RAPTOR_MALLOC(raptor_stringbuffer_node, sizeof(raptor_stringbuffer_node));
-  if(!node)
+  if(!node) {
+    if(!do_copy)
+      RAPTOR_FREE(cstring, string);
     return 1;
+  }
 
   if(do_copy) {
     /* Note this copy does not include the \0 character - not needed  */
