@@ -3,7 +3,7 @@
  * raptor_turtle_writer.c - Raptor Turtle Writer
  *
  * Copyright (C) 2006, Dave Robillard
- * Copyright (C) 2003-2008, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2003-2008, David Beckett http://www.dajobe.org/
  * Copyright (C) 2003-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -63,7 +63,7 @@ typedef enum {
 #define TURTLE_WRITER_AUTO_INDENT(turtle_writer) ((turtle_writer->flags & TURTLE_WRITER_AUTO_INDENT) != 0)
 
 struct raptor_turtle_writer_s {
-  int canonicalize;
+  raptor_world* world;
 
   int depth;
  
@@ -73,9 +73,6 @@ struct raptor_turtle_writer_s {
   raptor_namespace_stack *nstack;
   int nstack_depth;
 
-  const raptor_uri_handler *uri_handler;
-  void *uri_context;
-
   raptor_simple_message_handler error_handler;
   void *error_data;
 
@@ -146,11 +143,10 @@ raptor_turtle_writer_newline(raptor_turtle_writer *turtle_writer)
 
 /**
  * raptor_new_turtle_writer:
+ * @world: raptor_world object
  * @base_uri: Base URI for the writer
- * @write_base_uri: non-0 to write 'base' in output
+ * @write_base_uri: non-0 to write '@base' directive to output
  * @nstack: Namespace stack for the writer to start with (or NULL)
- * @uri_handler: URI handler function
- * @uri_context: URI handler context data
  * @iostr: I/O stream to write to
  * @error_handler: error handler function
  * @error_data: error handler data
@@ -160,10 +156,9 @@ raptor_turtle_writer_newline(raptor_turtle_writer *turtle_writer)
  * Return value: a new #raptor_turtle_writer object or NULL on failure
  **/
 raptor_turtle_writer*
-raptor_new_turtle_writer(raptor_uri* base_uri, int write_base_uri,
+raptor_new_turtle_writer(raptor_world* world,
+                         raptor_uri* base_uri, int write_base_uri,
                          raptor_namespace_stack *nstack,
-                         const raptor_uri_handler *uri_handler,
-                         void *uri_context,
                          raptor_iostream* iostr,
                          raptor_simple_message_handler error_handler,
                          void *error_data)
@@ -173,21 +168,19 @@ raptor_new_turtle_writer(raptor_uri* base_uri, int write_base_uri,
   if(!turtle_writer)
     return NULL;
 
-  turtle_writer->nstack_depth=0;
+  turtle_writer->world=world;
 
-  turtle_writer->uri_handler=uri_handler;
-  turtle_writer->uri_context=uri_context;
+  turtle_writer->nstack_depth=0;
 
   turtle_writer->error_handler=error_handler;
   turtle_writer->error_data=error_data;
 
   turtle_writer->nstack=nstack;
   if(!turtle_writer->nstack) {
-    turtle_writer->nstack=nstack=raptor_new_namespaces(uri_handler,
-                                                       uri_context,
-                                                       error_handler,
-                                                       error_data,
-                                                       1);
+    turtle_writer->nstack=nstack=raptor_new_namespaces_v2(world,
+                                                          error_handler,
+                                                          error_data,
+                                                          1);
     turtle_writer->my_nstack=1;
   }
 
@@ -196,13 +189,16 @@ raptor_new_turtle_writer(raptor_uri* base_uri, int write_base_uri,
   turtle_writer->flags = 0;
   turtle_writer->indent = 2;
 
+  turtle_writer->base_uri = NULL;
+  /* Ensure any initial base URI is not written relative */
   if(base_uri && write_base_uri)
     raptor_turtle_writer_base(turtle_writer, base_uri);
+  turtle_writer->base_uri = base_uri;
 
-  turtle_writer->xsd_boolean_uri=raptor_new_uri((const unsigned char*)"http://www.w3.org/2001/XMLSchema#boolean");
-  turtle_writer->xsd_decimal_uri=raptor_new_uri((const unsigned char*)"http://www.w3.org/2001/XMLSchema#decimal");
-  turtle_writer->xsd_double_uri=raptor_new_uri((const unsigned char*)"http://www.w3.org/2001/XMLSchema#double");
-  turtle_writer->xsd_integer_uri=raptor_new_uri((const unsigned char*)"http://www.w3.org/2001/XMLSchema#integer");
+  turtle_writer->xsd_boolean_uri=raptor_new_uri_v2(world, (const unsigned char*)"http://www.w3.org/2001/XMLSchema#boolean");
+  turtle_writer->xsd_decimal_uri=raptor_new_uri_v2(world, (const unsigned char*)"http://www.w3.org/2001/XMLSchema#decimal");
+  turtle_writer->xsd_double_uri=raptor_new_uri_v2(world, (const unsigned char*)"http://www.w3.org/2001/XMLSchema#double");
+  turtle_writer->xsd_integer_uri=raptor_new_uri_v2(world, (const unsigned char*)"http://www.w3.org/2001/XMLSchema#integer");
   
   return turtle_writer;
 }
@@ -222,13 +218,13 @@ raptor_free_turtle_writer(raptor_turtle_writer* turtle_writer)
     raptor_free_namespaces(turtle_writer->nstack);
 
   if(turtle_writer->xsd_boolean_uri)
-    raptor_free_uri(turtle_writer->xsd_boolean_uri);
+    raptor_free_uri_v2(turtle_writer->world, turtle_writer->xsd_boolean_uri);
   if(turtle_writer->xsd_decimal_uri)
-    raptor_free_uri(turtle_writer->xsd_decimal_uri);
+    raptor_free_uri_v2(turtle_writer->world, turtle_writer->xsd_decimal_uri);
   if(turtle_writer->xsd_double_uri)
-    raptor_free_uri(turtle_writer->xsd_double_uri);
+    raptor_free_uri_v2(turtle_writer->world, turtle_writer->xsd_double_uri);
   if(turtle_writer->xsd_integer_uri)
-    raptor_free_uri(turtle_writer->xsd_integer_uri);
+    raptor_free_uri_v2(turtle_writer->world, turtle_writer->xsd_integer_uri);
 
   RAPTOR_FREE(raptor_turtle_writer, turtle_writer);
 }
@@ -319,10 +315,6 @@ raptor_turtle_writer_base(raptor_turtle_writer* turtle_writer,
     raptor_turtle_writer_reference(turtle_writer, base_uri);
     raptor_iostream_write_counted_string(turtle_writer->iostr, " .\n", 3);
   }
-  
-  if(turtle_writer->base_uri)
-    raptor_free_uri(turtle_writer->base_uri);
-  turtle_writer->base_uri=base_uri;
 }
 
 
@@ -341,7 +333,7 @@ raptor_turtle_writer_reference(raptor_turtle_writer* turtle_writer,
   unsigned char* uri_str;
   size_t length;
   
-  uri_str = raptor_uri_to_relative_counted_uri_string(turtle_writer->base_uri, uri, &length);
+  uri_str = raptor_uri_to_relative_counted_uri_string_v2(turtle_writer->world, turtle_writer->base_uri, uri, &length);
 
   raptor_iostream_write_byte(turtle_writer->iostr, '<');
   if(uri_str)
@@ -484,6 +476,7 @@ raptor_iostream_write_string_python(raptor_iostream *iostr,
 }
 
 
+#ifndef RAPTOR_DISABLE_DEPRECATED
 /**
  * raptor_iostream_write_string_turtle:
  * @iostr: #raptor_iostream to write to
@@ -493,6 +486,7 @@ raptor_iostream_write_string_python(raptor_iostream *iostr,
  * Write an UTF-8 string using Turtle "longString" triple quoting to
  * an iostream.
  *
+ * @deprecated: use raptor_iostream_write_string_python() instead
  **/
 void
 raptor_iostream_write_string_turtle(raptor_iostream *iostr,
@@ -500,7 +494,7 @@ raptor_iostream_write_string_turtle(raptor_iostream *iostr,
 {
   raptor_iostream_write_string_python(iostr, string, len, '"', 1);
 }
-
+#endif
 
 
 /**
@@ -562,18 +556,38 @@ raptor_turtle_writer_literal(raptor_turtle_writer* turtle_writer,
   /* DBL_EPSILON = 52 digits */
   #define FRAC_MAX_LEN 52
 
+  char* endptr = (char *)s;
   int written = 0;
 
   /* typed literal special cases */
   if(datatype) {
-    /* integer, double, decimal */
-    if(raptor_uri_equals(datatype, turtle_writer->xsd_integer_uri) ||
-       raptor_uri_equals(datatype, turtle_writer->xsd_double_uri) ||
-       raptor_uri_equals(datatype, turtle_writer->xsd_decimal_uri)) {
-      raptor_iostream_write_string(turtle_writer->iostr, s);
-      written = 1;
+    /* integer */
+    if(raptor_uri_equals_v2(turtle_writer->world, datatype, turtle_writer->xsd_integer_uri)) {
+      /* FIXME. Work around that gcc < 4.5 cannot disable warn_unused_result */
+      long gcc_is_stupid = strtol((const char*)s, &endptr, 10);
+      if(endptr != (char*)s && !*endptr) {
+        raptor_iostream_write_string(turtle_writer->iostr, s);
+        /* More gcc madness to 'use' the variable I didn't want */
+        written = 1 + 0 * (int)gcc_is_stupid;
+      } else {
+        turtle_writer->error_handler(turtle_writer->error_data, "Illegal value for xsd:integer literal.");
+      }
+
+    /* double, decimal */
+    } else if(raptor_uri_equals_v2(turtle_writer->world, datatype, turtle_writer->xsd_double_uri) ||
+      raptor_uri_equals_v2(turtle_writer->world, datatype, turtle_writer->xsd_decimal_uri)) {
+      /* FIXME. Work around that gcc < 4.5 cannot disable warn_unused_result */
+      double gcc_is_doubly_stupid = strtod((const char*)s, &endptr);
+      if(endptr != (char*)s && !*endptr) {
+        raptor_iostream_write_string(turtle_writer->iostr, s);
+        /* More gcc madness to 'use' the variable I didn't want */
+        written = 1 +  0 * (int)gcc_is_doubly_stupid;
+      } else {
+        turtle_writer->error_handler(turtle_writer->error_data, "Illegal value for xsd:double or xsd:decimal literal.");
+      }
+
     /* boolean */
-    } else if(raptor_uri_equals(datatype, turtle_writer->xsd_boolean_uri)) {
+    } else if(raptor_uri_equals_v2(turtle_writer->world, datatype, turtle_writer->xsd_boolean_uri)) {
       if(!strcmp((const char*)s, "0") || !strcmp((const char*)s, "false")) {
         raptor_iostream_write_string(turtle_writer->iostr, "false");
         written = 1;
@@ -649,6 +663,7 @@ raptor_turtle_writer_comment(raptor_turtle_writer* turtle_writer,
 
 /**
  * raptor_turtle_writer_features_enumerate:
+ * @world: raptor_world object
  * @feature: feature enumeration (0+)
  * @name: pointer to store feature short name (or NULL)
  * @uri: pointer to store feature URI (or NULL)
@@ -657,16 +672,17 @@ raptor_turtle_writer_comment(raptor_turtle_writer* turtle_writer,
  * Get list of turtle_writer features.
  * 
  * If uri is not NULL, a pointer to a new raptor_uri is returned
- * that must be freed by the caller with raptor_free_uri().
+ * that must be freed by the caller with raptor_free_uri_v2().
  *
  * Return value: 0 on success, <0 on failure, >0 if feature is unknown
  **/
 int
-raptor_turtle_writer_features_enumerate(const raptor_feature feature,
+raptor_turtle_writer_features_enumerate(raptor_world* world,
+                                        const raptor_feature feature,
                                         const char **name, 
                                         raptor_uri **uri, const char **label)
 {
-  return raptor_features_enumerate_common(feature, name, uri, label, 8);
+  return raptor_features_enumerate_common(world, feature, name, uri, label, 8);
 }
 
 
@@ -740,6 +756,9 @@ raptor_turtle_writer_set_feature(raptor_turtle_writer *turtle_writer,
     /* JSON serializer features */
     case RAPTOR_FEATURE_JSON_CALLBACK:
     case RAPTOR_FEATURE_JSON_EXTRA_DATA:
+    case RAPTOR_FEATURE_RSS_TRIPLES:
+    case RAPTOR_FEATURE_ATOM_ENTRY_URI:
+    case RAPTOR_FEATURE_PREFIX_ELEMENTS:
     
     /* Turtle serializer feature */
     case RAPTOR_FEATURE_WRITE_BASE_URI:
@@ -851,6 +870,9 @@ raptor_turtle_writer_get_feature(raptor_turtle_writer *turtle_writer,
     /* JSON serializer features */
     case RAPTOR_FEATURE_JSON_CALLBACK:
     case RAPTOR_FEATURE_JSON_EXTRA_DATA:
+    case RAPTOR_FEATURE_RSS_TRIPLES:
+    case RAPTOR_FEATURE_ATOM_ENTRY_URI:
+    case RAPTOR_FEATURE_PREFIX_ELEMENTS:
     
     /* Turtle serializer feature */
     case RAPTOR_FEATURE_WRITE_BASE_URI:
@@ -904,9 +926,8 @@ const unsigned char* longstr=(const unsigned char*)"it's quoted\nand has newline
 int
 main(int argc, char *argv[]) 
 {
+  raptor_world *world;
   const char *program=raptor_basename(argv[0]);
-  const raptor_uri_handler *uri_handler;
-  void *uri_context;
   raptor_iostream *iostr;
   raptor_namespace_stack *nstack;
   raptor_namespace* ex_ns;
@@ -920,7 +941,9 @@ main(int argc, char *argv[])
   void *string=NULL;
   size_t string_len=0;
 
-  raptor_init();
+  world = raptor_new_world();
+  if(!world || raptor_world_open(world))
+    exit(1);
   
   iostr=raptor_new_iostream_to_string(&string, &string_len, NULL);
   if(!iostr) {
@@ -928,17 +951,15 @@ main(int argc, char *argv[])
     exit(1);
   }
 
-  raptor_uri_get_handler(&uri_handler, &uri_context);
+  nstack=raptor_new_namespaces_v2(world,
+                                  NULL, NULL, /* errors */
+                                  1);
 
-  nstack=raptor_new_namespaces(uri_handler, uri_context,
-                               NULL, NULL, /* errors */
-                               1);
+  base_uri=raptor_new_uri_v2(world, base_uri_string);
 
-  base_uri=raptor_new_uri(base_uri_string);
-
-  turtle_writer=raptor_new_turtle_writer(base_uri, 1,
+  turtle_writer=raptor_new_turtle_writer(world,
+                                         base_uri, 1,
                                          nstack,
-                                         uri_handler, uri_context,
                                          iostr,
                                          NULL, NULL /* errors */
                                          );
@@ -971,20 +992,21 @@ main(int argc, char *argv[])
                                    (const unsigned char*)" ;", 2);
   raptor_turtle_writer_newline(turtle_writer);
 
-  el_name=raptor_new_qname_from_namespace_local_name(ex_ns,
-                                                     (const unsigned char*)"bar", 
-                                                     NULL);
+  el_name=raptor_new_qname_from_namespace_local_name_v2(world,
+                                                        ex_ns,
+                                                        (const unsigned char*)"bar", 
+                                                        NULL);
 
   raptor_turtle_writer_qname(turtle_writer, el_name);
   raptor_free_qname(el_name);
 
   raptor_turtle_writer_raw_counted(turtle_writer, (const unsigned char*)" ", 1);
 
-  datatype=raptor_new_uri((const unsigned char*)"http://www.w3.org/2001/XMLSchema#decimal");
+  datatype=raptor_new_uri_v2(world, (const unsigned char*)"http://www.w3.org/2001/XMLSchema#decimal");
   raptor_turtle_writer_literal(turtle_writer, nstack,
                                (const unsigned char*)"10.0", NULL,
                                datatype);
-  raptor_free_uri(datatype);
+  raptor_free_uri_v2(world, datatype);
 
   raptor_turtle_writer_newline(turtle_writer);
 
@@ -1000,7 +1022,7 @@ main(int argc, char *argv[])
 
   raptor_free_namespaces(nstack);
 
-  raptor_free_uri(base_uri);
+  raptor_free_uri_v2(world, base_uri);
 
   
   count=raptor_iostream_tell(iostr);
@@ -1037,9 +1059,8 @@ main(int argc, char *argv[])
 #endif
 
   raptor_free_memory(string);
-  
 
-  raptor_finish();
+  raptor_free_world(world);
 
   /* keep gcc -Wall happy */
   return(0);
