@@ -2,7 +2,7 @@
  *
  * raptor_www_libxml.c - Raptor WWW retrieval via libxml2
  *
- * Copyright (C) 2003-2008, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2003-2008, David Beckett http://www.dajobe.org/
  * Copyright (C) 2003-2004, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -117,7 +117,7 @@ raptor_www_libxml_fetch(raptor_www *www)
     *p='\0';
   }
 
-  www->ctxt=xmlNanoHTTPMethod((const char*)raptor_uri_as_string(www->uri),
+  www->ctxt=xmlNanoHTTPMethod((const char*)raptor_uri_as_string_v2(www->world, www->uri),
                               NULL, /* HTTP method (default GET) */
                               NULL, /* input string */
                               &www->type,
