@@ -2,7 +2,7 @@
  *
  * raptor_xml_writer.c - Raptor XML Writer for SAX2 events API
  *
- * Copyright (C) 2003-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2003-2008, David Beckett http://www.dajobe.org/
  * Copyright (C) 2003-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -81,7 +81,7 @@ struct raptor_xml_writer_s {
   raptor_namespace_stack *nstack;
   int nstack_depth;
 
-  raptor_uri_handler *uri_handler;
+  const raptor_uri_handler *uri_handler;
   void *uri_context;
 
   raptor_simple_message_handler error_handler;
@@ -106,26 +106,40 @@ struct raptor_xml_writer_s {
 
   /* Has writing the XML declaration writing been checked? */
   int xml_declaration_checked;
+
+  /* An extra newline is wanted */
+  int pending_newline;
 };
 
 
-#define SPACES_BUFFER_SIZE 16
-static unsigned char spaces_buffer[SPACES_BUFFER_SIZE];
-static int spaces_inited = 0;
+/* 16 spaces */
+#define SPACES_BUFFER_SIZE sizeof(spaces_buffer)
+static const unsigned char spaces_buffer[] = {
+  ' ', ' ', ' ', ' ',
+  ' ', ' ', ' ', ' ',
+  ' ', ' ', ' ', ' ',
+  ' ', ' ', ' ', ' '
+};
+
 
 
 /* helper functions */
+
+/* Handle printing a pending newline OR newline with indenting */
 static int
 raptor_xml_writer_indent(raptor_xml_writer *xml_writer)
 {
   int num_spaces;
-  
-  if (!spaces_inited) {
-    int i;
-    for (i = 0; i < SPACES_BUFFER_SIZE; i++)
-      spaces_buffer[i] = ' ';
-    
-    spaces_inited = 1;
+
+  if(!XML_WRITER_AUTO_INDENT(xml_writer)) {
+    if(xml_writer->pending_newline) {
+      raptor_iostream_write_byte(xml_writer->iostr, '\n');
+      xml_writer->pending_newline=0;
+
+      if(xml_writer->current_element)
+        xml_writer->current_element->content_cdata_seen=1;
+    }
+    return 0;
   }
   
   num_spaces = xml_writer->depth * xml_writer->indent;
@@ -136,12 +150,14 @@ raptor_xml_writer_indent(raptor_xml_writer *xml_writer)
    */
   if(xml_writer->xml_declaration_checked == 1)
     xml_writer->xml_declaration_checked++;
-  else
+  else {
     raptor_iostream_write_byte(xml_writer->iostr, '\n');
+    xml_writer->pending_newline=0;
+  }
   
   while (num_spaces > 0) {
 
-    int count = (num_spaces > SPACES_BUFFER_SIZE) ? SPACES_BUFFER_SIZE : num_spaces;
+    int count = (num_spaces > (int)SPACES_BUFFER_SIZE) ? (int)SPACES_BUFFER_SIZE : num_spaces;
 
     raptor_iostream_write_counted_string(xml_writer->iostr, spaces_buffer, count);
 
@@ -178,15 +194,17 @@ raptor_xml_writer_nsd_compare(const void *a, const void *b)
 
 
 static int
-raptor_iostream_write_xml_element_start(raptor_iostream* iostr,
-                                        raptor_xml_element *element,
-                                        raptor_namespace_stack *nstack,
-                                        raptor_simple_message_handler error_handler,
-                                        void *error_data,
-                                        int auto_empty,
-                                        int depth,
-                                        int xml_version)
+raptor_xml_writer_start_element_common(raptor_xml_writer* xml_writer,
+                                       raptor_xml_element* element,
+                                       int auto_empty)
 {
+  raptor_iostream* iostr=xml_writer->iostr;
+  raptor_namespace_stack *nstack=xml_writer->nstack;
+  raptor_simple_message_handler error_handler=xml_writer->error_handler;
+  void *error_data=xml_writer->error_data;
+  int depth=xml_writer->depth;
+  int auto_indent=XML_WRITER_AUTO_INDENT(xml_writer);
+  int xml_version=xml_writer->xml_version;
   struct nsd *nspace_declarations=NULL;
   size_t nspace_declarations_count=0;  
   unsigned int i;
@@ -198,6 +216,8 @@ raptor_iostream_write_xml_element_start(raptor_iostream* iostr,
       nspace_max_count += raptor_sequence_size(element->declared_nspaces);
     
     nspace_declarations=(struct nsd*)RAPTOR_CALLOC(nsdarray, nspace_max_count, sizeof(struct nsd));
+    if(!nspace_declarations)
+      return 1;
   }
 
   if(element->name->nspace) {
@@ -205,6 +225,8 @@ raptor_iostream_write_xml_element_start(raptor_iostream* iostr,
       nspace_declarations[0].declaration=
         raptor_namespaces_format(element->name->nspace,
                                  &nspace_declarations[0].length);
+      if(!nspace_declarations[0].declaration)
+        goto error;
       nspace_declarations[0].nspace=element->name->nspace;
       nspace_declarations_count++;
     }
@@ -231,6 +253,8 @@ raptor_iostream_write_xml_element_start(raptor_iostream* iostr,
             nspace_declarations[nspace_declarations_count].declaration=
               raptor_namespaces_format(element->attributes[i]->nspace,
                                        &nspace_declarations[nspace_declarations_count].length);
+            if(!nspace_declarations[nspace_declarations_count].declaration)
+              goto error;
             nspace_declarations[nspace_declarations_count].nspace=element->attributes[i]->nspace;
             nspace_declarations_count++;
           }
@@ -257,6 +281,8 @@ raptor_iostream_write_xml_element_start(raptor_iostream* iostr,
         nspace_declarations[nspace_declarations_count].declaration=
           raptor_namespaces_format(nspace,
                                    &nspace_declarations[nspace_declarations_count].length);
+        if(!nspace_declarations[nspace_declarations_count].declaration)
+          goto error;
         nspace_declarations[nspace_declarations_count].nspace=nspace;
         nspace_declarations_count++;
       }
@@ -284,6 +310,13 @@ raptor_iostream_write_xml_element_start(raptor_iostream* iostr,
           raptor_xml_writer_nsd_compare);
     /* add them */
     for (i=0; i < nspace_declarations_count; i++) {
+      if(auto_indent && nspace_declarations_count > 1) {
+        /* indent xmlns namespace attributes */
+        raptor_xml_writer_newline(xml_writer);
+        xml_writer->depth++;
+        raptor_xml_writer_indent(xml_writer);
+        xml_writer->depth--;
+      }
       raptor_iostream_write_byte(iostr, ' ');
       raptor_iostream_write_counted_string(iostr, 
                                            (const char*)nspace_declarations[i].declaration,
@@ -291,9 +324,10 @@ raptor_iostream_write_xml_element_start(raptor_iostream* iostr,
       RAPTOR_FREE(cstring, nspace_declarations[i].declaration);
       nspace_declarations[i].declaration=NULL;
 
-      raptor_namespace_copy(nstack,
-                            (raptor_namespace*)nspace_declarations[i].nspace,
-                            depth);
+      if(raptor_namespace_copy(nstack,
+                               (raptor_namespace*)nspace_declarations[i].nspace,
+                               depth))
+        goto error;
     }
   }
 
@@ -333,14 +367,29 @@ raptor_iostream_write_xml_element_start(raptor_iostream* iostr,
     RAPTOR_FREE(stringarray, nspace_declarations);
 
   return 0;
+
+  /* Clean up nspace_declarations on error */
+  error:
+
+  for (i=0; i < nspace_declarations_count; i++) {
+    if(nspace_declarations[i].declaration)
+      RAPTOR_FREE(cstring, nspace_declarations[i].declaration);
+  }
+
+  if(nspace_declarations)
+    RAPTOR_FREE(stringarray, nspace_declarations);
+
+  return 1;
 }
 
 
 static int
-raptor_iostream_write_xml_element_end(raptor_iostream* iostr,
-                                      raptor_xml_element *element,
-                                      int is_empty)
+raptor_xml_writer_end_element_common(raptor_xml_writer* xml_writer,
+                                     raptor_xml_element *element,
+                                     int is_empty)
 {
+  raptor_iostream* iostr=xml_writer->iostr;
+
   if (is_empty)
     raptor_iostream_write_byte(iostr, '/');
   else {
@@ -367,11 +416,51 @@ raptor_iostream_write_xml_element_end(raptor_iostream* iostr,
 }
 
 
+#ifndef RAPTOR_DISABLE_V1
 /**
  * raptor_new_xml_writer:
  * @nstack: Namespace stack for the writer to start with (or NULL)
- * @uri_handler: URI handler function
- * @uri_context: URI handler context data
+ * @uri_handler: URI handler function (ignored)
+ * @uri_context: URI handler context data (ignored)
+ * @iostr: I/O stream to write to
+ * @error_handler: error handler function
+ * @error_data: error handler data
+ * @canonicalize: unused
+ * 
+ * Constructor - Create a new XML Writer writing XML to a raptor_iostream
+ *
+ * @uri_handler and @uri_context parameters are ignored but are retained
+ * in the API for backwards compatibility. Internally the same uri handler
+ * as returned by raptor_uri_get_handler() will be used.
+ *
+ * raptor_init() MUST have been called before calling this function.
+ * Use raptor_new_xml_writer_v2() if using raptor_world APIs.
+ * 
+ * Return value: a new #raptor_xml_writer object or NULL on failure
+ **/
+raptor_xml_writer*
+raptor_new_xml_writer(raptor_namespace_stack *nstack,
+                      const raptor_uri_handler *uri_handler,
+                      void *uri_context,
+                      raptor_iostream* iostr,
+                      raptor_simple_message_handler error_handler,
+                      void *error_data,
+                      int canonicalize)
+{
+  return raptor_new_xml_writer_v2(raptor_world_instance(),
+                                  nstack,
+                                  iostr,
+                                  error_handler,
+                                  error_data,
+                                  canonicalize);
+}
+#endif
+
+
+/**
+ * raptor_new_xml_writer_v2:
+ * @world: raptor_world object
+ * @nstack: Namespace stack for the writer to start with (or NULL)
  * @iostr: I/O stream to write to
  * @error_handler: error handler function
  * @error_data: error handler data
@@ -382,13 +471,12 @@ raptor_iostream_write_xml_element_end(raptor_iostream* iostr,
  * Return value: a new #raptor_xml_writer object or NULL on failure
  **/
 raptor_xml_writer*
-raptor_new_xml_writer(raptor_namespace_stack *nstack,
-                      raptor_uri_handler *uri_handler,
-                      void *uri_context,
-                      raptor_iostream* iostr,
-                      raptor_simple_message_handler error_handler,
-                      void *error_data,
-                      int canonicalize)
+raptor_new_xml_writer_v2(raptor_world* world,
+                         raptor_namespace_stack *nstack,
+                         raptor_iostream* iostr,
+                         raptor_simple_message_handler error_handler,
+                         void *error_data,
+                         int canonicalize)
 {
   raptor_xml_writer* xml_writer;
   
@@ -398,17 +486,14 @@ raptor_new_xml_writer(raptor_namespace_stack *nstack,
 
   xml_writer->nstack_depth=0;
 
-  xml_writer->uri_handler=uri_handler;
-  xml_writer->uri_context=uri_context;
-
   xml_writer->error_handler=error_handler;
   xml_writer->error_data=error_data;
 
   xml_writer->nstack=nstack;
   if(!xml_writer->nstack) {
-    xml_writer->nstack=nstack=raptor_new_namespaces(uri_handler, uri_context,
-                                                    error_handler, error_data,
-                                                    1);
+    xml_writer->nstack=nstack=raptor_new_namespaces_v2(world,
+                                                       error_handler, error_data,
+                                                       1);
     xml_writer->my_nstack=1;
   }
 
@@ -436,6 +521,8 @@ raptor_new_xml_writer(raptor_namespace_stack *nstack,
 void
 raptor_free_xml_writer(raptor_xml_writer* xml_writer)
 {
+  RAPTOR_ASSERT_OBJECT_POINTER_RETURN(xml_writer, raptor_xml_writer);
+
   if(xml_writer->nstack && xml_writer->my_nstack)
     raptor_free_namespaces(xml_writer->nstack);
 
@@ -484,16 +571,12 @@ raptor_xml_writer_empty_element(raptor_xml_writer* xml_writer,
 
   XML_WRITER_FLUSH_CLOSE_BRACKET(xml_writer);
   
-  raptor_iostream_write_xml_element_start(xml_writer->iostr,
-                                          element, 
-                                          xml_writer->nstack,
-                                          xml_writer->error_handler,
-                                          xml_writer->error_data,
-                                          1,
-                                          xml_writer->depth,
-                                          xml_writer->xml_version);
+  if(xml_writer->pending_newline || XML_WRITER_AUTO_INDENT(xml_writer))
+    raptor_xml_writer_indent(xml_writer);
+  
+  raptor_xml_writer_start_element_common(xml_writer, element, 1);
 
-  raptor_iostream_write_xml_element_end(xml_writer->iostr, element, 1);
+  raptor_xml_writer_end_element_common(xml_writer, element, 1);
   
   raptor_namespaces_end_for_depth(xml_writer->nstack, xml_writer->depth);
 }
@@ -519,17 +602,11 @@ raptor_xml_writer_start_element(raptor_xml_writer* xml_writer,
 
   XML_WRITER_FLUSH_CLOSE_BRACKET(xml_writer);
   
-  if(XML_WRITER_AUTO_INDENT(xml_writer))
+  if(xml_writer->pending_newline || XML_WRITER_AUTO_INDENT(xml_writer))
     raptor_xml_writer_indent(xml_writer);
   
-  raptor_iostream_write_xml_element_start(xml_writer->iostr,
-                                          element, 
-                                          xml_writer->nstack,
-                                          xml_writer->error_handler,
-                                          xml_writer->error_data,
-                                          XML_WRITER_AUTO_EMPTY(xml_writer),
-                                          xml_writer->depth,
-                                          xml_writer->xml_version);
+  raptor_xml_writer_start_element_common(xml_writer, element,
+                                         XML_WRITER_AUTO_EMPTY(xml_writer));
 
   xml_writer->depth++;
 
@@ -540,7 +617,7 @@ raptor_xml_writer_start_element(raptor_xml_writer* xml_writer,
    * test to fail. Subsequent calls to this function set
    * element->parent to its existing value. 
    */
-  if (xml_writer->current_element)
+  if(xml_writer->current_element)
     element->parent = xml_writer->current_element;
   
   xml_writer->current_element=element;
@@ -566,13 +643,14 @@ raptor_xml_writer_end_element(raptor_xml_writer* xml_writer,
 
   xml_writer->depth--;
   
-  if (XML_WRITER_AUTO_INDENT(xml_writer) && element->content_element_seen)
+  if(xml_writer->pending_newline ||
+     (XML_WRITER_AUTO_INDENT(xml_writer) && element->content_element_seen))
     raptor_xml_writer_indent(xml_writer);
 
   is_empty = XML_WRITER_AUTO_EMPTY(xml_writer) ?
     !(element->content_cdata_seen || element->content_element_seen) : 0;
   
-  raptor_iostream_write_xml_element_end(xml_writer->iostr, element, is_empty);
+  raptor_xml_writer_end_element_common(xml_writer, element, is_empty);
   
   raptor_namespaces_end_for_depth(xml_writer->nstack, xml_writer->depth);
 
@@ -581,6 +659,21 @@ raptor_xml_writer_end_element(raptor_xml_writer* xml_writer,
 }
 
 
+/**
+ * raptor_xml_writer_newline:
+ * @xml_writer: XML writer object
+ *
+ * Write a newline to the XML writer.
+ *
+ * Indents the next line if XML writer feature AUTO_INDENT is enabled.
+ **/
+void
+raptor_xml_writer_newline(raptor_xml_writer* xml_writer)
+{
+  xml_writer->pending_newline=1;
+}
+
+
 /**
  * raptor_xml_writer_cdata:
  * @xml_writer: XML writer object
@@ -744,6 +837,7 @@ raptor_xml_writer_comment_counted(raptor_xml_writer* xml_writer,
 }
 
 
+#ifndef RAPTOR_DISABLE_V1
 /**
  * raptor_xml_writer_features_enumerate:
  * @feature: feature enumeration (0+)
@@ -756,6 +850,9 @@ raptor_xml_writer_comment_counted(raptor_xml_writer* xml_writer,
  * If uri is not NULL, a pointer to a new raptor_uri is returned
  * that must be freed by the caller with raptor_free_uri().
  *
+ * raptor_init() MUST have been called before calling this function.
+ * Use raptor_xml_writer_features_enumerate_v2() if using raptor_world APIs.
+ *
  * Return value: 0 on success, <0 on failure, >0 if feature is unknown
  **/
 int
@@ -763,7 +860,51 @@ raptor_xml_writer_features_enumerate(const raptor_feature feature,
                                      const char **name, 
                                      raptor_uri **uri, const char **label)
 {
-  return raptor_features_enumerate_common(feature, name, uri, label, 8);
+  return raptor_xml_writer_features_enumerate_v2(raptor_world_instance(),
+                                                 feature, name, uri, label);
+}
+#endif
+
+
+/**
+ * raptor_xml_writer_features_enumerate_v2:
+ * @world: raptor_world object
+ * @feature: feature enumeration (0+)
+ * @name: pointer to store feature short name (or NULL)
+ * @uri: pointer to store feature URI (or NULL)
+ * @label: pointer to feature label (or NULL)
+ *
+ * Get list of xml_writer features.
+ * 
+ * If uri is not NULL, a pointer to a new raptor_uri is returned
+ * that must be freed by the caller with raptor_free_uri_v2().
+ *
+ * Return value: 0 on success, <0 on failure, >0 if feature is unknown
+ **/
+int
+raptor_xml_writer_features_enumerate_v2(raptor_world* world, 
+                                        const raptor_feature feature,
+                                        const char **name, 
+                                        raptor_uri **uri, const char **label)
+{
+  return raptor_features_enumerate_common(world, feature, name, uri, label, 8);
+}
+
+
+/**
+ * raptor_xml_writer_flush:
+ * @xml_writer: XML writer object
+ *
+ * Finish the XML writer.
+ *
+ **/
+void
+raptor_xml_writer_flush(raptor_xml_writer* xml_writer)
+{
+  if(xml_writer->pending_newline) {
+    raptor_iostream_write_byte(xml_writer->iostr, '\n');
+    xml_writer->pending_newline=0;
+  }
 }
 
 
@@ -825,6 +966,10 @@ raptor_xml_writer_set_feature(raptor_xml_writer *xml_writer,
     case RAPTOR_FEATURE_NON_NFC_FATAL:
     case RAPTOR_FEATURE_WARN_OTHER_PARSETYPES:
     case RAPTOR_FEATURE_CHECK_RDF_ID:
+    case RAPTOR_FEATURE_HTML_TAG_SOUP:
+    case RAPTOR_FEATURE_MICROFORMATS:
+    case RAPTOR_FEATURE_HTML_LINK:
+    case RAPTOR_FEATURE_WWW_TIMEOUT:
 
     /* Shared */
     case RAPTOR_FEATURE_NO_NET:
@@ -841,6 +986,20 @@ raptor_xml_writer_set_feature(raptor_xml_writer *xml_writer,
     case RAPTOR_FEATURE_LITERAL_FILL:
     case RAPTOR_FEATURE_BNODE_FILL:
 
+    /* JSON serializer features */
+    case RAPTOR_FEATURE_JSON_CALLBACK:
+    case RAPTOR_FEATURE_JSON_EXTRA_DATA:
+    case RAPTOR_FEATURE_RSS_TRIPLES:
+    case RAPTOR_FEATURE_ATOM_ENTRY_URI:
+    case RAPTOR_FEATURE_PREFIX_ELEMENTS:
+    
+    /* Turtle serializer feature */
+    case RAPTOR_FEATURE_WRITE_BASE_URI:
+
+    /* WWW feature */
+    case RAPTOR_FEATURE_WWW_HTTP_CACHE_CONTROL:
+    case RAPTOR_FEATURE_WWW_HTTP_USER_AGENT:
+      
     default:
       return -1;
       break;
@@ -879,7 +1038,7 @@ raptor_xml_writer_set_feature_string(raptor_xml_writer *xml_writer,
 
 /**
  * raptor_xml_writer_get_feature:
- * @xml_writer: #raptor_xml_writer serializer object
+ * @xml_writer: #raptor_xml_writer xml writer object
  * @feature: feature to get value
  *
  * Get various xml_writer features.
@@ -928,6 +1087,10 @@ raptor_xml_writer_get_feature(raptor_xml_writer *xml_writer,
     case RAPTOR_FEATURE_NON_NFC_FATAL:
     case RAPTOR_FEATURE_WARN_OTHER_PARSETYPES:
     case RAPTOR_FEATURE_CHECK_RDF_ID:
+    case RAPTOR_FEATURE_HTML_TAG_SOUP:
+    case RAPTOR_FEATURE_MICROFORMATS:
+    case RAPTOR_FEATURE_HTML_LINK:
+    case RAPTOR_FEATURE_WWW_TIMEOUT:
 
     /* Shared */
     case RAPTOR_FEATURE_NO_NET:
@@ -944,6 +1107,20 @@ raptor_xml_writer_get_feature(raptor_xml_writer *xml_writer,
     case RAPTOR_FEATURE_LITERAL_FILL:
     case RAPTOR_FEATURE_BNODE_FILL:
 
+    /* JSON serializer features */
+    case RAPTOR_FEATURE_JSON_CALLBACK:
+    case RAPTOR_FEATURE_JSON_EXTRA_DATA:
+    case RAPTOR_FEATURE_RSS_TRIPLES:
+    case RAPTOR_FEATURE_ATOM_ENTRY_URI:
+    case RAPTOR_FEATURE_PREFIX_ELEMENTS:
+    
+    /* Turtle serializer feature */
+    case RAPTOR_FEATURE_WRITE_BASE_URI:
+
+    /* WWW feature */
+    case RAPTOR_FEATURE_WWW_HTTP_CACHE_CONTROL:
+    case RAPTOR_FEATURE_WWW_HTTP_USER_AGENT:
+      
     default:
       break;
   }
@@ -954,7 +1131,7 @@ raptor_xml_writer_get_feature(raptor_xml_writer *xml_writer,
 
 /**
  * raptor_xml_writer_get_feature_string:
- * @xml_writer: #raptor_xml_writer serializer object
+ * @xml_writer: #raptor_xml_writer xml writer object
  * @feature: feature to get value
  *
  * Get xml_writer features with string values.
@@ -970,6 +1147,22 @@ raptor_xml_writer_get_feature_string(raptor_xml_writer *xml_writer,
   return NULL;
 }
 
+
+/**
+ * raptor_xml_writer_get_depth:
+ * @xml_writer: #raptor_xml_writer xml writer object
+ *
+ * Get the current XML Writer element depth
+ *
+ * Return value: element stack depth
+ */
+int
+raptor_xml_writer_get_depth(raptor_xml_writer *xml_writer)
+{
+  return xml_writer->depth;
+}
+
+
 #endif
 
 
@@ -987,9 +1180,8 @@ const unsigned char *base_uri_string=(const unsigned char*)"http://example.org/b
 int
 main(int argc, char *argv[]) 
 {
+  raptor_world *world;
   const char *program=raptor_basename(argv[0]);
-  raptor_uri_handler *uri_handler;
-  void *uri_context;
   raptor_iostream *iostr;
   raptor_namespace_stack *nstack;
   raptor_namespace* foo_ns;
@@ -997,7 +1189,7 @@ main(int argc, char *argv[])
   raptor_uri* base_uri;
   raptor_qname* el_name;
   raptor_xml_element *element;
-  size_t count;
+  unsigned long offset;
   raptor_qname **attrs;
   raptor_uri* base_uri_copy=NULL;
 
@@ -1005,7 +1197,9 @@ main(int argc, char *argv[])
   void *string=NULL;
   size_t string_len=0;
 
-  raptor_init();
+  world = raptor_new_world();
+  if(!world || raptor_world_open(world))
+    exit(1);
   
   iostr=raptor_new_iostream_to_string(&string, &string_len, NULL);
   if(!iostr) {
@@ -1013,23 +1207,21 @@ main(int argc, char *argv[])
     exit(1);
   }
 
-  raptor_uri_get_handler(&uri_handler, &uri_context);
+  nstack=raptor_new_namespaces_v2(world,
+                                  NULL, NULL, /* errors */
+                                  1);
 
-  nstack=raptor_new_namespaces(uri_handler, uri_context,
-                               NULL, NULL, /* errors */
-                               1);
-
-  xml_writer=raptor_new_xml_writer(nstack,
-                                   uri_handler, uri_context,
-                                   iostr,
-                                   NULL, NULL, /* errors */
-                                   1);
+  xml_writer=raptor_new_xml_writer_v2(world,
+                                      nstack,
+                                      iostr,
+                                      NULL, NULL, /* errors */
+                                      1);
   if(!xml_writer) {
     fprintf(stderr, "%s: Failed to create xml_writer to iostream\n", program);
     exit(1);
   }
 
-  base_uri=raptor_new_uri(base_uri_string);
+  base_uri=raptor_new_uri_v2(world, base_uri_string);
 
   foo_ns=raptor_new_namespace(nstack,
                               (const unsigned char*)"foo",
@@ -1037,10 +1229,11 @@ main(int argc, char *argv[])
                               0);
 
 
-  el_name=raptor_new_qname_from_namespace_local_name(foo_ns,
-                                                     (const unsigned char*)"bar", 
-                                                     NULL);
-  base_uri_copy=base_uri ? raptor_uri_copy(base_uri) : NULL;
+  el_name=raptor_new_qname_from_namespace_local_name_v2(world,
+                                                        foo_ns,
+                                                        (const unsigned char*)"bar", 
+                                                        NULL);
+  base_uri_copy=base_uri ? raptor_uri_copy_v2(world, base_uri) : NULL;
   element=raptor_new_xml_element(el_name,
                                   NULL, /* language */
                                   base_uri_copy);
@@ -1059,7 +1252,7 @@ main(int argc, char *argv[])
                            (const unsigned char*)"blah", 
                            NULL, /* no attribute value - element */
                            NULL, NULL); /* errors */
-  base_uri_copy=base_uri ? raptor_uri_copy(base_uri) : NULL;
+  base_uri_copy=base_uri ? raptor_uri_copy_v2(world, base_uri) : NULL;
   element=raptor_new_xml_element(el_name,
                                   NULL, /* language */
                                   base_uri_copy);
@@ -1083,19 +1276,19 @@ main(int argc, char *argv[])
 
   raptor_free_namespaces(nstack);
 
-  raptor_free_uri(base_uri);
+  raptor_free_uri_v2(world, base_uri);
 
   
-  count=raptor_iostream_get_bytes_written_count(iostr);
+  offset=raptor_iostream_tell(iostr);
 
-#ifdef RAPTOR_DEBUG
+#if RAPTOR_DEBUG > 1
   fprintf(stderr, "%s: Freeing iostream\n", program);
 #endif
   raptor_free_iostream(iostr);
 
-  if(count != OUT_BYTES_COUNT) {
+  if(offset != OUT_BYTES_COUNT) {
     fprintf(stderr, "%s: I/O stream wrote %d bytes, expected %d\n", program,
-            (int)count, (int)OUT_BYTES_COUNT);
+            (int)offset, (int)OUT_BYTES_COUNT);
     fputs("[[", stderr);
     (void)fwrite(string, 1, string_len, stderr);
     fputs("]]\n", stderr);
@@ -1107,20 +1300,21 @@ main(int argc, char *argv[])
     return 1;
   }
   string_len=strlen((const char*)string);
-  if(string_len != count) {
-    fprintf(stderr, "%s: I/O stream created a string length %d, expected %d\n", program, (int)string_len, (int)count);
+  if(string_len != offset) {
+    fprintf(stderr, "%s: I/O stream created a string length %d, expected %d\n", program, (int)string_len, (int)offset);
     return 1;
   }
 
+#if RAPTOR_DEBUG > 1
   fprintf(stderr, "%s: Made XML string of %d bytes\n", program, (int)string_len);
   fputs("[[", stderr);
   (void)fwrite(string, 1, string_len, stderr);
   fputs("]]\n", stderr);
+#endif
 
   raptor_free_memory(string);
   
-
-  raptor_finish();
+  raptor_free_world(world);
   
   /* keep gcc -Wall happy */
   return(0);
