@@ -2,7 +2,7 @@
  *
  * turtle_common.c - Raptor Turtle common code
  *
- * Copyright (C) 2003-2007, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2003-2007, David Beckett http://www.dajobe.org/
  * Copyright (C) 2003-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -140,6 +140,9 @@ raptor_stringbuffer_append_turtle_string(raptor_stringbuffer* stringbuffer,
   }
   *d='\0';
 
+  /* calculate output string size */
+  len = d-string;
+  
   /* string gets owned by the stringbuffer after this */
   return raptor_stringbuffer_append_counted_string(stringbuffer, 
                                                    string, len, 0);
