@@ -1,24 +1,23 @@
-/* A Bison parser, made by GNU Bison 2.3.  */
+
+/* A Bison parser, made by GNU Bison 2.4.1.  */
 
 /* Skeleton interface for Bison's Yacc-like parsers in C
-
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
+   
+      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
    Free Software Foundation, Inc.
-
-   This program is free software; you can redistribute it and/or modify
+   
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
-
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+   
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-
+   
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* As a special exception, you may create a larger work that contains
    part or all of the Bison parser skeleton and distribute that work
@@ -29,10 +28,11 @@
    special exception, which will cause the skeleton and the resulting
    Bison output files to be licensed under the GNU General Public
    License without this special exception.
-
+   
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
+
 /* Tokens.  */
 #ifndef YYTOKENTYPE
 # define YYTOKENTYPE
@@ -49,18 +49,22 @@
      RIGHT_SQUARE = 265,
      LEFT_ROUND = 266,
      RIGHT_ROUND = 267,
-     TRUE = 268,
-     FALSE = 269,
-     STRING_LITERAL = 270,
-     URI_LITERAL = 271,
-     BLANK_LITERAL = 272,
-     QNAME_LITERAL = 273,
-     PREFIX = 274,
-     IDENTIFIER = 275,
-     INTEGER_LITERAL = 276,
-     FLOATING_LITERAL = 277,
-     DECIMAL_LITERAL = 278,
-     ERROR_TOKEN = 279
+     LEFT_CURLY = 268,
+     RIGHT_CURLY = 269,
+     COLONMINUS = 270,
+     TRUE_TOKEN = 271,
+     FALSE_TOKEN = 272,
+     STRING_LITERAL = 273,
+     URI_LITERAL = 274,
+     BLANK_LITERAL = 275,
+     QNAME_LITERAL = 276,
+     PREFIX = 277,
+     BASE = 278,
+     IDENTIFIER = 279,
+     INTEGER_LITERAL = 280,
+     FLOATING_LITERAL = 281,
+     DECIMAL_LITERAL = 282,
+     ERROR_TOKEN = 283
    };
 #endif
 /* Tokens.  */
@@ -74,40 +78,49 @@
 #define RIGHT_SQUARE 265
 #define LEFT_ROUND 266
 #define RIGHT_ROUND 267
-#define TRUE 268
-#define FALSE 269
-#define STRING_LITERAL 270
-#define URI_LITERAL 271
-#define BLANK_LITERAL 272
-#define QNAME_LITERAL 273
-#define PREFIX 274
-#define IDENTIFIER 275
-#define INTEGER_LITERAL 276
-#define FLOATING_LITERAL 277
-#define DECIMAL_LITERAL 278
-#define ERROR_TOKEN 279
+#define LEFT_CURLY 268
+#define RIGHT_CURLY 269
+#define COLONMINUS 270
+#define TRUE_TOKEN 271
+#define FALSE_TOKEN 272
+#define STRING_LITERAL 273
+#define URI_LITERAL 274
+#define BLANK_LITERAL 275
+#define QNAME_LITERAL 276
+#define PREFIX 277
+#define BASE 278
+#define IDENTIFIER 279
+#define INTEGER_LITERAL 280
+#define FLOATING_LITERAL 281
+#define DECIMAL_LITERAL 282
+#define ERROR_TOKEN 283
 
 
 
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 122 "./turtle_parser.y"
 {
+
+/* Line 1676 of yacc.c  */
+#line 122 "./turtle_parser.y"
+
   unsigned char *string;
   raptor_identifier *identifier;
   raptor_sequence *sequence;
   raptor_uri *uri;
   int integer; /* 0+ for a xsd:integer datatyped RDF literal */
-  double floating;
-}
-/* Line 1489 of yacc.c.  */
-#line 106 "turtle_parser.tab.h"
-	YYSTYPE;
+
+
+
+/* Line 1676 of yacc.c  */
+#line 118 "turtle_parser.tab.h"
+} YYSTYPE;
+# define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
-# define YYSTYPE_IS_TRIVIAL 1
 #endif
 
 
 
+
