@@ -1,8 +1,8 @@
 /* -*- Mode: c; c-basic-offset: 2 -*-
  *
- * rdfdump.c - Raptor RDF Parser example code 
+ * rapper.c - Raptor RDF Parsing and Serializing utility
  *
- * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2000-2008, David Beckett http://www.dajobe.org/
  * Copyright (C) 2000-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -61,6 +61,7 @@ extern char *optarg;
 #endif
 
 static void print_triples(void *user_data, const raptor_statement *statement);
+static void print_graph(void *user_data, raptor_uri *graph);
 static void print_namespaces(void* user_data, raptor_namespace *nspace);
 static void relay_namespaces(void* user_data, raptor_namespace *nspace);
 int main(int argc, char *argv[]);
@@ -86,6 +87,8 @@ static int reported_guess=0;
 
 static int report_namespace=0;
 
+static int report_graph=0;
+
 static
 void print_triples(void *user_data, const raptor_statement *triple) 
 {
@@ -93,7 +96,7 @@ void print_triples(void *user_data, const raptor_statement *triple)
   triple_count++;
 
   if(guess && !quiet && !reported_guess) {
-     fprintf(stdout, "%s: Guessed parser name '%s'\n", program,
+     fprintf(stderr, "%s: Guessed parser name '%s'\n", program,
              raptor_get_name(rdf_parser));
      reported_guess=1;
   }
@@ -115,6 +118,20 @@ void print_triples(void *user_data, const raptor_statement *triple)
   return;
 }
 
+static
+void print_graph(void *user_data, raptor_uri *graph)
+{
+  raptor_parser *parser = (raptor_parser *)user_data;
+  
+  if(!report_graph)
+    return;
+
+  if(graph)
+    fprintf(stderr, "%s: Named graph: URI %s\n", program,
+            raptor_uri_as_string_v2(raptor_parser_get_world(parser), graph));
+  else
+    fprintf(stderr, "%s: Named graph: default\n", program);
+}
 
 static void
 print_namespaces(void* user_data, raptor_namespace *nspace)
@@ -150,21 +167,24 @@ relay_namespaces(void* user_data, raptor_namespace *nspace)
 #define HELP_TEXT(short, long, description) "  -" short ", --" long "  " description
 #define HELP_TEXT_LONG(long, description) "      --" long "  " description
 #define HELP_ARG(short, long) "--" #long
+#define HELP_ARG_BOTH(short, long) " \"-" short "\" or \"--" long "\""
 #define HELP_PAD "\n                          "
 #else
 #define HELP_TEXT(short, long, description) "  -" short "  " description
 #define HELP_TEXT_LONG(long, description)
 #define HELP_ARG(short, long) "-" #short
+#define HELP_ARG_BOTH(short, long) "\"-" short "\""
 #define HELP_PAD "\n      "
 #endif
 
 
-#define GETOPT_STRING "nsaf:ghrqo:wecm:i:v"
+#define GETOPT_STRING "nsaf:ghrqo:O:wecm:i:I:vt"
 
 #ifdef HAVE_GETOPT_LONG
 #define SHOW_NAMESPACES_FLAG 0x100
+#define SHOW_GRAPHS_FLAG 0x200
 
-static struct option long_options[] =
+static const struct option long_options[] =
 {
   /* name, has_arg, flag, val */
   {"count", 0, 0, 'c'},
@@ -173,13 +193,17 @@ static struct option long_options[] =
   {"guess", 0, 0, 'g'},
   {"help", 0, 0, 'h'},
   {"input", 1, 0, 'i'},
+  {"input-uri", 1, 0, 'I'},
   {"mode", 1, 0, 'm'},
   {"ntriples", 0, 0, 'n'},
   {"output", 1, 0, 'o'},
+  {"output-uri", 1, 0, 'O'},
   {"quiet", 0, 0, 'q'},
   {"replace-newlines", 0, 0, 'r'},
   {"scan", 0, 0, 's'},
+  {"show-graphs", 0, 0, SHOW_GRAPHS_FLAG},
   {"show-namespaces", 0, 0, SHOW_NAMESPACES_FLAG},
+  {"trace", 0, 0, 't'},
   {"ignore-warnings", 0, 0, 'w'},
   {"version", 0, 0, 'v'},
   {NULL, 0, 0, 0}
@@ -193,19 +217,21 @@ static int warning_count=0;
 static int ignore_warnings=0;
 static int ignore_errors=0;
 
-static const char *title_format_string="Raptor RDF syntax parsing and serializing utility %s\n";
+static const char * const title_format_string="Raptor RDF syntax parsing and serializing utility %s\n";
 
 
 static void
 rdfdump_error_handler(void *data, raptor_locator *locator,
                       const char *message)
 {
+  raptor_parser *parser = (raptor_parser *)data;
+  
   if(!ignore_errors) {
     fprintf(stderr, "%s: Error - ", program);
-    raptor_print_locator(stderr, locator);
+    raptor_print_locator_v2(raptor_parser_get_world(parser), stderr, locator);
     fprintf(stderr, " - %s\n", message);
     
-    raptor_parse_abort((raptor_parser*)data);
+    raptor_parse_abort(parser);
   }
 
   error_count++;
@@ -216,12 +242,14 @@ static void
 rdfdump_warning_handler(void *data, raptor_locator *locator,
                         const char *message) 
 {
+  raptor_parser *parser = (raptor_parser *)data;
+
   if(!ignore_warnings) {
     fprintf(stderr, "%s: Warning - ", program);
-    raptor_print_locator(stderr, locator);
+    raptor_print_locator_v2(raptor_parser_get_world(parser), stderr, locator);
     fprintf(stderr, " - %s\n", message);
   }
-  
+
   warning_count++;
 }
 
@@ -243,31 +271,59 @@ rdfdump_free_namespace_decl(void* data) {
 }
 
 
+static int 
+rapper_uri_trace(void *user_data, raptor_uri* uri)
+{
+  fprintf(stderr, "%s: Processing URI %s\n", program,
+          raptor_uri_as_string_v2(raptor_parser_get_world((raptor_parser*)user_data), uri));
+  return 0;
+}
+
+
+typedef struct
+{
+  raptor_feature feature;
+  int i_value;
+  unsigned char* s_value;
+} feature_value;
+
+
+
 int
 main(int argc, char *argv[]) 
 {
+  /* input variables - parser
+   * 'uri_string' is set to a URI otherwise 'filename' is file name
+   * or if NULL, stdin.  Base URI in 'base_uri_string' is required for stdin.
+   */
+  raptor_world* world = NULL;
   raptor_parser* rdf_parser=NULL;
+  char *filename=NULL;
+#define FILENAME_LABEL(name) ((name) ? (name) : "<stdin>")
   unsigned char *uri_string=NULL;
   int free_uri_string=0;
+  raptor_uri *uri;
   unsigned char *base_uri_string=NULL;
-  int rc;
-  int scanning=0;
+  raptor_uri *base_uri=NULL;
   const char *syntax_name="rdfxml";
-  const char *serializer_syntax_name="ntriples";
+  raptor_sequence* parser_features=NULL;
+  int scanning=0;
   int strict_mode=0;
+  int trace=0;
+
+  /* output variables - serializer */
+  /* 'serializer' object variable is a global */
+  const char *serializer_syntax_name="ntriples";
+  unsigned char *output_base_uri_string=NULL;
+  raptor_uri *output_base_uri=NULL;
+  raptor_sequence* serializer_features=NULL;
+  raptor_sequence *namespace_declarations=NULL;
+
+  /* other variables */
+  int rc;
   int usage=0;
   int help=0;
-  raptor_uri *base_uri;
-  raptor_uri *uri;
   char *p;
-  char *filename=NULL;
-  raptor_feature parser_feature=(raptor_feature)-1;
-  int parser_feature_value= -1;
-  unsigned char* parser_feature_string_value=NULL;
-  raptor_feature serializer_feature=(raptor_feature)-1;
-  int serializer_feature_value= -1;
-  unsigned char* serializer_feature_string_value=NULL;
-  raptor_sequence *namespace_declarations=NULL;
 
   program=argv[0];
   if((p=strrchr(program, '/')))
@@ -276,7 +332,12 @@ main(int argc, char *argv[])
     program=p+1;
   argv[0]=program;
 
-  raptor_init();
+  world = raptor_new_world();
+  if(!world)
+    exit(1);
+  rc = raptor_world_open(world);
+  if(rc)
+    exit(1);
 
   while (!usage && !help)
   {
@@ -302,6 +363,8 @@ main(int argc, char *argv[])
 
       case 'c':
         count=1;
+        if(serializer_syntax_name)
+          serializer_syntax_name = NULL;
         break;
 
       case 'f':
@@ -313,9 +376,9 @@ main(int argc, char *argv[])
             for(i=0; i < (int)raptor_get_feature_count(); i++) {
               const char *feature_name;
               const char *feature_label;
-              if(!raptor_features_enumerate((raptor_feature)i, &feature_name, NULL, &feature_label)) {
+              if(!raptor_features_enumerate_v2(world, (raptor_feature)i, &feature_name, NULL, &feature_label)) {
                 const char *feature_type=(raptor_feature_value_type((raptor_feature)i) == 0) ? "" : " (string)";
-                printf("  %-20s  %s%s\n", feature_name, feature_label, 
+                fprintf(stderr, "  %-21s  %s%s\n", feature_name, feature_label, 
                        feature_type);
               }
             }
@@ -323,16 +386,16 @@ main(int argc, char *argv[])
             for(i=0; i < (int)raptor_get_feature_count(); i++) {
               const char *feature_name;
               const char *feature_label;
-              if(!raptor_serializer_features_enumerate((raptor_feature)i, &feature_name, NULL, &feature_label)) {
+              if(!raptor_serializer_features_enumerate_v2(world, (raptor_feature)i, &feature_name, NULL, &feature_label)) {
                 const char *feature_type=(raptor_feature_value_type((raptor_feature)i) == 0) ? "" : " (string)";
-                printf("  %-20s  %s%s\n", feature_name, feature_label, 
+                fprintf(stderr, "  %-21s  %s%s\n", feature_name, feature_label, 
                        feature_type);
               }
             }
             fputs("Features are set with `" HELP_ARG(f, feature) " FEATURE=VALUE or `-f FEATURE'\nand take a decimal integer VALUE except where noted, defaulting to 1 if omitted.\n", stderr);
             fputs("\nA feature of the form xmlns:PREFIX=\"URI\" can be used to declare output\nnamespace prefixes and names for serializing using an XML-style syntax\nEither or both of PREFIX or URI can be omitted such as -f xmlns=\"URI\"\nThis form can be repeated for multiple declarations.\n", stderr);
 
-            raptor_finish();
+            raptor_free_world(world);
             exit(0);
           } else if(!strncmp(optarg, "xmlns", 5)) {
             struct namespace_decl *nd;
@@ -344,7 +407,7 @@ main(int argc, char *argv[])
                       optarg);
               rdfdump_free_namespace_decl(nd);
 
-              raptor_finish();
+              raptor_free_world(world);
               exit(0);
             }
 
@@ -355,27 +418,38 @@ main(int argc, char *argv[])
           } else {
             int i;
             size_t arg_len=strlen(optarg);
+            feature_value* fv;
+            int ok=0;
             
+            /* parser features */
             for(i=0; i < (int)raptor_get_feature_count(); i++) {
               const char *feature_name;
               size_t len;
               
-              if(raptor_features_enumerate((raptor_feature)i, &feature_name, NULL, NULL))
+              if(raptor_features_enumerate_v2(world, (raptor_feature)i, &feature_name, NULL, NULL))
                 continue;
+
               len=strlen(feature_name);
               if(!strncmp(optarg, feature_name, len)) {
-                parser_feature=(raptor_feature)i;
-                if(raptor_feature_value_type(parser_feature) == 0) {
+                fv=(feature_value*)raptor_calloc_memory(sizeof(feature_value), 1);
+
+                fv->feature=(raptor_feature)i;
+                if(raptor_feature_value_type(fv->feature) == 0) {
                   if(len < arg_len && optarg[len] == '=')
-                    parser_feature_value=atoi(&optarg[len+1]);
+                    fv->i_value=atoi(&optarg[len+1]);
                   else if(len == arg_len)
-                    parser_feature_value=1;
+                    fv->i_value=1;
                 } else {
                   if(len < arg_len && optarg[len] == '=')
-                    parser_feature_string_value=(unsigned char*)&optarg[len+1];
+                    fv->s_value=(unsigned char*)&optarg[len+1];
                   else if(len == arg_len)
-                    parser_feature_string_value=(unsigned char*)"";
+                    fv->s_value=(unsigned char*)"";
                 }
+
+                if(!parser_features)
+                  parser_features=raptor_new_sequence(raptor_free_memory, NULL);
+                raptor_sequence_push(parser_features, fv);
+                ok=1;
                 break;
               }
             }
@@ -384,28 +458,35 @@ main(int argc, char *argv[])
               const char *feature_name;
               size_t len;
               
-              if(raptor_serializer_features_enumerate((raptor_feature)i, &feature_name, NULL, NULL))
+              if(raptor_serializer_features_enumerate_v2(world, (raptor_feature)i, &feature_name, NULL, NULL))
                 continue;
+
               len=strlen(feature_name);
               if(!strncmp(optarg, feature_name, len)) {
-                serializer_feature=(raptor_feature)i;
-                if(raptor_feature_value_type(serializer_feature) == 0) {
+                fv=(feature_value*)raptor_calloc_memory(sizeof(feature_value), 1);
+
+                fv->feature=(raptor_feature)i;
+                if(raptor_feature_value_type(fv->feature) == 0) {
                   if(len < arg_len && optarg[len] == '=')
-                    serializer_feature_value=atoi(&optarg[len+1]);
+                    fv->i_value=atoi(&optarg[len+1]);
                   else if(len == arg_len)
-                    serializer_feature_value=1;
+                    fv->i_value=1;
                 } else {
                   if(len < arg_len && optarg[len] == '=')
-                    serializer_feature_string_value=(unsigned char*)&optarg[len+1];
+                    fv->s_value=(unsigned char*)&optarg[len+1];
                   else if(len == arg_len)
-                    serializer_feature_string_value=(unsigned char*)"";
+                    fv->s_value=(unsigned char*)"";
                 }
+
+                if(!serializer_features)
+                  serializer_features=raptor_new_sequence(raptor_free_memory, NULL);
+                raptor_sequence_push(serializer_features, fv);
+                ok=1;
                 break;
               }
             }
             
-            if(parser_feature_value < 0 && !parser_feature_string_value &&
-               serializer_feature_value < 0 && !serializer_feature_string_value) {
+            if(!ok) {
               fprintf(stderr, "%s: invalid argument `%s' for `" HELP_ARG(f, feature) "'\nTry '%s " HELP_ARG(f, feature) " help' for a list of valid features\n",
                       program, optarg, program);
               usage=1;
@@ -430,6 +511,10 @@ main(int argc, char *argv[])
         scanning=1;
         break;
 
+      case 't':
+        trace=1;
+        break;
+
       case 'q':
         quiet=1;
         break;
@@ -455,7 +540,7 @@ main(int argc, char *argv[])
 
       case 'o':
         if(optarg) {
-          if(raptor_serializer_syntax_name_check(optarg))
+          if(raptor_serializer_syntax_name_check_v2(world, optarg))
             serializer_syntax_name=optarg;
           else {
             int i;
@@ -466,9 +551,9 @@ main(int argc, char *argv[])
             for(i=0; 1; i++) {
               const char *help_name;
               const char *help_label;
-              if(raptor_serializers_enumerate(i, &help_name, &help_label, NULL, NULL))
+              if(raptor_serializers_enumerate_v2(world, i, &help_name, &help_label, NULL, NULL))
                 break;
-              printf("  %-12s for %s\n", help_name, help_label);
+              fprintf(stderr, "  %-14s for %s\n", help_name, help_label);
             }
             usage=1;
             break;
@@ -477,9 +562,14 @@ main(int argc, char *argv[])
         }
         break;
 
+      case 'O':
+        if(optarg)
+          output_base_uri_string=(unsigned char*)optarg;
+        break;
+        
       case 'i':
         if(optarg) {
-          if(raptor_syntax_name_check(optarg))
+          if(raptor_syntax_name_check_v2(world, optarg))
             syntax_name=optarg;
           else {
             int i;
@@ -490,9 +580,9 @@ main(int argc, char *argv[])
             for(i=0; 1; i++) {
               const char *help_name;
               const char *help_label;
-              if(raptor_syntaxes_enumerate(i, &help_name, &help_label, NULL, NULL))
+              if(raptor_syntaxes_enumerate_v2(world, i, &help_name, &help_label, NULL, NULL))
                 break;
-              printf("  %-12s for %s\n", help_name, help_label);
+              fprintf(stderr, "  %-14s for %s\n", help_name, help_label);
             }
             usage=1;
             break;
@@ -501,6 +591,11 @@ main(int argc, char *argv[])
         }
         break;
 
+      case 'I':
+        if(optarg)
+          base_uri_string=(unsigned char*)optarg;
+        break;
+        
       case 'w':
         ignore_warnings=1;
         break;
@@ -513,7 +608,7 @@ main(int argc, char *argv[])
         fputs(raptor_version_string, stdout);
         fputc('\n', stdout);
 
-        raptor_finish();
+        raptor_free_world(world);
         exit(0);
 
 #ifdef SHOW_NAMESPACES_FLAG
@@ -522,8 +617,14 @@ main(int argc, char *argv[])
         break;
 #endif
 
-    }
-    
+#ifdef SHOW_GRAPHS_FLAG
+      case SHOW_GRAPHS_FLAG:
+        report_graph=1;
+        break;
+#endif
+
+    } /* end switch */
+
   }
 
   if(optind != argc-1 && optind != argc-2 && !help && !usage) {
@@ -545,7 +646,7 @@ main(int argc, char *argv[])
     fprintf(stderr, "Try `%s " HELP_ARG(h, help) "' for more information.\n",
                     program);
 
-    raptor_finish();
+    raptor_free_world(world);
     exit(1);
   }
 
@@ -553,8 +654,8 @@ main(int argc, char *argv[])
     int i;
     
     printf(title_format_string, raptor_version_string);
-    puts("Parse RDF content at the source URI into RDF triples.");
-    printf("Usage: %s [OPTIONS] <source URI> [base URI]\n\n", program);
+    puts("Parse RDF syntax from a source into serialized RDF triples.");
+    printf("Usage: %s [OPTIONS] INPUT-URI [INPUT-BASE-URI]\n\n", program);
 
     fputs(raptor_copyright_string, stdout);
     fputs("\nLicense: ", stdout);
@@ -562,12 +663,20 @@ main(int argc, char *argv[])
     fputs("Raptor home page: ", stdout);
     puts(raptor_home_url_string);
 
-    puts("\nSyntax options:");
-    puts(HELP_TEXT("i FORMAT", "input FORMAT ", "Set the input format to one of:"));
+    puts("\nArguments:");
+    puts("  INPUT-URI       a filename, URI or '-' for standard input (stdin).");
+    puts("  INPUT-BASE-URI  the input/parser base URI or '-' for none."
+         "\n"
+         "    Default is INPUT-URI"
+         "\n"
+         "    Equivalent to" HELP_ARG_BOTH("I INPUT-BASE-URI", "input-uri INPUT-BASE-URI"));
+
+    puts("\nMain options:");
+    puts(HELP_TEXT("i FORMAT", "input FORMAT ", "Set the input format/parser to one of:"));
     for(i=0; 1; i++) {
       const char *help_name;
       const char *help_label;
-      if(raptor_syntaxes_enumerate(i, &help_name, &help_label, NULL, NULL))
+      if(raptor_syntaxes_enumerate_v2(world, i, &help_name, &help_label, NULL, NULL))
         break;
       printf("    %-14s  %s", help_name, help_label);
       if(!i)
@@ -575,12 +684,14 @@ main(int argc, char *argv[])
       else
         putchar('\n');
     }
+    puts(HELP_TEXT("I URI", "input-uri URI   ", "Set the input/parser base URI. '-' for none.") HELP_PAD "    Default is INPUT-BASE-URI argument value.");
     putchar('\n');
-    puts(HELP_TEXT("o FORMAT", "output FORMAT", "Set the output format to one of:"));
+
+    puts(HELP_TEXT("o FORMAT", "output FORMAT", "Set the output format/serializer to one of:"));
     for(i=0; 1; i++) {
       const char *help_name;
       const char *help_label;
-      if(raptor_serializers_enumerate(i, &help_name, &help_label, NULL, NULL))
+      if(raptor_serializers_enumerate_v2(world, i, &help_name, &help_label, NULL, NULL))
         break;
       printf("    %-14s  %s", help_name, help_label);
       if(!i)
@@ -588,7 +699,10 @@ main(int argc, char *argv[])
       else
         putchar('\n');
     }
-    puts("\nGeneral options:");
+    puts(HELP_TEXT("O URI", "output-uri URI  ", "Set the output/serializer base URI. '-' for none.")  HELP_PAD "    Default is input/parser base URI.");
+    putchar('\n');
+
+    puts("General options:");
     puts(HELP_TEXT("c", "count           ", "Count triples only - do not print them."));
     puts(HELP_TEXT("e", "ignore-errors   ", "Ignore error messages"));
     puts(HELP_TEXT("f FEATURE(=VALUE)", "feature FEATURE(=VALUE)", HELP_PAD "Set parser or serializer features" HELP_PAD "Use `-f help' for a list of valid features"));
@@ -598,14 +712,18 @@ main(int argc, char *argv[])
     puts(HELP_TEXT("q", "quiet           ", "No extra information messages"));
     puts(HELP_TEXT("r", "replace-newlines", "Replace newlines with spaces in literals"));
     puts(HELP_TEXT("s", "scan            ", "Scan for <rdf:RDF> element in source"));
+#ifdef SHOW_GRAPHS_FLAG
+    puts(HELP_TEXT_LONG("show-graphs     ", "Show named graphs as they are declared"));
+#endif
 #ifdef SHOW_NAMESPACES_FLAG
     puts(HELP_TEXT_LONG("show-namespaces ", "Show namespaces as they are declared"));
 #endif
+    puts(HELP_TEXT("t", "trace           ", "Trace URIs retrieved during parsing"));
     puts(HELP_TEXT("w", "ignore-warnings ", "Ignore warning messages"));
     puts(HELP_TEXT("v", "version         ", "Print the Raptor version"));
     puts("\nReport bugs to http://bugs.librdf.org/");
 
-    raptor_finish();
+    raptor_free_world(world);
     exit(0);
   }
 
@@ -637,7 +755,7 @@ main(int argc, char *argv[])
   }
 
   if(uri_string) {
-    uri=raptor_new_uri(uri_string);
+    uri=raptor_new_uri_v2(world, uri_string);
     if(!uri) {
       fprintf(stderr, "%s: Failed to create URI for %s\n",
               program, uri_string);
@@ -647,21 +765,42 @@ main(int argc, char *argv[])
     uri=NULL; /* stdin */
 
 
-  if(!base_uri_string) {
-    base_uri=raptor_uri_copy(uri);
-  } else {
-    base_uri=raptor_new_uri(base_uri_string);
-    if(!base_uri) {
-      fprintf(stderr, "%s: Failed to create URI for %s\n",
-              program, base_uri_string);
-      return(1);
+  /* Set the input/parser base URI */
+  if(base_uri_string) {
+    if(strcmp((const char*)base_uri_string, "-")) {
+      base_uri=raptor_new_uri_v2(world, base_uri_string);
+      if(!base_uri) {
+        fprintf(stderr, "%s: Failed to create URI for %s\n",
+                program, base_uri_string);
+        return(1);
+      }
     }
   }
 
+
+  /* Set the output/serializer base URI from the argument if explicitly
+   * set, otherwise default to the input base URI if present.
+   */
+  if(!output_base_uri_string) {
+    if(base_uri)
+      output_base_uri=raptor_uri_copy_v2(world, base_uri);
+  } else {
+    if(strcmp((const char*)output_base_uri_string, "-")) {
+      output_base_uri=raptor_new_uri_v2(world, (const unsigned char*)output_base_uri_string);
+      if(!output_base_uri) {
+        fprintf(stderr, "%s: Failed to create output base URI for %s\n",
+                program, output_base_uri_string);
+        return(1);
+      }
+    } else
+      output_base_uri=NULL;
+  }
+    
+
   if(guess)
     syntax_name="guess";
 
-  rdf_parser=raptor_new_parser(syntax_name);
+  rdf_parser=raptor_new_parser_v2(world, syntax_name);
   if(!rdf_parser) {
     fprintf(stderr, "%s: Failed to create raptor parser type %s\n", program,
             syntax_name);
@@ -676,39 +815,62 @@ main(int argc, char *argv[])
   if(scanning)
     raptor_set_feature(rdf_parser, RAPTOR_FEATURE_SCANNING, 1);
 
-  if(parser_feature_value >= 0)
-    raptor_set_feature(rdf_parser, 
-                       parser_feature, parser_feature_value);
-  if(parser_feature_string_value)
-    raptor_parser_set_feature_string(rdf_parser, 
-                                     parser_feature,
-                                     parser_feature_string_value);
+  if(parser_features) {
+    feature_value *fv;
+    while((fv=(feature_value*)raptor_sequence_pop(parser_features))) {
+      if(fv->s_value)
+        raptor_parser_set_feature_string(rdf_parser, fv->feature, fv->s_value);
+      else
+        raptor_set_feature(rdf_parser, fv->feature, fv->i_value);
+      raptor_free_memory(fv);
+    }
+    raptor_free_sequence(parser_features);
+    parser_features=NULL;
+  }
+
+  if(trace)
+    raptor_parser_set_uri_filter(rdf_parser, rapper_uri_trace, rdf_parser);
 
 
   if(!quiet) {
-    if (filename) {
+    if(uri_string) {
       if(base_uri_string)
-        fprintf(stdout, "%s: Parsing file %s with base URI %s\n", program,
-                filename, base_uri_string);
+        fprintf(stderr, "%s: Parsing URI %s with parser %s and base URI %s\n",
+                program, uri_string, syntax_name, base_uri_string);
       else
-        fprintf(stdout, "%s: Parsing file %s\n", program, filename);
+        fprintf(stderr, "%s: Parsing URI %s with parser %s\n", program,
+                uri_string, syntax_name);
     } else {
       if(base_uri_string)
-        fprintf(stdout, "%s: Parsing URI %s with base URI %s\n", program,
-                uri_string, base_uri_string);
+        fprintf(stderr, "%s: Parsing file %s with parser %s and base URI %s\n", program,
+                FILENAME_LABEL(filename), syntax_name, base_uri_string);
       else
-        fprintf(stdout, "%s: Parsing URI %s\n", program, uri_string);
+        fprintf(stderr, "%s: Parsing file %s with parser %s\n", program,
+                FILENAME_LABEL(filename), syntax_name);
     }
   }
   
   raptor_set_statement_handler(rdf_parser, rdf_parser, print_triples);
 
+  if(report_graph)
+    raptor_set_graph_handler(rdf_parser, rdf_parser, print_graph);
+
   if(report_namespace)
     raptor_set_namespace_handler(rdf_parser, rdf_parser, print_namespaces);
 
 
   if(serializer_syntax_name) {    
-    serializer=raptor_new_serializer(serializer_syntax_name);
+    if(!quiet) {
+      if(output_base_uri)
+        fprintf(stderr, "%s: Serializing with serializer %s and base URI %s\n",
+                program, serializer_syntax_name,
+                raptor_uri_as_string_v2(world, output_base_uri));
+      else
+        fprintf(stderr, "%s: Serializing with serializer %s\n",
+                program, serializer_syntax_name);
+    }
+
+    serializer=raptor_new_serializer_v2(world, serializer_syntax_name);
     if(!serializer) {
       fprintf(stderr, 
               "%s: Failed to create raptor serializer type %s\n", program,
@@ -722,37 +884,46 @@ main(int argc, char *argv[])
         struct namespace_decl *nd=(struct namespace_decl *)raptor_sequence_get_at(namespace_declarations, i);
         raptor_uri *ns_uri=NULL;
         if(nd->uri_string)
-          ns_uri=raptor_new_uri(nd->uri_string);
+          ns_uri=raptor_new_uri_v2(world, nd->uri_string);
         
         raptor_serialize_set_namespace(serializer, ns_uri, nd->prefix);
         if(ns_uri)
-          raptor_free_uri(ns_uri);
+          raptor_free_uri_v2(world, ns_uri);
       }
       raptor_free_sequence(namespace_declarations);
       namespace_declarations=NULL;
     }
     
-    if(serializer_feature_value >= 0)
-      raptor_serializer_set_feature(serializer, 
-                                    serializer_feature, serializer_feature_value);
-    if(serializer_feature_string_value)
-      raptor_serializer_set_feature_string(serializer, 
-                                           serializer_feature,
-                                           serializer_feature_string_value);
+    if(serializer_features) {
+      feature_value *fv;
+      while((fv=(feature_value*)raptor_sequence_pop(serializer_features))) {
+        if(fv->s_value)
+          raptor_serializer_set_feature_string(serializer, fv->feature,
+                                               fv->s_value);
+        else
+          raptor_serializer_set_feature(serializer, fv->feature, fv->i_value);
+        raptor_free_memory(fv);
+      }
+      raptor_free_sequence(serializer_features);
+      serializer_features=NULL;
+    }
 
-    raptor_serialize_start_to_file_handle(serializer, base_uri, stdout);
+    raptor_serialize_start_to_file_handle(serializer, 
+                                          output_base_uri, stdout);
 
     if(!report_namespace)
       raptor_set_namespace_handler(rdf_parser, serializer, relay_namespaces);
   }
   
 
-  /* PARSE the URI as RDF/XML */
+  /* Begin the parsing of the content from file or URI,
+   * sending it to serializer via callback print_triples()
+   */
   rc=0;
   if(!uri || filename) {
     if(raptor_parse_file(rdf_parser, uri, base_uri)) {
       fprintf(stderr, "%s: Failed to parse file %s %s content\n", program, 
-              filename, syntax_name);
+              FILENAME_LABEL(filename), syntax_name);
       rc=1;
     }
   } else {
@@ -771,20 +942,31 @@ main(int argc, char *argv[])
   }
   
 
-  if(!quiet)
-    fprintf(stdout, "%s: Parsing returned %d triple%s\n", program,
-            triple_count, (triple_count == 1 ? "" : "s"));
-
-  raptor_free_uri(base_uri);
+  if(!quiet) {
+    if(triple_count == 1)
+      fprintf(stderr, "%s: Parsing returned 1 triple\n", program);
+    else
+      fprintf(stderr, "%s: Parsing returned %d triples\n", program,
+              triple_count);
+  }
+  
+  if(output_base_uri)
+    raptor_free_uri_v2(world, output_base_uri);
+  if(base_uri)
+    raptor_free_uri_v2(world, base_uri);
   if(uri)
-    raptor_free_uri(uri);
+    raptor_free_uri_v2(world, uri);
   if(free_uri_string)
     raptor_free_memory(uri_string);
 
   if(namespace_declarations)
     raptor_free_sequence(namespace_declarations);
+  if(parser_features)
+    raptor_free_sequence(parser_features);
+  if(serializer_features)
+    raptor_free_sequence(serializer_features);
 
-  raptor_finish();
+  raptor_free_world(world);
 
   if(error_count && !ignore_errors)
     return 1;
